@@ -295,6 +295,11 @@ exp.section_builder = IPA.section_builder = function(spec) {
 
         var widget = that.widget_builder.build_widget(field_spec, section.widgets);
 
+        if (field_spec.field === false) {
+            // widget doesn't have field, skip
+            return;
+        }
+
         if (section.$field_adapter && !field_spec.adapter) {
             field_spec.adapter = section.$field_adapter;
         }
@@ -482,6 +487,7 @@ exp.details_facet_pre_op = function(spec, context) {
         spec.control_buttons.unshift(
             {
                 name: 'revert',
+                title: '@i18n:buttons.revert_title',
                 label: '@i18n:buttons.revert',
                 icon: 'fa-undo'
             },
@@ -494,6 +500,7 @@ exp.details_facet_pre_op = function(spec, context) {
     spec.control_buttons.unshift(
         {
             name: 'refresh',
+            title: '@i18n:buttons.refresh_title',
             label: '@i18n:buttons.refresh',
             icon: 'fa-refresh'
         });
@@ -1640,6 +1647,7 @@ exp.value_state_evaluator = IPA.value_state_evaluator = function(spec) {
 
     spec.name = spec.name || 'value_state_evaluator';
     spec.event = spec.event || 'post_load';
+    spec.adapter = spec.adapter || {};
 
     var that = IPA.state_evaluator(spec);
 
@@ -1664,22 +1672,25 @@ exp.value_state_evaluator = IPA.value_state_evaluator = function(spec) {
      */
     that.representation = spec.representation;
 
+    that.adapter = builder.build('adapter',
+                    spec.adapter, { context: that });
+
+    that.param = spec.param;
+
     /**
      * @inheritDoc
      */
     that.on_event = function(data) {
 
-        var old_state, record, state, value, loaded_value;
+        var old_state, value, loaded_value;
 
         old_state = that.state;
-        record = data.result.result;
         value = that.normalize_value(that.value);
-        loaded_value = record[that.attribute];
-        loaded_value = that.normalize_value(loaded_value);
-
         that.state = [];
 
-        if (!IPA.array_diff(value, loaded_value)) {
+        loaded_value = that.adapter.load(data, spec.attribute);
+
+        if(!IPA.array_diff(value, loaded_value)) {
             that.state.push(that.get_state_text());
         }
 
@@ -1732,6 +1743,28 @@ exp.value_state_evaluator = IPA.value_state_evaluator = function(spec) {
     return that;
 };
 
+
+/**
+ * has_keytab evaluator
+ *
+ * @class details.has_keytab_evaluator
+ * @alternateClassName IPA.has_keytab_evaluator
+ * @extends facet.value_state_evaluator
+ */
+exp.has_keytab_evaluator = IPA.has_keytab_evaluator = function(spec) {
+
+    spec.name = spec.name || 'has_keytab_evaluator';
+    spec.attribute = spec.attribute || 'has_keytab';
+    spec.value = spec.value || [true];
+    spec.representation = spec.representation || 'has_keytab';
+    spec.param = spec.param || 'has_keytab';
+    spec.adapter = spec.adapter || { $type: 'adapter' };
+
+    var that = IPA.value_state_evaluator(spec);
+
+    return that;
+};
+
 /**
  * Object class evaluator
  *
