@@ -87,11 +87,22 @@ return {
                                     label: '@i18n:objects.dnsconfig.forward_none'
                                 }
                             ]
+                        },
+                        {
+                            name: 'dnssec_key_master_server',
+                            read_only: true
+                        },
+                        {
+                            $type: 'multivalued',
+                            name: 'dns_server_server',
+                            read_only: true
                         }
                     ]
                 }
             ],
-            needs_update: true
+            needs_update: true,
+            actions: [ 'update_dns_records' ],
+            header_actions: [ 'update_dns_records' ]
         }
     ]
 };};
@@ -152,6 +163,7 @@ return {
                     'idnssoaretry',
                     'idnssoaexpire',
                     'idnssoaminimum',
+                    'dnsdefaultttl',
                     'dnsttl',
                     {
                         $type: 'radio',
@@ -492,6 +504,97 @@ return {
 };};
 
 
+var make_dnsserver_spec = function() {
+    var spec = {
+        name: 'dnsserver',
+        enable_test: function() {
+            return IPA.dns_enabled;
+        },
+        facets: [
+            {
+                $type: 'search',
+                no_update: true,
+                columns: [
+                    'idnsserverid'
+                ]
+            },
+            {
+                $type: 'details',
+                disable_facet_tabs: true,
+                fields: [
+                    'idnssoamname',
+                    'idnsserverid',
+                    {
+                        $type: 'multivalued',
+                        name: 'idnsforwarders',
+                        validators: ['dnsforwarder']
+                    },
+                    {
+                        $type: 'radio',
+                        name: 'idnsforwardpolicy',
+                        options: [
+                            {
+                                value: 'first',
+                                label: '@i18n:objects.dnsconfig.forward_first'
+                            },
+                            {
+                                value: 'only',
+                                label: '@i18n:objects.dnsconfig.forward_only'
+                            },
+                            {
+                                value: 'none',
+                                label: '@i18n:objects.dnsconfig.forward_none'
+                            }
+                        ]
+                    }
+                ]
+            }
+        ]
+    };
+
+    return spec;
+};
+
+
+IPA.dns.update_dns_records_action = function(spec) {
+
+    spec = spec || {};
+
+    spec.name = spec.name || 'update_dns_records';
+    spec.label = spec.label || '@i18n:objects.dnsconfig.update_dns';
+
+    var that = IPA.action(spec);
+
+    that.execute_action = function() {
+        var spec = {
+            title: '@i18n:objects.dnsconfig.update_dns',
+            message: '@i18n:objects.dnsconfig.update_dns_dialog_msg',
+            ok_label: '@i18n:buttons.update'
+        };
+
+        that.dialog = IPA.confirm_dialog(spec);
+
+        that.dialog.on_ok = function() {
+
+            var command = rpc.command({
+                entity: 'dns',
+                method: 'update_system_records',
+                on_success: function(data) {
+                    var status = data.result.value;
+                    if (status) IPA.notify_success(
+                        '@i18n:objects.dnsconfig.updated_dns');
+                }
+            });
+
+            command.execute();
+        };
+
+        that.dialog.open();
+    };
+
+    return that;
+};
+
 IPA.dnszone_details_facet = function(spec, no_init) {
 
     spec = spec || {};
@@ -2263,28 +2366,28 @@ IPA.dns.ptr_redirection_dialog = function(spec) {
     //2nd step: find target zone
     that.find_zone = function(data) {
         var zones = data.result.result;
-        var target_zone = null;
+        var target_zone = {
+            index: 100,
+            target_zone: ''
+        };
 
         for (var i=0; i<zones.length; i++) {
 
             var zone_name = rpc.extract_objects(zones[i].idnsname)[0];
-            if (that.reverse_address.indexOf(zone_name) > -1) {
-                var msg = text.get('@i18n:objects.dnsrecord.ptr_redir_zone');
-                msg = msg.replace('${zone}', zone_name);
-                that.append_status(msg);
+            var index = that.reverse_address.indexOf(zone_name);
 
-                if (!target_zone ||
-                    (target_zone && zone_name.length > target_zone.length)) {
-
-                    target_zone = zone_name;
-                }
-
-                break;
+            if (index > -1 && target_zone.index > index) {
+                target_zone.index = index;
+                target_zone.target_zone = zone_name;
             }
         }
 
-        if (target_zone) {
-            that.zone = target_zone;
+        if (target_zone.target_zone !== '') {
+
+            that.zone = target_zone.target_zone;
+            var msg = text.get('@i18n:objects.dnsrecord.ptr_redir_zone');
+            msg = msg.replace('${zone}', that.zone);
+            that.append_status(msg);
             that.check_record();
         } else {
             that.append_status(text.get('@i18n:objects.dnsrecord.ptr_redir_zone_err'));
@@ -2521,6 +2624,7 @@ exp.config_spec = make_config_spec();
 exp.zone_spec = make_zone_spec();
 exp.record_spec = make_record_spec();
 exp.forwardzone_spec = make_forwardzone_spec();
+exp.dnsserver_spec = make_dnsserver_spec();
 exp.register = function() {
     var e = reg.entity;
     var w = reg.widget;
@@ -2532,6 +2636,7 @@ exp.register = function() {
     e.register({type: 'dnszone', spec: exp.zone_spec});
     e.register({type: 'dnsrecord', spec: exp.record_spec});
     e.register({type: 'dnsforwardzone', spec: exp.forwardzone_spec});
+    e.register({type: 'dnsserver', spec: exp.dnsserver_spec});
 
     w.register('dnszone_name', IPA.dnszone_name_widget);
     w.register('force_dnszone_add_checkbox', IPA.force_dnszone_add_checkbox_widget);
@@ -2553,6 +2658,7 @@ exp.register = function() {
 
     a.register('dns_add_permission', IPA.dns.add_permission_action);
     a.register('dns_remove_permission', IPA.dns.remove_permission_action);
+    a.register('update_dns_records', IPA.dns.update_dns_records_action);
 };
 
 phases.on('registration', exp.register);
