@@ -19,7 +19,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-define(['./ipa',
+define(['./builder',
+        './ipa',
         './jquery',
         './phases',
         './reg',
@@ -30,7 +31,7 @@ define(['./ipa',
         './association',
         './entity',
         './certificate'],
-    function(IPA, $, phases, reg, rpc, text) {
+    function(builder, IPA, $, phases, reg, rpc, text) {
 
 var exp = IPA.host = {};
 
@@ -87,7 +88,14 @@ return {
                             name: 'fqdn',
                             other_entity: 'dnsrecord'
                         },
-                        'krbprincipalname',
+                        {
+                            $type: 'krb_principal_multivalued',
+                            name: 'krbprincipalname',
+                            item_name: 'principal',
+                            child_spec: {
+                                $type: 'krb_principal'
+                            }
+                        },
                         {
                             $type: 'textarea',
                             name: 'description'
@@ -107,12 +115,37 @@ return {
                             name: 'macaddress',
                             flags: ['w_if_no_aci']
                         },
+                        {
+                            $type: 'custom_checkboxes',
+                            label: '@i18n:authtype.auth_indicators',
+                            name: 'krbprincipalauthind',
+                            add_dialog_title: '@i18n:authtype.custom_auth_ind_title',
+                            add_field_label: '@i18n:authtype.auth_indicator',
+                            options: [
+                                {
+                                    label: '@i18n:authtype.otp',
+                                    value: 'otp'
+                                },
+                                {
+                                    label: '@i18n:authtype.type_radius',
+                                    value: 'radius'
+                                }
+                            ],
+                            tooltip: {
+                                title: '@mc-opt:host_add:krbprincipalauthind:doc'
+                            }
+                        },
                         {
                             name: 'ipakrbokasdelegate',
                             $type: 'checkbox',
                             acl_param: 'krbticketflags',
                             flags: ['w_if_no_aci']
                         },
+                        {
+                            name: 'ipakrboktoauthasdelegate',
+                            $type: 'checkbox',
+                            acl_param: 'krbticketflags'
+                        },
                         {
                             name: 'ipaassignedidview',
                             $type: 'link',
@@ -140,8 +173,12 @@ return {
                     name: 'certificate',
                     fields: [
                         {
-                            $type: 'certificate',
-                            name: 'usercertificate'
+                            $type: 'certs',
+                            adapter: {
+                                $type: 'object_adapter',
+                                result_index: 1
+                            },
+                            label: '@i18n:objects.cert.certificates'
                         }
                     ]
                 },
@@ -187,6 +224,38 @@ return {
                                     link: true
                                 }
                             ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_read_keys_host',
+                            name: 'ipaallowedtoperform_read_keys_host',
+                            add_method: 'allow_retrieve_keytab',
+                            remove_method: 'disallow_retrieve_keytab',
+                            add_title: '@i18n:keytab.add_retrive',
+                            remove_title: '@i18n:keytab.remove_retrieve',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_read_keys_host',
+                                    label: '@mo:host.label_singular',
+                                    link: true
+                                }
+                            ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_read_keys_hostgroup',
+                            name: 'ipaallowedtoperform_read_keys_hostgroup',
+                            add_method: 'allow_retrieve_keytab',
+                            remove_method: 'disallow_retrieve_keytab',
+                            add_title: '@i18n:keytab.add_retrive',
+                            remove_title: '@i18n:keytab.remove_retrieve',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_read_keys_hostgroup',
+                                    label: '@mo:hostgroup.label_singular',
+                                    link: true
+                                }
+                            ]
                         }
                     ]
                 },
@@ -226,6 +295,38 @@ return {
                                     link: true
                                 }
                             ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_write_keys_host',
+                            name: 'ipaallowedtoperform_write_keys_host',
+                            add_method: 'allow_create_keytab',
+                            remove_method: 'disallow_create_keytab',
+                            add_title: '@i18n:keytab.add_create',
+                            remove_title: '@i18n:keytab.remove_create',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_write_keys_host',
+                                    label: '@mo:host.label_singular',
+                                    link: true
+                                }
+                            ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_write_keys_hostgroup',
+                            name: 'ipaallowedtoperform_write_keys_hostgroup',
+                            add_method: 'allow_create_keytab',
+                            remove_method: 'disallow_create_keytab',
+                            add_title: '@i18n:keytab.add_create',
+                            remove_title: '@i18n:keytab.remove_create',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_write_keys_hostgroup',
+                                    label: '@mo:hostgroup.label_singular',
+                                    link: true
+                                }
+                            ]
                         }
                     ]
                 }
@@ -270,8 +371,16 @@ return {
                 'request_cert'],
             state: {
                 evaluators: [
-                    IPA.host.has_password_evaluator,
-                    IPA.host.has_keytab_evaluator,
+                    {
+                        $factory: IPA.host.has_password_evaluator,
+                        param: 'has_password',
+                        adapter: { $type: 'batch', result_index: 0 }
+                    },
+                    {
+                        $factory: IPA.has_keytab_evaluator,
+                        param: 'has_keytab',
+                        adapter: { $type: 'batch', result_index: 0 }
+                    },
                     IPA.host.userpassword_acl_evaluator,
                     IPA.host.krbprincipalkey_acl_evaluator,
                     IPA.cert.certificate_evaluator
@@ -347,6 +456,12 @@ return {
                         $type: 'force_host_add_checkbox',
                         name: 'force',
                         metadata: '@mc-opt:host_add:force'
+                    },
+                    {
+                        $type: 'checkbox',
+                        name: 'random',
+                        label: '@i18n:objects.host.generate_otp',
+                        title: '@i18n:objects.host.generate_otp'
                     }
                 ]
             }
@@ -363,6 +478,32 @@ IPA.host.details_facet = function(spec, no_init) {
     that.certificate_loaded = IPA.observer();
     that.certificate_updated = IPA.observer();
 
+    that.create_refresh_command = function() {
+        var pkey = that.get_pkey();
+
+        var batch = rpc.batch_command({
+            name: 'host_details_refresh'
+        });
+
+        var host_command = that.details_facet_create_refresh_command();
+        batch.add_command(host_command);
+
+        var certificates = rpc.command({
+            entity: 'cert',
+            method: 'find',
+            retry: false,
+            options: {
+                host: [ pkey ],
+                all: true
+            }
+        });
+
+        batch.add_command(certificates);
+
+        return batch;
+
+    };
+
     that.get_refresh_command_name = function() {
         return that.entity.name+'_show_'+that.get_pkey();
     };
@@ -529,13 +670,67 @@ IPA.host_adder_dialog = function(spec) {
 
     var that = IPA.entity_adder_dialog(spec);
 
+    that.init = function() {
+        that.added.attach(that.handle_random_otp);
+    };
+
     that.create_content = function() {
         that.entity_adder_dialog_create_content();
         that.container.addClass('host-adder-dialog');
     };
 
+    that.create_result_dialog = function() {
+        spec = {
+            title: text.get('@i18n:dialogs.result'),
+            sections: [
+                {
+                    show_header: false,
+                    fields: [
+                        {
+                            $type: 'text',
+                            name: 'randompassword',
+                            label: '@i18n:objects.host.generated_otp',
+                            read_only: true
+                        }
+                    ]
+                }
+            ]
+        };
+
+        var dialog = IPA.dialog(spec);
+
+        dialog.create_button({
+            name: 'close',
+            label: text.get('@i18n:buttons.close'),
+            click: function() {
+                dialog.close();
+            }
+        });
+
+
+        return dialog;
+    };
+
+    that.handle_random_otp = function(data, method) {
+        var dialog = that.create_result_dialog();
+        var random_passwd = that.get_field('random').get_value()[0];
+
+        if ((method === 'add' || method === 'add_and_edit') && random_passwd) {
+            var field = dialog.get_field('randompassword');
+            dialog.open();
+            field.load(data);
+        }
+        else if (method === 'add_and_add_another' && random_passwd) {
+            var message = text.get('@i18n:objects.host.generated_otp') +
+                ": " + data.result.result.randompassword;
+            that.show_message(message);
+        }
+    };
+
     that.on_error = rpc.create_4304_error_handler(that);
 
+    that.init();
+
     return that;
 };
 
@@ -795,17 +990,6 @@ IPA.host.krbprincipalkey_acl_evaluator = function(spec) {
     return that;
 };
 
-IPA.host.has_keytab_evaluator = function(spec) {
-
-    spec.name = spec.name || 'has_keytab_evaluator';
-    spec.attribute = spec.attribute || 'has_keytab';
-    spec.value = spec.value || [true];
-    spec.representation = spec.representation || 'has_keytab';
-
-    var that = IPA.value_state_evaluator(spec);
-    return that;
-};
-
 IPA.host_password_widget = function(spec) {
 
     spec = spec || {};
@@ -883,8 +1067,11 @@ IPA.host.has_password_evaluator = function(spec) {
     spec.attribute = spec.attribute || 'has_password';
     spec.value = spec.value || [true];
     spec.representation = spec.representation || 'has_password';
+    spec.param = spec.param || 'has_password';
+    spec.adapter = spec.adapter || { $type: 'adapter' };
 
     var that = IPA.value_state_evaluator(spec);
+
     return that;
 };
 
