@@ -21,6 +21,7 @@
 define([
     'dojo/_base/declare',
     './field',
+    './builder',
     './ipa',
     './jquery',
     './phases',
@@ -31,7 +32,7 @@ define([
     './search',
     './association',
     './entity'],
-        function(declare, field_mod, IPA, $, phases, reg, rpc, text) {
+        function(declare, field_mod, builder, IPA, $, phases, reg, rpc, text) {
 
 var exp =IPA.service = {};
 
@@ -57,7 +58,16 @@ return {
     facets: [
         {
             $type: 'search',
-            columns: [ 'krbprincipalname' ]
+            $factory: IPA.service.search_facet,
+            columns: [
+                {
+                    name: 'krbcanonicalname',
+                    adapter: {
+                        $type: 'alternate_attr_field_adapter',
+                        alt_attr: 'krbprincipalname'
+                    }
+                }
+            ]
         },
         {
             $type: 'details',
@@ -66,7 +76,14 @@ return {
                 {
                     name: 'details',
                     fields: [
-                        'krbprincipalname',
+                        {
+                            $type: 'krb_principal_multivalued',
+                            name: 'krbprincipalname',
+                            item_name: 'principal',
+                            child_spec: {
+                                $type: 'krb_principal'
+                            }
+                        },
                         {
                             name: 'service',
                             label: '@i18n:objects.service.service',
@@ -108,11 +125,36 @@ return {
                                 }
                             ]
                         },
+                        {
+                            $type: 'custom_checkboxes',
+                            label: '@i18n:authtype.auth_indicators',
+                            name: 'krbprincipalauthind',
+                            add_dialog_title: '@i18n:authtype.custom_auth_ind_title',
+                            add_field_label: '@i18n:authtype.auth_indicator',
+                            options: [
+                                {
+                                    label: '@i18n:authtype.otp',
+                                    value: 'otp'
+                                },
+                                {
+                                    label: '@i18n:authtype.type_radius',
+                                    value: 'radius'
+                                }
+                            ],
+                            tooltip: {
+                                title: '@mc-opt:service_add:krbprincipalauthind:doc'
+                            }
+                        },
                         {
                             name: 'ipakrbokasdelegate',
                             $type: 'checkbox',
                             acl_param: 'krbticketflags'
                         },
+                        {
+                            name: 'ipakrboktoauthasdelegate',
+                            $type: 'checkbox',
+                            acl_param: 'krbticketflags'
+                        },
                         {
                             name: 'ipakrbrequirespreauth',
                             $type: 'checkbox',
@@ -134,8 +176,12 @@ return {
                     name: 'certificate',
                     fields: [
                         {
-                            $type: 'certificate',
-                            name: 'usercertificate'
+                            $type: 'certs',
+                            adapter: {
+                                $type: 'object_adapter',
+                                result_index: 1
+                            },
+                            label: '@i18n:objects.cert.certificates'
                         }
                     ]
                 },
@@ -181,6 +227,38 @@ return {
                                     link: true
                                 }
                             ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_read_keys_host',
+                            name: 'ipaallowedtoperform_read_keys_host',
+                            add_method: 'allow_retrieve_keytab',
+                            remove_method: 'disallow_retrieve_keytab',
+                            add_title: '@i18n:keytab.add_retrive',
+                            remove_title: '@i18n:keytab.remove_retrieve',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_read_keys_host',
+                                    label: '@mo:host.label_singular',
+                                    link: true
+                                }
+                            ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_read_keys_hostgroup',
+                            name: 'ipaallowedtoperform_read_keys_hostgroup',
+                            add_method: 'allow_retrieve_keytab',
+                            remove_method: 'disallow_retrieve_keytab',
+                            add_title: '@i18n:keytab.add_retrive',
+                            remove_title: '@i18n:keytab.remove_retrieve',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_read_keys_hostgroup',
+                                    label: '@mo:hostgroup.label_singular',
+                                    link: true
+                                }
+                            ]
                         }
                     ]
                 },
@@ -220,6 +298,38 @@ return {
                                     link: true
                                 }
                             ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_write_keys_host',
+                            name: 'ipaallowedtoperform_write_keys_host',
+                            add_method: 'allow_create_keytab',
+                            remove_method: 'disallow_create_keytab',
+                            add_title: '@i18n:keytab.add_create',
+                            remove_title: '@i18n:keytab.remove_create',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_write_keys_host',
+                                    label: '@mo:host.label_singular',
+                                    link: true
+                                }
+                            ]
+                        },
+                        {
+                            $type: 'association_table',
+                            id: 'service_ipaallowedtoperform_write_keys_hostgroup',
+                            name: 'ipaallowedtoperform_write_keys_hostgroup',
+                            add_method: 'allow_create_keytab',
+                            remove_method: 'disallow_create_keytab',
+                            add_title: '@i18n:keytab.add_create',
+                            remove_title: '@i18n:keytab.remove_create',
+                            columns: [
+                                {
+                                    name: 'ipaallowedtoperform_write_keys_hostgroup',
+                                    label: '@mo:hostgroup.label_singular',
+                                    link: true
+                                }
+                            ]
                         }
                     ]
                 }
@@ -234,7 +344,11 @@ return {
             ],
             state: {
                 evaluators: [
-                    IPA.service.has_keytab_evaluator,
+                    {
+                        $factory: IPA.has_keytab_evaluator,
+                        param: 'has_keytab',
+                        adapter: { $type: 'batch', result_index: 0 }
+                    },
                     IPA.service.krbprincipalkey_acl_evaluator,
                     IPA.cert.certificate_evaluator
                 ]
@@ -298,12 +412,78 @@ return {
     }
 };};
 
+
+/**
+ * Custom search facet for services. It has alternative primary key, in case
+ * that the service doesn't have canonical name.
+ */
+IPA.service.search_facet = function(spec) {
+    spec = spec || {};
+
+    spec.alternative_pkey = spec.alternative_pkey || 'krbprincipalname';
+
+    var that = IPA.search_facet(spec);
+
+    that.alternative_pkey = spec.alternative_pkey;
+
+    that.get_records_map = function(data) {
+
+        var records_map = $.ordered_map();
+
+        var result = data.result.result;
+        var pkey_name = that.managed_entity.metadata.primary_key ||
+                                                        that.primary_key_name;
+        var adapter = builder.build('adapter', 'adapter', {context: that});
+
+        for (var i=0; i<result.length; i++) {
+            var record = result[i];
+            var pkey = adapter.load(record, pkey_name)[0];
+            if (pkey === undefined && that.alternative_pkey) {
+                pkey = adapter.load(record, that.alternative_pkey)[0];
+            }
+            if (that.filter_records(records_map, pkey, record)) {
+                records_map.put(pkey, record);
+            }
+        }
+
+        return records_map;
+    };
+
+    return that;
+};
+
+
 IPA.service.details_facet = function(spec, no_init) {
 
     var that = IPA.details_facet(spec, true);
     that.certificate_loaded = IPA.observer();
     that.certificate_updated = IPA.observer();
 
+    that.create_refresh_command = function() {
+        var pkey = that.get_pkey();
+
+        var batch = rpc.batch_command({
+            name: 'services_details_refresh'
+        });
+
+        var service_command = that.details_facet_create_refresh_command();
+        batch.add_command(service_command);
+
+        var certificates = rpc.command({
+            entity: 'cert',
+            method: 'find',
+            retry: false,
+            options: {
+                service: [ pkey ],
+                all: true
+            }
+        });
+
+        batch.add_command(certificates);
+
+        return batch;
+    };
+
     if (!no_init) that.init_details_facet();
 
     return that;
@@ -317,14 +497,14 @@ IPA.service_adder_dialog = function(spec) {
 
     var init = function() {
 
-        //small hack - krbprincipalname should not be displayed. This way
+        //small hack - krbcanonicalname should not be displayed. This way
         //creation of associated widget is skipped.
         //In future it would be better split section definion into widget and
         //fields definition and create custom field with two associated
         //widgets - 'service' and 'host' with this dialog's save logic.
         that.builder.build_field({
             $type: 'field',
-            name: 'krbprincipalname',
+            name: 'krbcanonicalname',
             required: false
         });
     };
@@ -337,7 +517,7 @@ IPA.service_adder_dialog = function(spec) {
         field = that.fields.get_field('host');
         var host = field.save()[0];
 
-        record['krbprincipalname'] = [ service+'/'+host ];
+        record['krbcanonicalname'] = [ service+'/'+host ];
 
         field = that.fields.get_field('force');
         record['force'] = field.save();
@@ -516,17 +696,6 @@ IPA.service.krbprincipalkey_acl_evaluator = function(spec) {
     return that;
 };
 
-IPA.service.has_keytab_evaluator = function(spec) {
-
-    spec.name = spec.name || 'has_keytab_evaluator';
-    spec.attribute = spec.attribute || 'has_keytab';
-    spec.value = spec.value || [true];
-    spec.representation = spec.representation || 'has_keytab';
-
-    var that = IPA.value_state_evaluator(spec);
-    return that;
-};
-
 IPA.service.certificate_policy = function(spec) {
 
     spec = spec || {};
