@@ -21,24 +21,24 @@
 Base classes for all front-end plugins.
 """
 
-import re
 from distutils import version
-from textwrap import wrap
 
 import six
 
 from ipapython.version import API_VERSION
 from ipapython.ipa_log_manager import root_logger
 from ipalib.base import NameSpace
-from ipalib.plugable import Plugin
-from ipalib.parameters import create_param, Param, Str, Flag, Password
+from ipalib.plugable import Plugin, APINameSpace
+from ipalib.parameters import create_param, Param, Str, Flag
+from ipalib.parameters import Password  # pylint: disable=unused-import
 from ipalib.output import Output, Entry, ListOfEntries
 from ipalib.text import _
 from ipalib.errors import (ZeroArgumentError, MaxArgumentError, OverlapError,
-    VersionError, OptionError, InvocationError,
+    VersionError, OptionError,
     ValidationError, ConversionError)
-from ipalib import messages
-from ipalib.util import json_serialize
+from ipalib import errors, messages
+from ipalib.request import context, context_frame
+from ipalib.util import classproperty, json_serialize
 
 if six.PY3:
     unicode = str
@@ -350,13 +350,6 @@ class HasParam(Plugin):
         for spec in get():
             param = create_param(spec)
             if env is None or param.use_in_context(env):
-                if env is not None and not hasattr(param, 'env'):
-                    # Force specified environment. The way it is done is violation of ReadOnly promise.
-                    # Unfortunately, all alternatives are worse from both performance and code complexity
-                    # points of view. See following threads on freeipa-devel@ for references:
-                    # https://www.redhat.com/archives/freeipa-devel/2011-August/msg00000.html
-                    # https://www.redhat.com/archives/freeipa-devel/2011-August/msg00011.html
-                    object.__setattr__(param, 'env', env)
                 yield param
 
     def _create_param_namespace(self, name, env=None):
@@ -370,6 +363,13 @@ class HasParam(Plugin):
                 check(namespace)
         setattr(self, name, namespace)
 
+    @property
+    def context(self):
+        return context.current_frame
+
+
+_callback_registry = {}
+
 
 class Command(HasParam):
     """
@@ -386,15 +386,21 @@ class Command(HasParam):
     >>> class my_command(Command):
     ...     pass
     ...
-    >>> api.register(my_command)
+    >>> api.add_plugin(my_command)
     >>> api.finalize()
     >>> list(api.Command)
-    ['my_command']
+    [<class '__main__.my_command'>]
     >>> api.Command.my_command # doctest:+ELLIPSIS
     ipalib.frontend.my_command()
-    """
 
-    finalize_early = False
+    This class's subclasses allow different types of callbacks to be added and
+    removed to them.
+    Registering a callback is done either by ``register_callback``, or by
+    defining a ``<type>_callback`` method.
+
+    Subclasses should define the `callback_types` attribute as a tuple of
+    allowed callback types.
+    """
 
     takes_options = tuple()
     takes_args = tuple()
@@ -416,6 +422,20 @@ class Command(HasParam):
     msg_summary = None
     msg_truncated = _('Results are truncated, try a more specific search')
 
+    callback_types = ('interactive_prompt',)
+
+    api_version = API_VERSION
+
+    @classmethod
+    def __topic_getter(cls):
+        return cls.__module__.rpartition('.')[2]
+
+    topic = classproperty(__topic_getter)
+
+    @property
+    def forwarded_name(self):
+        return self.full_name
+
     def __call__(self, *args, **options):
         """
         Perform validation and then execute the command.
@@ -424,32 +444,40 @@ class Command(HasParam):
         XML-RPC and the executed an the nearest IPA server.
         """
         self.ensure_finalized()
-        version_provided = 'version' in options
-        if version_provided:
+        with context_frame():
+            self.context.principal = getattr(context, 'principal', None)
+            return self.__do_call(*args, **options)
+
+    def __do_call(self, *args, **options):
+        self.context.__messages = []
+        if 'version' in options:
             self.verify_client_version(unicode(options['version']))
         elif self.api.env.skip_version_check and not self.api.env.in_server:
             options['version'] = u'2.0'
         else:
-            options['version'] = API_VERSION
+            options['version'] = self.api_version
+            if self.api.env.in_server:
+                # add message only on server side
+                self.add_message(
+                    messages.VersionMissing(server_version=self.api_version))
         params = self.args_options_2_params(*args, **options)
         self.debug(
             'raw: %s(%s)', self.name, ', '.join(self._repr_iter(**params))
         )
-        params.update(self.get_default(**params))
+        if self.api.env.in_server:
+            params.update(self.get_default(**params))
         params = self.normalize(**params)
         params = self.convert(**params)
         self.debug(
             '%s(%s)', self.name, ', '.join(self._repr_iter(**params))
         )
-        self.validate(**params)
+        if self.api.env.in_server:
+            self.validate(**params)
         (args, options) = self.params_2_args_options(**params)
         ret = self.run(*args, **options)
-        if (not version_provided and isinstance(ret, dict) and
-                self.api.env.in_server):
-            # add message only on server side
-            messages.add_message(
-                API_VERSION, ret,
-                messages.VersionMissing(server_version=API_VERSION))
+        if isinstance(ret, dict):
+            for message in self.context.__messages:
+                messages.add_message(options['version'], ret, message)
         if (
             isinstance(ret, dict)
             and 'summary' in self.output
@@ -460,18 +488,8 @@ class Command(HasParam):
             self.validate_output(ret, options['version'])
         return ret
 
-    def soft_validate(self, values):
-        errors = dict()
-        for p in self.params():
-            try:
-                value = values.get(p.name)
-                values[p.name] = p(value, **values)
-            except InvocationError as e:
-                errors[p.name] = str(e)
-        return dict(
-            values=values,
-            errors=errors,
-        )
+    def add_message(self, message):
+        self.context.__messages.append(message)
 
     def _repr_iter(self, **params):
         """
@@ -564,8 +582,21 @@ class Command(HasParam):
         """
         Split params into (args, options).
         """
-        args = tuple(params.get(name, None) for name in self.args)
+        args = tuple()
         options = dict(self.__params_2_options(params))
+
+        is_arg = True
+        for name in self.args:
+            try:
+                value = params[name]
+            except KeyError:
+                is_arg = False
+                continue
+            if is_arg:
+                args += (value,)
+            else:
+                options[name] = value
+
         return (args, options)
 
     def __params_2_options(self, params):
@@ -590,7 +621,7 @@ class Command(HasParam):
             # Backend.textui.prompt does not fill in the default value,
             # we have to do it ourselves
             if not raw.strip():
-                raw = default
+                return None
 
             try:
                 return param(raw, **kw)
@@ -645,7 +676,7 @@ class Command(HasParam):
             if kw.get(param.name, None) is None:
                 continue
 
-    def get_default(self, **kw):
+    def get_default(self, _params=None, **kw):
         """
         Return a dictionary of defaults for all missing required values.
 
@@ -662,15 +693,17 @@ class Command(HasParam):
         >>> c.get_default(color=u'Yellow')
         {}
         """
-        params = [p.name for p in self.params() if p.name not in kw and (p.required or p.autofill)]
-        return dict(self.__get_default_iter(params, kw))
+        if _params is None:
+            _params = [p.name for p in self.params()
+                       if p.name not in kw and (p.required or p.autofill)]
+        return dict(self.__get_default_iter(_params, kw))
 
-    def get_default_of(self, name, **kw):
+    def get_default_of(self, _name, **kw):
         """
-        Return default value for parameter `name`.
+        Return default value for parameter `_name`.
         """
-        default = dict(self.__get_default_iter([name], kw))
-        return default.get(name)
+        default = dict(self.__get_default_iter([_name], kw))
+        return default.get(_name)
 
     def __get_default_iter(self, params, kw):
         """
@@ -692,13 +725,18 @@ class Command(HasParam):
             if param.name in dep:
                 if param.name in kw:
                     # Parameter is specified, convert and validate the value.
-                    kw[param.name] = param(kw[param.name], **kw)
+                    value = param(kw[param.name], **kw)
+                    if self.api.env.in_server:
+                        param.validate(value, supplied=True)
+                    kw[param.name] = value
                 else:
                     # Parameter is not specified, use default value. Convert
                     # and validate the value, it might not be returned so
                     # there's no guarantee it will be converted and validated
                     # later.
                     default = param(None, **kw)
+                    if self.api.env.in_server:
+                        param.validate(default)
                     if default is not None:
                         kw[param.name] = default
                     hasdefault = True
@@ -721,7 +759,7 @@ class Command(HasParam):
         """
         for param in self.params():
             value = kw.get(param.name, None)
-            param.validate(value, self.env.context, supplied=param.name in kw)
+            param.validate(value, supplied=param.name in kw)
 
     def verify_client_version(self, client_version):
         """
@@ -744,8 +782,6 @@ class Command(HasParam):
 
         if server_major != client_major:
             raise VersionError(cver=client_version, sver=API_VERSION, server=self.env.xmlrpc_uri)
-        if client_minor > server_minor:
-            raise VersionError(cver=client_version, sver=API_VERSION, server=self.env.xmlrpc_uri)
 
     def run(self, *args, **options):
         """
@@ -782,7 +818,16 @@ class Command(HasParam):
         """
         Forward call over RPC to this same command on server.
         """
-        return self.Backend.rpcclient.forward(self.name, *args, **kw)
+        try:
+            return self.Backend.rpcclient.forward(self.forwarded_name,
+                                                  *args, **kw)
+        except errors.RequirementError as e:
+            if self.api.env.context != 'cli':
+                raise
+            name = getattr(e, 'name', None)
+            if name is None or name not in self.params:
+                raise
+            raise errors.RequirementError(name=self.params[name].cli_name)
 
     def _on_finalize(self):
         """
@@ -955,12 +1000,6 @@ class Command(HasParam):
     def get_output_params(self):
         for param in self._get_param_iterable('output_params', verb='has'):
             yield param
-        if self.params is None:
-            return
-        for param in self.params():
-            if 'no_output' in param.flags:
-                continue
-            yield param
 
     def get_summary_default(self, output):
         if self.msg_summary:
@@ -1018,9 +1057,11 @@ class Command(HasParam):
             outp = self.output[o]
             if 'no_display' in outp.flags:
                 continue
-            result = output[o]
+            result = output.get(o)
 
-            if o.lower() == 'count' and result == 0:
+            if o == 'value':
+                continue
+            elif o.lower() == 'count' and result == 0:
                 rv = 1
             elif o.lower() == 'failed':
                 if entry_count(result) == 0:
@@ -1066,6 +1107,54 @@ class Command(HasParam):
 
         return json_dict
 
+    @classmethod
+    def get_callbacks(cls, callback_type):
+        """Yield callbacks of the given type"""
+        # Use one shared callback registry, keyed on class, to avoid problems
+        # with missing attributes being looked up in superclasses
+        callbacks = _callback_registry.get(callback_type, {}).get(cls, [None])
+        for callback in callbacks:
+            if callback is None:
+                try:
+                    yield getattr(cls, '%s_callback' % callback_type)
+                except AttributeError:
+                    pass
+            else:
+                yield callback
+
+    @classmethod
+    def register_callback(cls, callback_type, callback, first=False):
+        """Register a callback
+
+        :param callback_type: The callback type (e.g. 'pre', 'post')
+        :param callback: The callable added
+        :param first: If true, the new callback will be added before all
+            existing callbacks; otherwise it's added after them
+
+        Note that callbacks registered this way will be attached to this class
+        only, not to its subclasses.
+        """
+        assert callback_type in cls.callback_types
+        assert callable(callback)
+        _callback_registry.setdefault(callback_type, {})
+        try:
+            callbacks = _callback_registry[callback_type][cls]
+        except KeyError:
+            callbacks = _callback_registry[callback_type][cls] = [None]
+        if first:
+            callbacks.insert(0, callback)
+        else:
+            callbacks.append(callback)
+
+    @classmethod
+    def register_interactive_prompt_callback(cls, callback, first=False):
+        """Shortcut for register_callback('interactive_prompt', ...)"""
+        cls.register_callback('interactive_prompt', callback, first)
+
+    def interactive_prompt_callback(self, kw):
+        return
+
+
 class LocalOrRemote(Command):
     """
     A command that is explicitly executed locally or remotely.
@@ -1092,7 +1181,7 @@ class LocalOrRemote(Command):
         When running in a server context, this command is always executed
         locally and the value of ``options['server']`` is ignored.
         """
-        if options['server'] and not self.env.in_server:
+        if options.get('server', False) and not self.env.in_server:
             return self.forward(*args, **options)
         return self.execute(*args, **options)
 
@@ -1111,10 +1200,11 @@ class Local(Command):
         """
         return self.forward(*args, **options)
 
+    def forward(self, *args, **options):
+        return self.execute(*args, **options)
+
 
 class Object(HasParam):
-    finalize_early = False
-
     # Create stubs for attributes that are set in _on_finalize()
     backend = Plugin.finalize_attr('backend')
     methods = Plugin.finalize_attr('methods')
@@ -1175,8 +1265,10 @@ class Object(HasParam):
         if name not in self.api:
             return
         namespace = self.api[name]
-        assert type(namespace) is NameSpace
+        assert type(namespace) is APINameSpace
         for plugin in namespace(): # Equivalent to dict.itervalues()
+            if plugin is not namespace[plugin.name]:
+                continue
             if plugin.obj_name == self.name:
                 yield plugin
 
@@ -1247,41 +1339,31 @@ class Attribute(Plugin):
     In practice the `Attribute` class is not used directly, but rather is
     only the base class for the `Method` class.  Also see the `Object` class.
     """
-    finalize_early = False
+    obj_version = '1'
 
-    NAME_REGEX = re.compile(
-        '^(?P<obj>[a-z][a-z0-9]+)_(?P<attr>[a-z][a-z0-9]+(?:_[a-z][a-z0-9]+)*)$'
-    )
+    @property
+    def obj_name(self):
+        return self.name.partition('_')[0]
 
-    # Create stubs for attributes that are set in _on_finalize()
-    __obj = Plugin.finalize_attr('_Attribute__obj')
+    @property
+    def obj_full_name(self):
+        if self.obj is not None:
+            return self.obj.full_name
+        else:
+            return None
 
-    def __init__(self, api):
-        m = self.NAME_REGEX.match(type(self).__name__)
-        assert m
-        self.__obj_name = m.group('obj')
-        self.__attr_name = m.group('attr')
-        super(Attribute, self).__init__(api)
+    @property
+    def attr_name(self):
+        prefix = '{}_'.format(self.obj_name)
+        assert self.name.startswith(prefix)
+        return self.name[len(prefix):]
 
-    def __get_obj_name(self):
-        return self.__obj_name
-    obj_name = property(__get_obj_name)
-
-    def __get_attr_name(self):
-        return self.__attr_name
-    attr_name = property(__get_attr_name)
-
-    def __get_obj(self):
-        """
-        Returns the obj instance this attribute is associated with, or None
-        if no association has been set.
-        """
-        return self.__obj
-    obj = property(__get_obj)
-
-    def _on_finalize(self):
-        self.__obj = self.api.Object[self.obj_name]
-        super(Attribute, self)._on_finalize()
+    @property
+    def obj(self):
+        if self.obj_name is not None and self.obj_version is not None:
+            return self.api.Object[self.obj_name, self.obj_version]
+        else:
+            return None
 
 
 class Method(Attribute, Command):
@@ -1314,15 +1396,15 @@ class Method(Attribute, Command):
     >>> class user(Object):
     ...     pass
     ...
-    >>> api.register(user_add)
-    >>> api.register(user)
+    >>> api.add_plugin(user_add)
+    >>> api.add_plugin(user)
     >>> api.finalize()
 
     First, the ``user_add`` plugin can be accessed through the ``api.Method``
     namespace:
 
     >>> list(api.Method)
-    ['user_add']
+    [<class '__main__.user_add'>]
     >>> api.Method.user_add(version=u'2.88')  # Will call user_add.run()
     {'result': 'Added the user!'}
 
@@ -1333,7 +1415,7 @@ class Method(Attribute, Command):
     plugin can also be accessed through the ``api.Command`` namespace:
 
     >>> list(api.Command)
-    ['user_add']
+    [<class '__main__.user_add'>]
     >>> api.Command.user_add(version=u'2.88') # Will call user_add.run()
     {'result': 'Added the user!'}
 
@@ -1341,7 +1423,7 @@ class Method(Attribute, Command):
     `Object`:
 
     >>> list(api.Object)
-    ['user']
+    [<class '__main__.user'>]
     >>> list(api.Object.user.methods)
     ['add']
     >>> api.Object.user.methods.add(version=u'2.88') # Will call user_add.run()
@@ -1354,16 +1436,12 @@ class Method(Attribute, Command):
     extra_args_first = False
 
     def get_output_params(self):
-        for param in self.obj.params():
-            if 'no_output' in param.flags:
-                continue
-            yield param
-        for param in self.params():
-            if param.name not in list(self.obj.params):
+        if self.obj is not None:
+            for param in self.obj.params():
                 if 'no_output' in param.flags:
                     continue
                 yield param
-        for param in self._get_param_iterable('output_params', verb='has'):
+        for param in super(Method, self).get_output_params():
             yield param
 
 
@@ -1382,14 +1460,14 @@ class Updater(Plugin):
     >>> class my(Object):
     ...     pass
     ...
-    >>> api.register(my)
+    >>> api.add_plugin(my)
     >>> class my_update(Updater):
     ...     pass
     ...
-    >>> api.register(my_update)
+    >>> api.add_plugin(my_update)
     >>> api.finalize()
     >>> list(api.Updater)
-    ['my_update']
+    [<class '__main__.my_update'>]
     >>> api.Updater.my_update # doctest:+ELLIPSIS
     ipalib.frontend.my_update()
     """
