@@ -104,21 +104,22 @@ import decimal
 import base64
 import datetime
 from six.moves.xmlrpc_client import MAXINT, MININT
-import encodings.idna
 
-import dns.name
 import six
 
 from ipalib.text import _ as ugettext
-from ipalib.plugable import ReadOnly, lock, check_name
+from ipalib.base import check_name
+from ipalib.plugable import ReadOnly, lock
 from ipalib.errors import ConversionError, RequirementError, ValidationError
 from ipalib.errors import PasswordMismatch, Base64DecodeError
 from ipalib.constants import TYPE_ERROR, CALLABLE_ERROR, LDAP_GENERALIZED_TIME_FORMAT
 from ipalib.text import Gettext, FixMe
 from ipalib.util import json_serialize, validate_idna_domain
+from ipapython import kerberos
 from ipapython.dn import DN
 from ipapython.dnsutil import DNSName
 
+
 def _is_null(value):
     return not value and value != 0 # NOTE: False == 0
 
@@ -222,7 +223,7 @@ class DefaultFrom(ReadOnly):
         lock(self)
 
     def __repr__(self):
-        args = (self.callback.__name__,) + tuple(repr(k) for k in self.keys)
+        args = tuple(repr(k) for k in self.keys)
         return '%s(%s)' % (
             self.__class__.__name__,
             ', '.join(args)
@@ -376,7 +377,7 @@ class Param(ReadOnly):
         parameter is not `required`
       - sortorder: used to sort a list of parameters for Command. See
         `Command.finalize()` for further information
-      - csv: this multivalue attribute used to be given in CSV format in CLI
+      - confirm: if password, ask for confirmation
     """
 
     # This is a dummy type so that most of the functionality of Param can be
@@ -414,8 +415,11 @@ class Param(ReadOnly):
         ('hint', (str, Gettext), None),
         ('alwaysask', bool, False),
         ('sortorder', int, 2), # see finalize()
-        ('csv', bool, False),
         ('option_group', unicode, None),
+        ('cli_metavar', str, None),
+        ('no_convert', bool, False),
+        ('deprecated', bool, False),
+        ('confirm', bool, True),
 
         # The 'default' kwarg gets appended in Param.__init__():
         # ('default', self.type, None),
@@ -427,61 +431,34 @@ class Param(ReadOnly):
         return (self.type,)
 
     def __init__(self, name, *rules, **kw):
-        # We keep these values to use in __repr__():
-        self.param_spec = name
-        self.__kw = dict(kw)
-
         # Merge in kw from parse_param_spec():
         (name, kw_from_spec) = parse_param_spec(name)
+        check_name(name)
         if not 'required' in kw:
             kw['required'] = kw_from_spec['required']
         if not 'multivalue' in kw:
             kw['multivalue'] = kw_from_spec['multivalue']
-        self.name = check_name(name)
-        self.nice = '%s(%r)' % (self.__class__.__name__, self.param_spec)
 
-        # Add 'default' to self.kwargs and makes sure no unknown kw were given:
-        assert all(type(t) is type for t in self.allowed_types)
+        # Add 'default' to self.kwargs
         if kw.get('multivalue', True):
             self.kwargs += (('default', tuple, None),)
         else:
             self.kwargs += (('default', self.type, None),)
-        if not set(t[0] for t in self.kwargs).issuperset(self.__kw):
-            extra = set(kw) - set(t[0] for t in self.kwargs)
-            raise TypeError(
-                '%s: takes no such kwargs: %s' % (self.nice,
-                    ', '.join(repr(k) for k in sorted(extra))
-                )
-            )
-
-        # Merge in default for 'cli_name', label, doc if not given:
-        if kw.get('cli_name') is None:
-            kw['cli_name'] = self.name
-
-        if kw.get('label') is None:
-            kw['label'] = FixMe(self.name)
-
-        if kw.get('doc') is None:
-            kw['doc'] = kw['label']
 
         # Wrap 'default_from' in a DefaultFrom if not already:
-        df = kw.get('default_from', None)
+        df = kw.get('default_from')
         if callable(df) and not isinstance(df, DefaultFrom):
             kw['default_from'] = DefaultFrom(df)
 
-        # We keep this copy with merged values also to use when cloning:
-        self.__clonekw = kw
-
-        # Perform type validation on kw, add in class rules:
-        class_rules = []
+        # Perform type validation on kw:
         for (key, kind, default) in self.kwargs:
-            value = kw.get(key, default)
+            value = kw.get(key)
             if value is not None:
-                if kind is frozenset:
-                    if type(value) in (list, tuple, set):
-                        value = frozenset(value)
+                if kind in (tuple, frozenset):
+                    if type(value) in (list, tuple, set, frozenset):
+                        value = kind(value)
                     elif type(value) is str:
-                        value = frozenset([value])
+                        value = kind([value])
                 if (
                     type(kind) is type and not isinstance(value, kind)
                     or
@@ -494,6 +471,58 @@ class Param(ReadOnly):
                     raise TypeError(
                         CALLABLE_ERROR % (key, value, type(value))
                     )
+                kw[key] = value
+            elif key not in ('required', 'multivalue'):
+                kw.pop(key, None)
+
+        # We keep these values to use in __repr__():
+        if kw['required']:
+            if kw['multivalue']:
+                self.param_spec = name + '+'
+            else:
+                self.param_spec = name
+        else:
+            if kw['multivalue']:
+                self.param_spec = name + '*'
+            else:
+                self.param_spec = name + '?'
+        self.__kw = dict(kw)
+        del self.__kw['required']
+        del self.__kw['multivalue']
+
+        self.name = name
+        self.nice = '%s(%r)' % (self.__class__.__name__, self.param_spec)
+
+        # Make sure no unknown kw were given:
+        assert all(type(t) is type for t in self.allowed_types)
+        if not set(t[0] for t in self.kwargs).issuperset(self.__kw):
+            extra = set(kw) - set(t[0] for t in self.kwargs)
+            raise TypeError(
+                '%s: takes no such kwargs: %s' % (self.nice,
+                    ', '.join(repr(k) for k in sorted(extra))
+                )
+            )
+
+        # We keep this copy with merged values also to use when cloning:
+        self.__clonekw = dict(kw)
+
+        # Merge in default for 'cli_name', label, doc if not given:
+        if kw.get('cli_name') is None:
+            kw['cli_name'] = self.name
+
+        if kw.get('cli_metavar') is None:
+            kw['cli_metavar'] = self.__class__.__name__.upper()
+
+        if kw.get('label') is None:
+            kw['label'] = FixMe(self.name)
+
+        if kw.get('doc') is None:
+            kw['doc'] = kw['label']
+
+        # Add in class rules:
+        class_rules = []
+        for (key, kind, default) in self.kwargs:
+            value = kw.get(key, default)
             if hasattr(self, key):
                 raise ValueError('kwarg %r conflicts with attribute on %s' % (
                     key, self.__class__.__name__)
@@ -514,10 +543,6 @@ class Param(ReadOnly):
                 )
             )
 
-        # Check that if csv is set, multivalue is set too
-        if self.csv and not self.multivalue:
-            raise ValueError('%s: cannot have csv without multivalue' % self.nice)
-
         # Check that all the rules are callable
         self.class_rules = tuple(class_rules)
         self.rules = rules
@@ -561,6 +586,8 @@ class Param(ReadOnly):
                 value = value.__name__
             elif isinstance(value, six.integer_types):
                 value = str(value)
+            elif isinstance(value, (tuple, set, frozenset)):
+                value = repr(list(value))
             else:
                 value = repr(value)
             yield '%s=%s' % (key, value)
@@ -573,10 +600,6 @@ class Param(ReadOnly):
             value = self.get_default(**kw)
         else:
             value = self.convert(self.normalize(value))
-        if hasattr(self, 'env'):
-            self.validate(value, self.env.context, supplied=self.name in kw)  #pylint: disable=E1101
-        else:
-            self.validate(value, supplied=self.name in kw)
         return value
 
     def get_param_name(self):
@@ -791,19 +814,26 @@ class Param(ReadOnly):
 
         :param value: A proposed value for this parameter.
         """
+        if not self.no_convert:
+            convert = self._convert_scalar
+        else:
+            def convert(value):
+                if isinstance(value, unicode):
+                    return value
+                return self._convert_scalar(value)
+
         if _is_null(value):
             return
         if self.multivalue:
             if type(value) not in (tuple, list):
                 value = (value,)
             values = tuple(
-                self._convert_scalar(v, i)
-                for (i, v) in enumerate(value) if not _is_null(v)
+                convert(v) for v in value if not _is_null(v)
             )
             if len(values) == 0:
                 return
             return values
-        return self._convert_scalar(value)
+        return convert(value)
 
     def _convert_scalar(self, value, index=None):
         """
@@ -811,25 +841,22 @@ class Param(ReadOnly):
         """
         if type(value) in self.allowed_types:
             return value
-        raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.type_error),
-        )
+        raise ConversionError(name=self.name, error=ugettext(self.type_error))
 
-    def validate(self, value, context=None, supplied=None):
+    def validate(self, value, supplied=None):
         """
         Check validity of ``value``.
 
         :param value: A proposed value for this parameter.
-        :param context: The context we are running in.
         :param supplied: True if this parameter was supplied explicitly.
         """
         if value is None:
             if self.required or (supplied and 'nonempty' in self.flags):
-                if context == 'cli':
-                    raise RequirementError(name=self.cli_name)
-                else:
-                    raise RequirementError(name=self.name)
+                raise RequirementError(name=self.name)
             return
+        if self.deprecated:
+            raise ValidationError(name=self.get_param_name(),
+                                  error=_('this option is deprecated'))
         if self.multivalue:
             if type(value) is not tuple:
                 raise TypeError(
@@ -837,8 +864,8 @@ class Param(ReadOnly):
                 )
             if len(value) < 1:
                 raise ValueError('value: empty tuple must be converted to None')
-            for (i, v) in enumerate(value):
-                self._validate_scalar(v, i)
+            for v in value:
+                self._validate_scalar(v)
         else:
             self._validate_scalar(value)
 
@@ -847,20 +874,10 @@ class Param(ReadOnly):
             raise TypeError(
                 TYPE_ERROR % (self.name, self.type, value, type(value))
             )
-        if index is not None and type(index) is not int:
-            raise TypeError(
-                TYPE_ERROR % ('index', int, index, type(index))
-            )
         for rule in self.all_rules:
             error = rule(ugettext, value)
             if error is not None:
-                raise ValidationError(
-                    name=self.get_param_name(),
-                    value=value,
-                    index=index,
-                    error=error,
-                    rule=rule,
-                )
+                raise ValidationError(name=self.get_param_name(), error=error)
 
     def get_default(self, **kw):
         """
@@ -976,11 +993,9 @@ class Bool(Param):
         if value in self.falsehoods:
             return False
         if type(value) in (tuple, list):
-            raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.scalar_error))
-        raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.type_error),
-        )
+            raise ConversionError(name=self.name,
+                                  error=ugettext(self.scalar_error))
+        raise ConversionError(name=self.name, error=ugettext(self.type_error))
 
 
 class Flag(Bool):
@@ -1042,11 +1057,9 @@ class Number(Param):
             except ValueError:
                 pass
         if type(value) in (tuple, list):
-            raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.scalar_error))
-        raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.type_error),
-        )
+            raise ConversionError(name=self.name,
+                                  error=ugettext(self.scalar_error))
+        raise ConversionError(name=self.name, error=ugettext(self.type_error))
 
 
 class Int(Number):
@@ -1098,7 +1111,6 @@ class Int(Number):
             return Int.convert_int(value)
         except ValueError:
             raise ConversionError(name=self.get_param_name(),
-                                  index=index,
                                   error=ugettext(self.type_error))
 
     def _rule_minvalue(self, _, value):
@@ -1245,13 +1257,13 @@ class Decimal(Number):
             try:
                 value = decimal.Decimal(value)
             except decimal.DecimalException as e:
-                raise ConversionError(name=self.get_param_name(), index=index,
+                raise ConversionError(name=self.get_param_name(),
                                       error=unicode(e))
 
         if isinstance(value, decimal.Decimal):
             return self._test_and_normalize(value)
 
-        return super(Decimal, self)._convert_scalar(value, index)
+        return super(Decimal, self)._convert_scalar(value)
 
     def _normalize_scalar(self, value):
         if isinstance(value, decimal.Decimal):
@@ -1387,7 +1399,7 @@ class Bytes(Data):
                 value = base64.b64decode(value)
             except (TypeError, ValueError) as e:
                 raise Base64DecodeError(reason=str(e))
-        return super(Bytes, self)._convert_scalar(value, index)
+        return super(Bytes, self)._convert_scalar(value)
 
 
 class Str(Data):
@@ -1427,11 +1439,9 @@ class Str(Data):
         if type(value) in (float, decimal.Decimal) + six.integer_types:
             return self.type(value)
         if type(value) in (tuple, list):
-            raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.scalar_error))
-        raise ConversionError(name=self.name, index=index,
-            error=ugettext(self.type_error),
-        )
+            raise ConversionError(name=self.name,
+                                  error=ugettext(self.scalar_error))
+        raise ConversionError(name=self.name, error=ugettext(self.type_error))
 
     def _rule_noextrawhitespace(self, _, value):
         """
@@ -1489,11 +1499,10 @@ class IA5Str(Str):
             for char in value:
                 if ord(char) > 127:
                     raise ConversionError(name=self.get_param_name(),
-                        index=index,
                         error=_('The character %(char)r is not allowed.') %
                             dict(char=char,)
                     )
-        return super(IA5Str, self)._convert_scalar(value, index)
+        return super(IA5Str, self)._convert_scalar(value)
 
 
 class Password(Str):
@@ -1503,17 +1512,13 @@ class Password(Str):
 
     password = True
 
-    kwargs = Str.kwargs + (
-        ('confirm', bool, True),
-    )
-
     def _convert_scalar(self, value, index=None):
         if isinstance(value, (tuple, list)) and len(value) == 2:
             (p1, p2) = value
             if p1 != p2:
-                raise PasswordMismatch(name=self.name, index=index)
+                raise PasswordMismatch(name=self.name)
             value = p1
-        return super(Password, self)._convert_scalar(value, index)
+        return super(Password, self)._convert_scalar(value)
 
 
 class Enum(Param):
@@ -1526,6 +1531,7 @@ class Enum(Param):
     )
 
     def __init__(self, name, *rules, **kw):
+        kw['cli_metavar'] = str([str(v) for v in kw.get('values', tuple())])
         super(Enum, self).__init__(name, *rules, **kw)
         for (i, v) in enumerate(self.values):
             if type(v) not in self.allowed_types:
@@ -1589,7 +1595,6 @@ class IntEnum(Enum):
             return Int.convert_int(value)
         except ValueError:
             raise ConversionError(name=self.get_param_name(),
-                                  index=index,
                                   error=ugettext(self.type_error))
 
 
@@ -1607,13 +1612,7 @@ class Any(Param):
         for rule in self.all_rules:
             error = rule(ugettext, value)
             if error is not None:
-                raise ValidationError(
-                    name=self.name,
-                    value=value,
-                    index=index,
-                    error=error,
-                    rule=rule,
-                )
+                raise ValidationError(name=self.name, error=error)
 
 
 class File(Str):
@@ -1672,10 +1671,9 @@ class DateTime(Param):
                       (', '.join(self.accepted_formats)))
 
             raise ConversionError(name=self.get_param_name(),
-                                  index=index,
                                   error=error)
 
-        return super(DateTime, self)._convert_scalar(value, index)
+        return super(DateTime, self)._convert_scalar(value)
 
 
 class AccessTime(Str):
@@ -1871,27 +1869,11 @@ class DNParam(Param):
         try:
             dn = DN(value)
         except Exception as e:
-            raise ConversionError(name=self.get_param_name(), index=index,
+            raise ConversionError(name=self.get_param_name(),
                                   error=ugettext(e))
         return dn
 
 
-class DeprecatedParam(Any):
-    kwargs = Param.kwargs + (
-        ('deprecate', bool, True),
-    )
-
-    def __init__(self, name, *rules, **kw):
-        if 'flags' in kw:
-            kw['flags'] = list(kw['flags']) + ['no_option']
-        else:
-            kw['flags'] = ['no_option']
-
-        super(DeprecatedParam, self).__init__(name, *rules, **kw)
-
-    def _rule_deprecate(self, _, value):
-        return _('this option is deprecated')
-
 def create_param(spec):
     """
     Create an `Str` instance from the shorthand ``spec``.
@@ -1962,14 +1944,14 @@ class DNSNameParam(Param):
             try:
                 validate_idna_domain(value)
             except ValueError as e:
-                raise ConversionError(name=self.get_param_name(), index=index,
+                raise ConversionError(name=self.get_param_name(),
                                       error=unicode(e))
             value = DNSName(value)
 
             if self.only_absolute and not value.is_absolute():
                 value = value.make_absolute()
 
-        return super(DNSNameParam, self)._convert_scalar(value, index)
+        return super(DNSNameParam, self)._convert_scalar(value)
 
     def _rule_only_absolute(self, _, value):
         if self.only_absolute and not value.is_absolute():
@@ -1978,3 +1960,47 @@ class DNSNameParam(Param):
     def _rule_only_relative(self, _, value):
         if self.only_relative and value.is_absolute():
             return _('must be relative')
+
+
+class Dict(Param):
+    """
+    A parameter for dictionary.
+    """
+
+    type = dict
+    type_error = _("must be dictionary")
+
+
+class Principal(Param):
+    """
+    Kerberos principal name
+    """
+
+    type = kerberos.Principal
+    type_error = _('must be Kerberos principal')
+    kwargs = Param.kwargs + (
+        ('require_service', bool, False),
+    )
+
+    @property
+    def allowed_types(self):
+        return (self.type, unicode)
+
+    def _convert_scalar(self, value, index=None):
+        if isinstance(value, unicode):
+            try:
+                value = kerberos.Principal(value)
+            except ValueError:
+                raise ConversionError(
+                    name=self.get_param_name(),
+                    error=_("Malformed principal: '%(value)s'") % dict(
+                        value=value))
+
+        return super(Principal, self)._convert_scalar(value)
+
+    def _rule_require_service(self, _, value):
+        if self.require_service and not value.is_service:
+            raise ValidationError(
+                name=self.get_param_name(),
+                error=_("Service principal is required")
+            )
