@@ -25,9 +25,9 @@ you are unfamiliar with this Python feature, see
 http://docs.python.org/ref/sequence-types.html
 """
 
-import re
+from distutils.version import LooseVersion
+import operator
 import sys
-import inspect
 import threading
 import os
 from os import path
@@ -40,12 +40,15 @@ import six
 
 from ipalib import errors
 from ipalib.config import Env
-from ipalib import text
 from ipalib.text import _
-from ipalib.base import ReadOnly, NameSpace, lock, islocked, check_name
+from ipalib.util import classproperty
+from ipalib.base import ReadOnly, lock, islocked
 from ipalib.constants import DEFAULT_CONFIG
-from ipapython.ipa_log_manager import *
-from ipapython.version import VERSION, API_VERSION
+from ipapython.ipa_log_manager import (
+    log_mgr,
+    LOGGING_FORMAT_FILE,
+    LOGGING_FORMAT_STDERR)
+from ipapython.version import VERSION, API_VERSION, DEFAULT_PLUGINS
 
 if six.PY3:
     unicode = str
@@ -90,12 +93,32 @@ class Registry(object):
     For forward compatibility, make sure that the module-level instance of
     this object is named "register".
     """
-    def __call__(self):
-        def decorator(cls):
-            API.register(cls)
-            return cls
+    def __init__(self):
+        self.__registry = collections.OrderedDict()
 
-        return decorator
+    def __call__(self, **kwargs):
+        def register(plugin):
+            """
+            Register the plugin ``plugin``.
+
+            :param plugin: A subclass of `Plugin` to attempt to register.
+            """
+            if not callable(plugin):
+                raise TypeError('plugin must be callable; got %r' % plugin)
+
+            # Raise DuplicateError if this exact class was already registered:
+            if plugin in self.__registry:
+                raise errors.PluginDuplicateError(plugin=plugin)
+
+            # The plugin is okay, add to __registry:
+            self.__registry[plugin] = dict(kwargs, plugin=plugin)
+
+            return plugin
+
+        return register
+
+    def __iter__(self):
+        return iter(self.__registry.values())
 
 
 class Plugin(ReadOnly):
@@ -103,9 +126,7 @@ class Plugin(ReadOnly):
     Base class for all plugins.
     """
 
-    finalize_early = True
-
-    label = None
+    version = '1'
 
     def __init__(self, api):
         assert api is not None
@@ -113,30 +134,42 @@ class Plugin(ReadOnly):
         self.__finalize_called = False
         self.__finalized = False
         self.__finalize_lock = threading.RLock()
-        cls = self.__class__
-        self.name = cls.__name__
-        self.module = cls.__module__
-        self.fullname = '%s.%s' % (self.module, self.name)
-        self.bases = tuple(
-            '%s.%s' % (b.__module__, b.__name__) for b in cls.__bases__
-        )
-        self.doc = _(cls.__doc__)
-        if not self.doc.msg:
-            self.summary = '<%s>' % self.fullname
-        else:
-            self.summary = unicode(self.doc).split('\n\n', 1)[0].strip()
         log_mgr.get_logger(self, True)
-        if self.label is None:
-            self.label = text.FixMe(self.name + '.label')
-        if not isinstance(self.label, text.LazyText):
-            raise TypeError(
-                TYPE_ERROR % (
-                    self.fullname + '.label',
-                    text.LazyText,
-                    type(self.label),
-                    self.label
-                )
-            )
+
+    @classmethod
+    def __name_getter(cls):
+        return cls.__name__
+
+    # you know nothing, pylint
+    name = classproperty(__name_getter)
+
+    @classmethod
+    def __full_name_getter(cls):
+        return '{}/{}'.format(cls.name, cls.version)
+
+    full_name = classproperty(__full_name_getter)
+
+    @classmethod
+    def __bases_getter(cls):
+        return cls.__bases__
+
+    bases = classproperty(__bases_getter)
+
+    @classmethod
+    def __doc_getter(cls):
+        return cls.__doc__
+
+    doc = classproperty(__doc_getter)
+
+    @classmethod
+    def __summary_getter(cls):
+        doc = cls.doc
+        if not _(doc).msg:
+            return u'<%s.%s>' % (cls.__module__, cls.__name__)
+        else:
+            return unicode(doc).split('\n\n', 1)[0].strip()
+
+    summary = classproperty(__summary_getter)
 
     @property
     def api(self):
@@ -246,44 +279,61 @@ class Plugin(ReadOnly):
         )
 
 
-class Registrar(collections.Mapping):
-    """
-    Collects plugin classes as they are registered.
+class APINameSpace(collections.Mapping):
+    def __init__(self, api, base):
+        self.__api = api
+        self.__base = base
+        self.__plugins = None
+        self.__plugins_by_key = None
 
-    The Registrar does not instantiate plugins... it only implements the
-    override logic and stores the plugins in a namespace per allowed base
-    class.
+    def __enumerate(self):
+        if self.__plugins is not None and self.__plugins_by_key is not None:
+            return
 
-    The plugins are instantiated when `API.finalize()` is called.
-    """
-    def __init__(self):
-        self.__registry = collections.OrderedDict()
+        default_map = self.__api._API__default_map
+        plugins = set()
+        key_dict = self.__plugins_by_key = {}
 
-    def __call__(self, klass, override=False):
-        """
-        Register the plugin ``klass``.
+        for plugin in self.__api._API__plugins:
+            if not any(issubclass(b, self.__base) for b in plugin.bases):
+                continue
+            plugins.add(plugin)
+            key_dict[plugin] = plugin
+            key_dict[plugin.name, plugin.version] = plugin
+            key_dict[plugin.full_name] = plugin
+            if plugin.version == default_map.get(plugin.name, '1'):
+                key_dict[plugin.name] = plugin
 
-        :param klass: A subclass of `Plugin` to attempt to register.
-        :param override: If true, override an already registered plugin.
-        """
-        if not inspect.isclass(klass):
-            raise TypeError('plugin must be a class; got %r' % klass)
-
-        # Raise DuplicateError if this exact class was already registered:
-        if klass in self.__registry:
-            raise errors.PluginDuplicateError(plugin=klass)
-
-        # The plugin is okay, add to __registry:
-        self.__registry[klass] = dict(override=override)
-
-    def __getitem__(self, key):
-        return self.__registry[key]
-
-    def __iter__(self):
-        return iter(self.__registry)
+        self.__plugins = sorted(plugins, key=operator.attrgetter('full_name'))
 
     def __len__(self):
-        return len(self.__registry)
+        self.__enumerate()
+        return len(self.__plugins)
+
+    def __contains__(self, key):
+        self.__enumerate()
+        return key in self.__plugins_by_key
+
+    def __iter__(self):
+        self.__enumerate()
+        return iter(self.__plugins)
+
+    def get_plugin(self, key):
+        self.__enumerate()
+        return self.__plugins_by_key[key]
+
+    def __getitem__(self, key):
+        plugin = self.get_plugin(key)
+        return self.__api._get(plugin)
+
+    def __call__(self):
+        return six.itervalues(self)
+
+    def __getattr__(self, key):
+        try:
+            return self[key]
+        except KeyError:
+            raise AttributeError(key)
 
 
 class API(ReadOnly):
@@ -291,11 +341,13 @@ class API(ReadOnly):
     Dynamic API object through which `Plugin` instances are accessed.
     """
 
-    register = Registrar()
-
     def __init__(self):
         super(API, self).__init__()
-        self.__plugins = {}
+        self.__plugins = set()
+        self.__plugins_by_key = {}
+        self.__default_map = {}
+        self.__instances = {}
+        self.__next = {}
         self.__done = set()
         self.env = Env()
 
@@ -304,7 +356,7 @@ class API(ReadOnly):
         raise NotImplementedError
 
     @property
-    def modules(self):
+    def packages(self):
         raise NotImplementedError
 
     def __len__(self):
@@ -530,41 +582,30 @@ class API(ReadOnly):
         self.__do_if_not_done('bootstrap')
         if self.env.mode in ('dummy', 'unit_test'):
             return
-        for module in self.modules:
-            self.import_plugins(module)
+        for package in self.packages:
+            self.add_package(package)
 
     # FIXME: This method has no unit test
-    def import_plugins(self, module):
+    def add_package(self, package):
         """
-        Import plugins from ``module``.
+        Add plugin modules from the ``package``.
 
-        :param module: Name of the module to import. This might be a wildcard
-                       in the form ```package.*``` in which case all modules
-                       from the given package are loaded.
+        :param package: A package from which to add modules.
         """
-        if module.endswith('.*'):
-            subpackage = module[:-2]
-            try:
-                plugins = importlib.import_module(subpackage)
-            except ImportError as e:
-                self.log.error("cannot import plugins sub-package %s: %s",
-                               subpackage, e)
-                raise
-            package, dot, part = subpackage.rpartition('.')
-            parent = sys.modules[package]
+        package_name = package.__name__
+        package_file = package.__file__
+        package_dir = path.dirname(path.abspath(package_file))
 
-            parent_dir = path.dirname(path.abspath(parent.__file__))
-            plugins_dir = path.dirname(path.abspath(plugins.__file__))
-            if parent_dir == plugins_dir:
-                raise errors.PluginsPackageError(
-                    name=subpackage, file=plugins.__file__
-                )
+        parent = sys.modules[package_name.rpartition('.')[0]]
+        parent_dir = path.dirname(path.abspath(parent.__file__))
+        if parent_dir == package_dir:
+            raise errors.PluginsPackageError(
+                name=package_name, file=package_file
+            )
 
-            self.log.debug("importing all plugin modules in %s...", subpackage)
-            modules = find_modules_in_dir(plugins_dir)
-            modules = ['.'.join((subpackage, name)) for name in modules]
-        else:
-            modules = [module]
+        self.log.debug("importing all plugin modules in %s...", package_name)
+        modules = getattr(package, 'modules', find_modules_in_dir(package_dir))
+        modules = ['.'.join((package_name, name)) for name in modules]
 
         for name in modules:
             self.log.debug("importing plugin module %s", name)
@@ -572,14 +613,18 @@ class API(ReadOnly):
                 module = importlib.import_module(name)
             except errors.SkipPluginModule as e:
                 self.log.debug("skipping plugin module %s: %s", name, e.reason)
+                continue
             except Exception as e:
                 if self.env.startup_traceback:
                     import traceback
                     self.log.error("could not load plugin module %s\n%s", name,
                                    traceback.format_exc())
                 raise
-            else:
+
+            try:
                 self.add_module(module)
+            except errors.PluginModuleError as e:
+                self.log.debug("%s", e)
 
     def add_module(self, module):
         """
@@ -587,63 +632,69 @@ class API(ReadOnly):
 
         :param module: A module from which to add plugins.
         """
-        for name in dir(module):
-            klass = getattr(module, name)
-            if not inspect.isclass(klass):
-                continue
-            if klass not in self.register:
-                continue
-            kwargs = self.register[klass]
-            self.add_plugin(klass, **kwargs)
+        try:
+            register = module.register
+        except AttributeError:
+            pass
+        else:
+            if isinstance(register, Registry):
+                for kwargs in register:
+                    self.add_plugin(**kwargs)
+                return
 
-    def add_plugin(self, klass, override=False):
+        raise errors.PluginModuleError(name=module.__name__)
+
+    def add_plugin(self, plugin, override=False, no_fail=False):
         """
-        Add the plugin ``klass``.
+        Add the plugin ``plugin``.
 
-        :param klass: A subclass of `Plugin` to attempt to add.
+        :param plugin: A subclass of `Plugin` to attempt to add.
         :param override: If true, override an already added plugin.
         """
-        if not inspect.isclass(klass):
-            raise TypeError('plugin must be a class; got %r' % klass)
+        if not callable(plugin):
+            raise TypeError('plugin must be callable; got %r' % plugin)
 
         # Find the base class or raise SubclassError:
-        found = False
-        for base in self.bases:
-            if not issubclass(klass, base):
-                continue
+        for base in plugin.bases:
+            if issubclass(base, self.bases):
+                break
+        else:
+            raise errors.PluginSubclassError(
+                plugin=plugin,
+                bases=self.bases,
+            )
 
-            sub_d = self.__plugins.setdefault(base, {})
-            found = True
-
-            if sub_d.get(klass.__name__) is klass:
-                continue
-
-            # Check override:
-            if klass.__name__ in sub_d:
-                if not override:
+        # Check override:
+        prev = self.__plugins_by_key.get(plugin.full_name)
+        if prev:
+            if not override:
+                if no_fail:
+                    return
+                else:
                     # Must use override=True to override:
                     raise errors.PluginOverrideError(
                         base=base.__name__,
-                        name=klass.__name__,
-                        plugin=klass,
+                        name=plugin.name,
+                        plugin=plugin,
                     )
-            else:
-                if override:
+
+            self.__plugins.remove(prev)
+            self.__next[plugin] = prev
+        else:
+            if override:
+                if no_fail:
+                    return
+                else:
                     # There was nothing already registered to override:
                     raise errors.PluginMissingOverrideError(
                         base=base.__name__,
-                        name=klass.__name__,
-                        plugin=klass,
+                        name=plugin.name,
+                        plugin=plugin,
                     )
 
-            # The plugin is okay, add to sub_d:
-            sub_d[klass.__name__] = klass
-
-        if not found:
-            raise errors.PluginSubclassError(
-                plugin=klass,
-                bases=self.bases,
-            )
+        # The plugin is okay, add to sub_d:
+        self.__plugins.add(plugin)
+        self.__plugins_by_key[plugin.full_name] = plugin
 
     def finalize(self):
         """
@@ -655,43 +706,68 @@ class API(ReadOnly):
         self.__doing('finalize')
         self.__do_if_not_done('load_plugins')
 
+        for plugin in self.__plugins:
+            if not self.env.validate_api:
+                if plugin.full_name not in DEFAULT_PLUGINS:
+                    continue
+            else:
+                try:
+                    default_version = self.__default_map[plugin.name]
+                except KeyError:
+                    pass
+                else:
+                    version = LooseVersion(plugin.version)
+                    default_version = LooseVersion(default_version)
+                    if version < default_version:
+                        continue
+            self.__default_map[plugin.name] = plugin.version
+
         production_mode = self.is_production_mode()
-        plugins = {}
-        plugin_info = {}
 
         for base in self.bases:
+            for plugin in self.__plugins:
+                if not any(issubclass(b, base) for b in plugin.bases):
+                    continue
+                if not self.env.plugins_on_demand:
+                    self._get(plugin)
+
             name = base.__name__
-            sub_d = self.__plugins.get(base, {})
-
-            members = []
-            for klass in sub_d.values():
-                try:
-                    instance = plugins[klass]
-                except KeyError:
-                    instance = plugins[klass] = klass(self)
-                members.append(instance)
-                plugin_info.setdefault(
-                    '%s.%s' % (klass.__module__, klass.__name__),
-                    []).append(name)
-
             if not production_mode:
                 assert not hasattr(self, name)
-            setattr(self, name, NameSpace(members))
+            setattr(self, name, APINameSpace(self, base))
 
-        for klass, instance in plugins.items():
+        for instance in six.itervalues(self.__instances):
             if not production_mode:
                 assert instance.api is self
-            if klass.finalize_early or not self.env.plugins_on_demand:
+            if not self.env.plugins_on_demand:
                 instance.ensure_finalized()
                 if not production_mode:
                     assert islocked(instance)
 
         self.__finalized = True
-        self.plugins = tuple((k, tuple(v)) for k, v in plugin_info.items())
 
         if not production_mode:
             lock(self)
 
+    def _get(self, plugin):
+        if not callable(plugin):
+            raise TypeError('plugin must be callable; got %r' % plugin)
+        if plugin not in self.__plugins:
+            raise KeyError(plugin)
+
+        try:
+            instance = self.__instances[plugin]
+        except KeyError:
+            instance = self.__instances[plugin] = plugin(self)
+
+        return instance
+
+    def get_plugin_next(self, plugin):
+        if not callable(plugin):
+            raise TypeError('plugin must be callable; got %r' % plugin)
+
+        return self.__next[plugin]
+
 
 class IPAHelpFormatter(optparse.IndentedHelpFormatter):
     def format_epilog(self, text):
