@@ -30,8 +30,9 @@ For documentation on the ``xmlrpclib`` module, see:
 Also see the `ipaserver.rpcserver` module.
 """
 
+from __future__ import absolute_import
+
 from decimal import Decimal
-import sys
 import datetime
 import os
 import locale
@@ -56,7 +57,6 @@ from ipalib.request import context, Connection
 from ipapython.ipa_log_manager import root_logger
 from ipapython import ipautil
 from ipapython import kernel_keyring
-from ipaplatform.paths import paths
 from ipapython.cookie import Cookie
 from ipapython.dnsutil import DNSName
 from ipalib.text import _
@@ -66,6 +66,7 @@ from ipalib.krb_utils import KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN, KRB5KRB_AP_ERR_TKT
                              KRB5_FCC_PERM, KRB5_FCC_NOFILE, KRB5_CC_FORMAT, \
                              KRB5_REALM_CANT_RESOLVE, KRB5_CC_NOTFOUND, get_principal
 from ipapython.dn import DN
+from ipapython.kerberos import Principal
 from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
 from ipalib import api
 
@@ -194,6 +195,9 @@ def xml_wrap(value, version):
         else:
             return unicode(value)
 
+    if isinstance(value, Principal):
+        return unicode(value)
+
     assert type(value) in (unicode, float, bool, type(None)) + six.integer_types
     return value
 
@@ -316,6 +320,8 @@ def json_encode_binary(val, version):
             return {'__dns_name__': unicode(val)}
         else:
             return unicode(val)
+    elif isinstance(val, Principal):
+        return unicode(val)
     else:
         return val
 
@@ -404,13 +410,13 @@ def xml_loads(data, encoding='UTF-8'):
 
 class DummyParser(object):
     def __init__(self):
-        self.data = b''
+        self.data = []
 
     def feed(self, data):
-        self.data += data
+        self.data.append(data)
 
     def close(self):
-        return self.data
+        return b''.join(self.data)
 
 
 class MultiProtocolTransport(Transport):
@@ -488,7 +494,7 @@ class SSLTransport(LanguageAwareTransport):
         if self._connection and host == self._connection[0]:
             return self._connection[1]
 
-        dbdir = getattr(context, 'nss_dir', paths.IPA_NSSDB_DIR)
+        dbdir = context.nss_dir
         connection_dbdir = self.get_connection_dbdir()
 
         if connection_dbdir:
@@ -693,8 +699,11 @@ class KerbTransport(SSLTransport):
 
         # Search for the session cookie
         try:
-            session_cookie = Cookie.get_named_cookie_from_string(cookie_header,
-                                                                 COOKIE_NAME, request_url)
+            session_cookie = (
+                    Cookie.get_named_cookie_from_string(
+                        cookie_header, COOKIE_NAME, request_url,
+                        timestamp=datetime.datetime.utcnow())
+            )
         except Exception as e:
             root_logger.error("unable to parse cookie header '%s': %s", cookie_header, e)
             return
@@ -788,8 +797,13 @@ class RPCClient(Connectible):
 
         # Search for the session cookie within the cookie string
         try:
-            session_cookie = Cookie.get_named_cookie_from_string(cookie_string, COOKIE_NAME)
+            session_cookie = Cookie.get_named_cookie_from_string(
+                cookie_string, COOKIE_NAME,
+                timestamp=datetime.datetime.utcnow())
         except Exception as e:
+            self.log.debug(
+                'Error retrieving cookie from the persistent storage: {err}'
+                .format(err=e))
             return None
 
         return session_cookie
@@ -860,8 +874,16 @@ class RPCClient(Connectible):
 
         return session_url
 
-    def create_connection(self, ccache=None, verbose=0, fallback=True,
-                          delegate=False, nss_dir=None):
+    def create_connection(self, ccache=None, verbose=None, fallback=None,
+                          delegate=None, nss_dir=None):
+        if verbose is None:
+            verbose = self.api.env.verbose
+        if fallback is None:
+            fallback = self.api.env.fallback
+        if delegate is None:
+            delegate = self.api.env.delegate
+        if nss_dir is None:
+            nss_dir = self.api.env.nss_dir
         try:
             rpc_uri = self.env[self.env_rpc_uri_key]
             principal = get_principal()
@@ -873,9 +895,7 @@ class RPCClient(Connectible):
         except (errors.CCacheError, ValueError):
             # No session key, do full Kerberos auth
             pass
-        # This might be dangerous. Use at your own risk!
-        if nss_dir:
-            context.nss_dir = nss_dir
+        context.nss_dir = nss_dir
         urls = self.get_url_list(rpc_uri)
         serverproxy = None
         for url in urls:
@@ -965,10 +985,6 @@ class RPCClient(Connectible):
         :param args: Positional arguments to pass to remote command.
         :param kw: Keyword arguments to pass to remote command.
         """
-        if name not in self.Command:
-            raise ValueError(
-                '%s.forward(): %r not in api.Command' % (self.name, name)
-            )
         server = getattr(context, 'request_url', None)
         self.log.info("Forwarding '%s' to %s server '%s'",
                       name, self.protocol, server)
@@ -1079,7 +1095,7 @@ class JSONServerProxy(object):
         try:
             response = json_decode_binary(json.loads(response.decode('ascii')))
         except ValueError as e:
-            raise JSONError(str(e))
+            raise JSONError(error=str(e))
 
         if self.__verbose >= 2:
             root_logger.info(
@@ -1098,7 +1114,9 @@ class JSONServerProxy(object):
                     server=self.__host,
                 )
             else:
-                raise error_class(message=error['message'])
+                kw = error.get('data', {})
+                kw['message'] = error['message']
+                raise error_class(**kw)
 
         return response['result']
 
