@@ -6,17 +6,12 @@ from __future__ import print_function
 
 from binascii import hexlify
 import collections
-import logging
 from pprint import pprint
-import sys
-import time
 
 import ipalib
 from ipapython.dn import DN
 from ipapython import ipaldap
-from ipapython import ipautil
 from ipapython import ipa_log_manager
-from ipaplatform.paths import paths
 
 from ipapython.dnssec.abshsm import (
     attrs_name2id,
