@@ -30,10 +30,10 @@ import re
 
 import six
 
+from ipaserver.dns_data_management import IPASystemRecords
 from ipaserver.install import service
 from ipaserver.install import installutils
-from ipaserver.install.bindinstance import get_rr, add_rr, del_rr, \
-                                           dns_zone_exists
+from ipaserver.install.bindinstance import dns_zone_exists
 from ipaserver.install.replication import wait_for_task
 from ipalib import errors, api
 from ipalib.util import normalize_zone
@@ -81,6 +81,8 @@ def ipa_smb_conf_exists():
     except IOError as err:
         if err.errno == errno.ENOENT:
             return False
+        else:
+            raise
 
     lines = conf_fd.readlines()
     conf_fd.close()
@@ -92,7 +94,7 @@ def ipa_smb_conf_exists():
 
 def check_netbios_name(name):
     # Empty NetBIOS name is not allowed
-    if name is None:
+    if not name:
         return False
 
     # NetBIOS names may not be longer than 15 allowed characters
@@ -122,7 +124,6 @@ class ADTRUSTInstance(service.Service):
     def __init__(self, fstore=None):
         self.netbios_name = None
         self.reset_netbios_name = None
-        self.no_msdcs = None
         self.add_sids = None
         self.smbd_user = None
         self.smb_dn_pwd = None
@@ -135,7 +136,6 @@ class ADTRUSTInstance(service.Service):
         self.fqdn = None
         self.host_netbios_name = None
         self.realm = None
-        self.domain_name = None
 
         service.Service.__init__(self, "smb", service_desc="CIFS",
                                  dm_password=None, ldapi=True)
@@ -162,7 +162,6 @@ class ADTRUSTInstance(service.Service):
         self.fqdn = self.fqdn or api.env.host
         self.host_netbios_name = make_netbios_name(self.fqdn)
         self.realm = self.realm or api.env.realm
-        self.domain_name = self.domain_name or api.env.domain
 
         self.cifs_principal = "cifs/" + self.fqdn + "@" + self.realm
         self.suffix = ipautil.realm_to_suffix(self.realm)
@@ -177,7 +176,7 @@ class ADTRUSTInstance(service.Service):
                          ('cn', 'etc'),
                          self.suffix)
 
-        self.smb_dom_dn = DN(('cn', self.domain_name),
+        self.smb_dom_dn = DN(('cn', api.env.domain),
                              api.env.container_cifsdomains,
                              self.suffix)
 
@@ -426,7 +425,7 @@ class ADTRUSTInstance(service.Service):
             self.smb_dom_dn,
             {
                 'objectclass': [self.OBJC_DOMAIN, "nsContainer"],
-                'cn': [self.domain_name],
+                'cn': [api.env.domain],
                 self.ATTR_FLAT_NAME: [self.netbios_name],
                 self.ATTR_SID: [self.__gen_sid_string()],
                 self.ATTR_GUID: [str(uuid.uuid4())],
@@ -581,58 +580,32 @@ class ADTRUSTInstance(service.Service):
         their values are used. Otherwise default values are used.
         """
 
-        zone = self.domain_name
-        host_in_rr = normalize_zone(self.fqdn)
-
-        priority = 0
-
-        ipa_srv_rec = (
-            ("_ldap._tcp", [self.srv_rec(host_in_rr, 389, priority)], 389),
-            ("_kerberos._tcp", [self.srv_rec(host_in_rr, 88, priority)], 88),
-            ("_kerberos._udp", [self.srv_rec(host_in_rr, 88, priority)], 88),
-        )
-        win_srv_suffix = (".Default-First-Site-Name._sites.dc._msdcs",
-                          ".dc._msdcs")
+        zone = api.env.domain
 
         err_msg = None
 
-        if self.no_msdcs:
-            err_msg = '--no-msdcs was given, special DNS service records ' \
-                      'are not added to local DNS server'
+        ret = api.Command['dns_is_enabled']()
+        if not ret['result']:
+            err_msg = "DNS management was not enabled at install time."
         else:
-            ret = api.Command['dns_is_enabled']()
-            if not ret['result']:
-                err_msg = "DNS management was not enabled at install time."
-            else:
-                if not dns_zone_exists(zone):
-                    err_msg = "DNS zone %s cannot be managed " \
-                              "as it is not defined in IPA" % zone
+            if not dns_zone_exists(zone):
+                err_msg = (
+                    "DNS zone %s cannot be managed as it is not defined in "
+                    "IPA" % zone)
 
         if err_msg:
             self.print_msg(err_msg)
             self.print_msg("Add the following service records to your DNS " \
                            "server for DNS zone %s: " % zone)
-            for suff in win_srv_suffix:
-                for srv in ipa_srv_rec:
-                    self.print_msg("%s%s IN SRV %s"  % (srv[0], suff, " ".join(srv[1])))
-            self.print_msg("")
-            return
-
-        for (srv, rdata, port) in ipa_srv_rec:
-            cifs_rdata = list()
-            for fqdn in self.cifs_hosts:
-                cifs_srv = self.srv_rec(fqdn, port, priority)
-                cifs_rdata.append(cifs_srv)
-            cifs_rdata.extend(rdata)
-
-            for suff in win_srv_suffix:
-                win_srv = srv+suff
-                win_rdata = get_rr(zone, win_srv, "SRV")
-                if win_rdata:
-                    for rec in win_rdata:
-                        del_rr(zone, win_srv, "SRV", rec)
-                for rec in cifs_rdata:
-                    add_rr(zone, win_srv, "SRV", rec)
+            system_records = IPASystemRecords(api)
+            adtrust_records = system_records.get_base_records(
+                [self.fqdn], ["AD trust controller"],
+                include_master_role=False, include_kerberos_realm=False)
+            for r_name, node in adtrust_records.items():
+                for rec in IPASystemRecords.records_list_from_node(r_name, node):
+                    self.print_msg(rec)
+        else:
+            api.Command.dns_update_system_records()
 
     def __configure_selinux_for_smbd(self):
         try:
@@ -787,17 +760,16 @@ class ADTRUSTInstance(service.Service):
                              LDAPI_SOCKET = self.ldapi_socket,
                              FQDN = self.fqdn)
 
-    def setup(self, fqdn, realm_name, domain_name, netbios_name,
+    def setup(self, fqdn, realm_name, netbios_name,
               reset_netbios_name, rid_base, secondary_rid_base,
-              no_msdcs=False, add_sids=False, smbd_user="samba", enable_compat=False):
+              add_sids=False, smbd_user="samba",
+              enable_compat=False):
         self.fqdn = fqdn
         self.realm = realm_name
-        self.domain_name = domain_name
         self.netbios_name = netbios_name
         self.reset_netbios_name = reset_netbios_name
         self.rid_base = rid_base
         self.secondary_rid_base = secondary_rid_base
-        self.no_msdcs = no_msdcs
         self.add_sids = add_sids
         self.enable_compat = enable_compat
         self.smbd_user = smbd_user
