@@ -42,7 +42,6 @@ from ipapython.dn import DN
 from ipalib import pkcs10, x509, api
 from ipalib.errors import CertificateOperationError
 from ipalib.text import _
-from ipaplatform import services
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 
@@ -128,7 +127,7 @@ class CertDB(object):
             shutil.rmtree(self.reqdir, ignore_errors=True)
         try:
             os.chdir(self.cwd)
-        except:
+        except OSError:
             pass
 
     def setup_cert_request(self):
@@ -333,7 +332,7 @@ class CertDB(object):
             cdb = self
         if subject is None:
             subject=DN(('CN', hostname), self.subject_base)
-        self.request_cert(subject)
+        self.request_cert(subject, san_dnsnames=[hostname])
         cdb.issue_server_cert(self.certreq_fname, self.certder_fname)
         self.import_cert(self.certder_fname, nickname)
         fd = open(self.certder_fname, "r")
@@ -357,7 +356,9 @@ class CertDB(object):
         os.unlink(self.certreq_fname)
         os.unlink(self.certder_fname)
 
-    def request_cert(self, subject, certtype="rsa", keysize="2048"):
+    def request_cert(
+            self, subject, certtype="rsa", keysize="2048",
+            san_dnsnames=None):
         assert isinstance(subject, DN)
         self.create_noise_file()
         self.setup_cert_request()
@@ -368,6 +369,8 @@ class CertDB(object):
                 "-z", self.noise_fname,
                 "-f", self.passwd_fname,
                 "-a"]
+        if san_dnsnames is not None and len(san_dnsnames) > 0:
+            args += ['-8', ','.join(san_dnsnames)]
         result = self.run_certutil(args,
                                    capture_output=True, capture_error=True)
         os.remove(self.noise_fname)
@@ -401,6 +404,7 @@ class CertDB(object):
             self.host_name, 8443, "/ca/ee/ca/profileSubmitSSLClient",
             self.secdir, password, "ipaCert", **params)
         http_status, http_headers, http_body = result
+        root_logger.debug("CA answer: %s", http_body)
 
         if http_status != 200:
             raise CertificateOperationError(
