@@ -17,7 +17,7 @@ from ipapython import p11helper as _ipap11helper
 from ipapython.dnsutil import DNSName
 from ipaserver.install import service
 from ipaserver.install import installutils
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 from ipapython.dn import DN
 from ipapython import ipaldap
 from ipapython import sysrestore, ipautil
@@ -56,6 +56,12 @@ def dnssec_container_exists(fqdn, suffix, dm_password=None, ldapi=False,
 
     return ret
 
+
+def remove_replica_public_keys(hostname):
+    keysyncd = DNSKeySyncInstance()
+    keysyncd.remove_replica_public_keys(hostname)
+
+
 class DNSKeySyncInstance(service.Service):
     def __init__(self, fstore=None, dm_password=None, logger=root_logger,
                  ldapi=False, start_tls=False):
@@ -123,7 +129,7 @@ class DNSKeySyncInstance(service.Service):
         self.suffix = ipautil.realm_to_suffix(self.realm)
         try:
             self.stop()
-        except:
+        except Exception:
             pass
 
         # get a connection to the DS
@@ -492,7 +498,6 @@ class DNSKeySyncInstance(service.Service):
                 self.fstore.restore_file(f)
             except ValueError as error:
                 self.logger.debug(error)
-                pass
 
         # remove softhsm pin, to make sure new installation will generate
         # new token database
