@@ -39,7 +39,6 @@ from dns import resolver, rdatatype
 from dns.exception import DNSException
 import ldap
 import ldapurl
-from nss.error import NSPRError
 import six
 from six.moves.configparser import SafeConfigParser, NoOptionError
 
@@ -47,17 +46,15 @@ import ipaplatform
 
 from ipapython import ipautil, sysrestore, admintool, version
 from ipapython.admintool import ScriptError
-from ipapython.ipa_log_manager import root_logger, log_mgr
+from ipapython.ipa_log_manager import root_logger
 from ipalib.util import validate_hostname
 from ipapython import config
-from ipalib import api, errors, x509
+from ipalib import errors, x509
 from ipapython.dn import DN
 from ipaserver.install import certs, service, sysupgrade
 from ipaplatform import services
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks
-from ipapython import certmonger
-from ipapython import dnsutil
 
 if six.PY3:
     unicode = str
@@ -123,10 +120,10 @@ def get_fqdn():
     fqdn = ""
     try:
         fqdn = socket.getfqdn()
-    except:
+    except Exception:
         try:
             fqdn = socket.gethostname()
-        except:
+        except Exception:
             fqdn = ""
     return fqdn
 
@@ -378,13 +375,35 @@ def update_file(filename, orig, subst):
         print("File %s doesn't exist." % filename)
         return 1
 
-def set_directive(filename, directive, value, quotes=True, separator=' '):
+
+def set_directive(filename, directive, value, quotes=True, separator=' ',
+                  quote_char='\"'):
     """Set a name/value pair directive in a configuration file.
 
-       A value of None means to drop the directive.
+    A value of None means to drop the directive.
 
-       This has only been tested with nss.conf
+    This has only been tested with nss.conf
+
+   :param directive: directive name
+   :param value: value of the directive
+   :param quotes: whether to quote `value` in `quote_char`. If true, then
+        the `quote_char` are first escaped to avoid unparseable directives
+   :param quote_char: the character used for quoting `value`
     """
+
+    def format_directive(directive, value, separator, quotes, quote_char):
+        directive_sep = "{directive}{separator}".format(directive=directive,
+                                                        separator=separator)
+        transformed_value = value
+        if quotes:
+            transformed_value = "{quote}{value}{quote}".format(
+                quote=quote_char,
+                value="".join(ipautil.escape_seq(quote_char, value))
+            )
+
+        return "{directive_sep}{value}\n".format(
+            directive_sep=directive_sep, value=transformed_value)
+
     valueset = False
     st = os.stat(filename)
     fd = open(filename)
@@ -393,19 +412,17 @@ def set_directive(filename, directive, value, quotes=True, separator=' '):
         if line.lstrip().startswith(directive):
             valueset = True
             if value is not None:
-                if quotes:
-                    newfile.append('%s%s"%s"\n' % (directive, separator, value))
-                else:
-                    newfile.append('%s%s%s\n' % (directive, separator, value))
+                newfile.append(
+                    format_directive(
+                        directive, value, separator, quotes, quote_char))
         else:
             newfile.append(line)
     fd.close()
     if not valueset:
         if value is not None:
-            if quotes:
-                newfile.append('%s%s"%s"\n' % (directive, separator, value))
-            else:
-                newfile.append('%s%s%s\n' % (directive, separator, value))
+            newfile.append(
+                format_directive(
+                    directive, value, separator, quotes, quote_char))
 
     fd = open(filename, "w")
     fd.write("".join(newfile))
@@ -429,14 +446,17 @@ def get_directive(filename, directive, separator=' '):
     return None
 
 def kadmin(command):
-    ipautil.run(["kadmin.local", "-q", command,
-                                 "-x", "ipa-setup-override-restrictions"])
+    return ipautil.run(["kadmin.local", "-q", command,
+                        "-x", "ipa-setup-override-restrictions"],
+                       capture_output=True,
+                       capture_error=True)
+
 
 def kadmin_addprinc(principal):
-    kadmin("addprinc -randkey " + principal)
+    return kadmin("addprinc -randkey " + principal)
 
 def kadmin_modprinc(principal, options):
-    kadmin("modprinc " + options + " " + principal)
+    return kadmin("modprinc " + options + " " + principal)
 
 def create_keytab(path, principal):
     try:
@@ -445,16 +465,16 @@ def create_keytab(path, principal):
     except os.error:
         root_logger.critical("Failed to remove %s." % path)
 
-    kadmin("ktadd -k " + path + " " + principal)
+    return kadmin("ktadd -k " + path + " " + principal)
 
 def resolve_ip_addresses_nss(fqdn):
     """Get list of IP addresses for given host (using NSS/getaddrinfo).
     :returns:
         list of IP addresses as UnsafeIPAddress objects
     """
-    # make sure the name is fully qualified
-    # so search path from resolv.conf does not apply
-    fqdn = str(dnsutil.DNSName(fqdn).make_absolute())
+    # it would be good disable search list processing from resolv.conf
+    # to avoid cases where we get IP address for an totally different name
+    # but there is no way to do this using getaddrinfo parameters
     try:
         addrinfos = socket.getaddrinfo(fqdn, None,
                                        socket.AF_UNSPEC, socket.SOCK_STREAM)
@@ -504,7 +524,7 @@ def get_server_ip_address(host_name, unattended, setup_dns, ip_addresses):
         print("The KDC service does not listen on localhost", file=sys.stderr)
         print("", file=sys.stderr)
         print("Please fix your /etc/hosts file and restart the setup program", file=sys.stderr)
-        sys.exit(1)
+        raise ScriptError()
 
     ips = []
     if len(hostaddr):
@@ -531,11 +551,11 @@ def get_server_ip_address(host_name, unattended, setup_dns, ip_addresses):
                 print("or /etc/hosts file and restart the installation.", file=sys.stderr)
                 print("Provided but not resolved address(es): %s" % \
                                     ", ".join(str(ip) for ip in (set(ip_addresses) - set(ips))), file=sys.stderr)
-                sys.exit(1)
+                raise ScriptError()
 
     if not ips:
         print("No usable IP address provided nor resolved.", file=sys.stderr)
-        sys.exit(1)
+        raise ScriptError()
 
     for ip_address in ips:
         # check /etc/hosts sanity
@@ -550,7 +570,7 @@ def get_server_ip_address(host_name, unattended, setup_dns, ip_addresses):
                 print("Chosen hostname %s does not match configured canonical hostname %s" \
                         % (host_name, primary_host), file=sys.stderr)
                 print("Please fix your /etc/hosts file and restart the installation.", file=sys.stderr)
-                sys.exit(1)
+                raise ScriptError()
 
     return ips
 
@@ -629,9 +649,9 @@ def create_replica_config(dirman_password, filename, options):
         top_dir, dir = expand_replica_info(filename, dirman_password)
     except Exception as e:
         root_logger.error("Failed to decrypt or open the replica file.")
-        print("ERROR: Failed to decrypt or open the replica file.")
-        print("Verify you entered the correct Directory Manager password.")
-        sys.exit(1)
+        raise ScriptError(
+            "ERROR: Failed to decrypt or open the replica file.\n"
+            "Verify you entered the correct Directory Manager password.")
     config = ReplicaConfig(top_dir)
     read_replica_info(dir, config)
     root_logger.debug(
@@ -641,13 +661,13 @@ def create_replica_config(dirman_password, filename, options):
         root_logger.error(
             'A replica file from a newer release (%d) cannot be installed on an older version (%d)',
             config.version, version.NUM_VERSION)
-        sys.exit(1)
+        raise ScriptError()
     config.dirman_password = dirman_password
     try:
         host = get_host_name(options.no_host_dns)
     except BadHostError as e:
         root_logger.error(str(e))
-        sys.exit(1)
+        raise ScriptError()
     if config.host_name != host:
         try:
             print("This replica was created for '%s' but this machine is named '%s'" % (config.host_name, host))
@@ -661,7 +681,7 @@ def create_replica_config(dirman_password, filename, options):
             print("")
         except KeyboardInterrupt:
             root_logger.debug("Keyboard Interrupt")
-            sys.exit(0)
+            raise ScriptError(rval=0)
     config.dir = dir
     config.ca_ds_port = read_replica_info_dogtag_port(config.dir)
     return config
@@ -1040,7 +1060,7 @@ def load_external_cert(files, subject_base):
             raise ScriptError(
                 "IPA CA certificate not found in %s" % (", ".join(files)))
 
-        trust_chain = reversed(nssdb.get_trust_chain(ca_nickname))
+        trust_chain = list(reversed(nssdb.get_trust_chain(ca_nickname)))
         ca_cert_chain = []
         for nickname in trust_chain:
             cert, subject, issuer = cache[nickname]
@@ -1049,8 +1069,9 @@ def load_external_cert(files, subject_base):
                 break
         else:
             raise ScriptError(
-                "CA certificate chain in %s is incomplete" %
-                (", ".join(files)))
+                "CA certificate chain in %s is incomplete: "
+                "missing certificate with subject '%s'" %
+                (", ".join(files), issuer))
 
         for nickname in trust_chain:
             try:
@@ -1124,27 +1145,12 @@ def realm_to_ldapi_uri(realm_name):
     return 'ldapi://' + ldapurl.ldapUrlEscape(socketname)
 
 
-def install_service_keytab(principal, server, path, force_service_add=False):
-
+def install_service_keytab(api, principal, server, path,
+                           force_service_add=False):
     try:
-        api.Backend.rpcclient.connect()
-
-        # Create services if none exists (we use the .forward method
-        # here so that we can control the client version number and avoid
-        # errors. This is a workaround until the API becomes version
-        # independent: FIXME
-
-        api.Backend.rpcclient.forward(
-            'service_add',
-            krbprincipalname=principal,
-            force=force_service_add,
-            version=u'2.112'    # All the way back to 3.0 servers
-        )
+        api.Command.service_add(principal, force=force_service_add)
     except errors.DuplicateEntry:
         pass
-    finally:
-        if api.Backend.rpcclient.isconnected():
-            api.Backend.rpcclient.disconnect()
 
     args = [paths.IPA_GETKEYTAB, '-k', path, '-p', principal, '-s', server]
     ipautil.run(args)
