@@ -18,12 +18,10 @@
 #
 
 import os
-import sys
 import shutil
 import tempfile
 import time
 import pwd
-from optparse import OptionGroup
 from ipaplatform.paths import paths
 from ipaplatform import services
 
@@ -33,14 +31,12 @@ from ipalib import api, errors
 from ipapython import version
 from ipapython.ipautil import run, write_tmp_file
 from ipapython import admintool
-from ipapython.config import IPAOptionParser
 from ipapython.dn import DN
 from ipaserver.install.replication import wait_for_task
 from ipaserver.install import installutils
+from ipaserver.session import ISO8601_DATETIME_FMT
 from ipapython import ipaldap
-from ipalib.session import ISO8601_DATETIME_FMT
 from ipalib.constants import CACERT
-from six.moves.configparser import SafeConfigParser
 from ipaplatform.constants import constants
 from ipaplatform.tasks import tasks
 
@@ -259,7 +255,7 @@ class Backup(admintool.AdminTool):
         options = self.options
         super(Backup, self).run()
 
-        api.bootstrap(in_server=False, context='backup')
+        api.bootstrap(in_server=True, context='backup')
         api.finalize()
 
         self.log.info("Preparing backup on %s", api.env.host)
@@ -341,9 +337,12 @@ class Backup(admintool.AdminTool):
             if os.path.exists(dir):
                 self.dirs.append(dir)
 
-        file = paths.SYSCONFIG_DIRSRV_INSTANCE % serverid
-        if os.path.exists(file):
-            self.files.append(file)
+        for file in (
+            paths.SYSCONFIG_DIRSRV_INSTANCE % serverid,
+            paths.ETC_TMPFILESD_DIRSRV % serverid,
+        ):
+            if os.path.exists(file):
+                self.files.append(file)
 
         self.logs.append(paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % serverid)
 
