@@ -30,7 +30,6 @@ import time
 import os
 import pwd
 import fnmatch
-import re
 
 import ldap
 import six
@@ -43,7 +42,7 @@ from ipalib import constants
 from ipaplatform.paths import paths
 from ipaplatform import services
 from ipapython.dn import DN
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import log_mgr
 
 if six.PY3:
     unicode = str
@@ -84,8 +83,8 @@ def connect(ldapi=False, realm=None, fqdn=None, dm_password=None, pw_name=None):
 class BadSyntax(installutils.ScriptError):
     def __init__(self, value):
         self.value = value
-        self.msg = "LDAPUpdate: syntax error: \n  %s" % value
-        self.rval = 1
+        super(BadSyntax, self).__init__(
+            msg="LDAPUpdate: syntax error: \n  %s" % value, rval=1)
 
     def __str__(self):
         return repr(self.value)
@@ -922,20 +921,6 @@ class LDAPUpdate:
 
         return self.modified
 
-    def update_from_dict(self, updates):
-        """
-        Apply updates internally as opposed to from a file.
-        updates is a dictionary containing the updates
-        """
-        self.modified = False
-        try:
-            self.create_connection()
-            self._run_updates(updates)
-        finally:
-            self.close_connection()
-
-        return self.modified
-
     def close_connection(self):
         """Close ldap connection"""
         if self.conn:
