@@ -17,19 +17,25 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import ldap as _ldap
+from __future__ import absolute_import
+
+import dns.exception
 import re
 import traceback
 import time
 
 from ldif import LDIFWriter
 
-from ipalib import api, errors, util
+from ipalib import Registry, errors, util
 from ipalib import Updater
 from ipapython.dn import DN
 from ipapython import dnsutil
-from ipalib.plugins.dns import dns_container_exists
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
+from ipaserver.install import sysupgrade
+from ipaserver.install.bindinstance import ensure_dnsserver_container_exists
+from ipaserver.plugins.dns import dns_container_exists
+
+register = Registry()
 
 
 class DNSUpdater(Updater):
@@ -107,6 +113,7 @@ class DNSUpdater(Updater):
                 self.ldif_writer.unparse(dn, record)
 
 
+@register()
 class update_ipaconfigstring_dnsversion_to_ipadnsversion(Updater):
     """
     IPA <= 4.3.1 used ipaConfigString "DNSVersion 1" on DNS container.
@@ -147,9 +154,8 @@ class update_ipaconfigstring_dnsversion_to_ipadnsversion(Updater):
         self.log.debug('ipaDNSVersion = %s', version)
         return False, []
 
-api.register(update_ipaconfigstring_dnsversion_to_ipadnsversion)
-
 
+@register()
 class update_dnszones(Updater):
     """
     Update all zones to meet requirements in the new FreeIPA versions
@@ -209,9 +215,8 @@ class update_dnszones(Updater):
 
         return False, []
 
-api.register(update_dnszones)
-
 
+@register()
 class update_dns_limits(Updater):
     """
     bind-dyndb-ldap persistent search queries LDAP for all DNS records.
@@ -257,9 +262,8 @@ class update_dns_limits(Updater):
 
         return False, [dnsupdate]
 
-api.register(update_dns_limits)
-
 
+@register()
 class update_master_to_dnsforwardzones(DNSUpdater):
     """
     Update all zones to meet requirements in the new FreeIPA versions
@@ -400,9 +404,8 @@ class update_master_to_dnsforwardzones(DNSUpdater):
 
         return False, []
 
-api.register(update_master_to_dnsforwardzones)
-
 
+@register()
 class update_dnsforward_emptyzones(DNSUpdater):
     """
     Migrate forward policies which conflict with automatic empty zones
@@ -489,9 +492,55 @@ class update_dnsforward_emptyzones(DNSUpdater):
         self.api.Command['dnsconfig_mod'](ipadnsversion=2)
 
         self.update_zones()
-        if dnsutil.has_empty_zone_addresses(self.api.env.host):
-            self.update_global_ldap_forwarder()
+        try:
+            if dnsutil.has_empty_zone_addresses(self.api.env.host):
+                self.update_global_ldap_forwarder()
+        except dns.exception.DNSException as ex:
+            self.log.error('Skipping update of global DNS forwarder in LDAP: '
+                           'Unable to determine if local server is using an '
+                           'IP address belonging to an automatic empty zone. '
+                           'Consider changing forwarding policy to "only". '
+                           'DNS exception: %s', ex)
 
         return False, []
 
-api.register(update_dnsforward_emptyzones)
+
+@register()
+class update_dnsserver_configuration_into_ldap(DNSUpdater):
+    """
+    DNS Locations feature requires to have DNS configuration stored in LDAP DB.
+    Create DNS server configuration in LDAP for each old server
+    """
+    def execute(self, **options):
+        ldap = self.api.Backend.ldap2
+        if sysupgrade.get_upgrade_state('dns', 'server_config_to_ldap'):
+            self.log.debug('upgrade is not needed')
+            return False, []
+
+        dns_container_dn = DN(self.api.env.container_dns, self.api.env.basedn)
+        try:
+            ldap.get_entry(dns_container_dn)
+        except errors.NotFound:
+            self.log.debug('DNS container not found, nothing to upgrade')
+            sysupgrade.set_upgrade_state('dns', 'server_config_to_ldap', True)
+            return False, []
+
+        result = self.api.Command.server_show(self.api.env.host)['result']
+        if not 'DNS server' in result.get('enabled_role_servrole', []):
+            self.log.debug('This server is not DNS server, nothing to upgrade')
+            sysupgrade.set_upgrade_state('dns', 'server_config_to_ldap', True)
+            return False, []
+
+        # create container first, if doesn't exist
+        ensure_dnsserver_container_exists(ldap, self.api, logger=self.log)
+
+        try:
+            self.api.Command.dnsserver_add(self.api.env.host)
+        except errors.DuplicateEntry:
+            self.log.debug("DNS server configuration already exists "
+                           "in LDAP database")
+        else:
+            self.log.debug("DNS server configuration has been sucessfully "
+                           "created in LDAP database")
+        sysupgrade.set_upgrade_state('dns', 'server_config_to_ldap', True)
+        return False, []
