@@ -17,16 +17,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import os
-import pwd
-from ipapython import ipaldap
 from ipaserver.install import replication
-from ipalib import api
+from ipalib import Registry
 from ipalib import Updater
 
+register = Registry()
+
 EXCLUDE_TEMPLATE = '(objectclass=*) $ EXCLUDE %s'
 
 
+@register()
 class update_replica_attribute_lists(Updater):
     """
     Run through all replication agreements and ensure that EXCLUDE list
@@ -112,5 +112,3 @@ class update_replica_attribute_lists(Updater):
                         attribute, str(e))
             else:
                 self.log.debug("%s: No update necessary" % attribute)
-
-api.register(update_replica_attribute_lists)
