@@ -11,6 +11,8 @@ import pwd
 import fileinput
 import sys
 
+import dns.exception
+
 import six
 from six.moves.configparser import SafeConfigParser
 
@@ -22,7 +24,7 @@ from ipaplatform import services
 from ipaplatform.tasks import tasks
 from ipapython import ipautil, sysrestore, version, certdb
 from ipapython import ipaldap
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 from ipapython import certmonger
 from ipapython import dnsutil
 from ipapython.dn import DN
@@ -32,6 +34,7 @@ from ipaserver.install import installutils
 from ipaserver.install import dsinstance
 from ipaserver.install import httpinstance
 from ipaserver.install import memcacheinstance
+from ipaserver.install import ntpinstance
 from ipaserver.install import bindinstance
 from ipaserver.install import service
 from ipaserver.install import cainstance
@@ -43,6 +46,7 @@ from ipaserver.install import sysupgrade
 from ipaserver.install import dnskeysyncinstance
 from ipaserver.install import krainstance
 from ipaserver.install import dogtaginstance
+from ipaserver.install import adtrustinstance
 from ipaserver.install.upgradeinstance import IPAUpgrade
 from ipaserver.install.ldapupdate import BadSyntax
 
@@ -95,23 +99,6 @@ def update_conf(sub_dict, filename, template_filename):
     fd.write(template)
     fd.close()
 
-def find_hostname():
-    """Find the hostname currently configured in ipa-rewrite.conf"""
-    filename=paths.HTTPD_IPA_REWRITE_CONF
-
-    if not ipautil.file_exists(filename):
-        return None
-
-    pattern = "^[\s#]*.*https:\/\/([A-Za-z0-9\.\-]*)\/.*"
-    p = re.compile(pattern)
-    for line in fileinput.input(filename):
-        if p.search(line):
-            fileinput.close()
-            return p.search(line).group(1)
-    fileinput.close()
-
-    raise RuntimeError("Unable to determine the fully qualified hostname from %s" % filename)
-
 def find_autoredirect(fqdn):
     """
     When upgrading ipa-rewrite.conf we need to see if the automatic redirect
@@ -293,6 +280,26 @@ def setup_firefox_extension(fstore):
     http.setup_firefox_extension(realm, domain)
 
 
+def upgrade_adtrust_config():
+    """
+    Upgrade 'dedicated keytab file' in smb.conf to omit FILE: prefix
+    """
+
+    if not adtrustinstance.ipa_smb_conf_exists():
+        return
+
+    root_logger.info("[Remove FILE: prefix from 'dedicated keytab file' "
+                     "in Samba configuration]")
+
+    args = [paths.NET, "conf", "setparm", "global",
+            "dedicated keytab file", paths.SAMBA_KEYTAB]
+
+    try:
+        ipautil.run(args)
+    except ipautil.CalledProcessError as e:
+        root_logger.warning("Error updating Samba registry: %s", e)
+
+
 def ca_configure_profiles_acl(ca):
     root_logger.info('[Authorizing RA Agent to modify profiles]')
 
@@ -303,6 +310,16 @@ def ca_configure_profiles_acl(ca):
     return cainstance.configure_profiles_acl()
 
 
+def ca_configure_lightweight_ca_acls(ca):
+    root_logger.info('[Authorizing RA Agent to manage lightweight CAs]')
+
+    if not ca.is_configured():
+        root_logger.info('CA is not configured')
+        return False
+
+    return cainstance.configure_lightweight_ca_acls()
+
+
 def ca_enable_ldap_profile_subsystem(ca):
     root_logger.info('[Ensuring CA is using LDAPProfileSubsystem]')
     if not ca.is_configured():
@@ -352,6 +369,38 @@ def ca_import_included_profiles(ca):
     return cainstance.import_included_profiles()
 
 
+def ca_ensure_lightweight_cas_container(ca):
+    root_logger.info('[Ensuring Lightweight CAs container exists in Dogtag database]')
+
+    if not ca.is_configured():
+        root_logger.info('CA is not configured')
+        return False
+
+    return cainstance.ensure_lightweight_cas_container()
+
+
+def ca_add_default_ocsp_uri(ca):
+    root_logger.info('[Adding default OCSP URI configuration]')
+    if not ca.is_configured():
+        root_logger.info('CA is not configured')
+        return False
+
+    value = installutils.get_directive(
+        paths.CA_CS_CFG_PATH,
+        'ca.defaultOcspUri',
+        separator='=')
+    if value:
+        return False  # already set; restart not needed
+
+    installutils.set_directive(
+        paths.CA_CS_CFG_PATH,
+        'ca.defaultOcspUri',
+        'http://ipa-ca.%s/ca/ocsp' % ipautil.format_netloc(api.env.domain),
+        quotes=False,
+        separator='=')
+    return True  # restart needed
+
+
 def upgrade_ca_audit_cert_validity(ca):
     """
     Update the Dogtag audit signing certificate.
@@ -814,9 +863,18 @@ def named_update_global_forwarder_policy():
         'forward_policy_conflict_with_empty_zones_handled',
         True
     )
-    if not dnsutil.has_empty_zone_addresses(api.env.host):
-        # guess: local server does not have IP addresses from private ranges
-        # so hopefully automatic empty zones are not a problem
+    try:
+        if not dnsutil.has_empty_zone_addresses(api.env.host):
+            # guess: local server does not have IP addresses from private
+            # ranges so hopefully automatic empty zones are not a problem
+            return False
+    except dns.exception.DNSException as ex:
+        root_logger.error(
+            'Skipping update of global DNS forwarder in named.conf: '
+            'Unable to determine if local server is using an '
+            'IP address belonging to an automatic empty zone. '
+            'Consider changing forwarding policy to "only". '
+            'DNS exception: %s', ex)
         return False
 
     if bindinstance.named_conf_get_directive(
@@ -838,6 +896,28 @@ def named_update_global_forwarder_policy():
     return True
 
 
+def named_add_server_id():
+    """
+    DNS Locations feature requires to have configured server_id in IPA section
+    of named.conf
+    :return: if named.conf has been changed
+    """
+    bind = bindinstance.BindInstance()
+    if not bindinstance.named_conf_exists() or not bind.is_configured():
+        # DNS service may not be configured
+        root_logger.info('DNS is not configured')
+        return False
+
+    if sysupgrade.get_upgrade_state('named.conf', 'add_server_id'):
+        # upgrade was done already
+        return False
+
+    root_logger.info('[Adding server_id to named.conf]')
+    bindinstance.named_conf_set_directive('server_id', api.env.host)
+    sysupgrade.set_upgrade_state('named.conf', 'add_server_id', True)
+    return True
+
+
 def certificate_renewal_update(ca, ds, http):
     """
     Update certmonger certificate renewal configuration.
@@ -1101,12 +1181,9 @@ def add_ca_dns_records():
 
     bind = bindinstance.BindInstance()
 
-    bind.convert_ipa_ca_cnames(api.env.domain)
+    bind.remove_ipa_ca_cnames(api.env.domain)
 
-    # DNS is enabled, so let bindinstance find out if CA is enabled
-    # and let it add the record in that case
-    bind.add_ipa_ca_dns_records(api.env.host, api.env.domain,
-                                ca_configured=None)
+    bind.update_system_records()
 
     sysupgrade.set_upgrade_state('dns', 'ipa_ca_records', True)
 
@@ -1421,6 +1498,10 @@ def update_mod_nss_cipher_suite(http):
         'cipher_suite_updated',
         httpinstance.NSS_CIPHER_REVISION)
 
+def update_ipa_httpd_service_conf(http):
+    root_logger.info('[Updating HTTPD service IPA configuration]')
+    http.update_httpd_service_ipa_conf()
+
 
 def ds_enable_sidgen_extdom_plugins(ds):
     """For AD trust agents, make sure we enable sidgen and extdom plugins
@@ -1441,7 +1522,10 @@ def ca_upgrade_schema(ca):
         root_logger.info('CA is not configured')
         return False
 
-    schema_files=['/usr/share/pki/server/conf/schema-certProfile.ldif']
+    schema_files=[
+        '/usr/share/pki/server/conf/schema-certProfile.ldif',
+        '/usr/share/pki/server/conf/schema-authority.ldif',
+    ]
     try:
         modified = schemaupdate.update_schema(schema_files, ldapi=True)
     except Exception as e:
@@ -1478,10 +1562,7 @@ def upgrade_configuration():
 
     fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
-    fqdn = find_hostname()
-    if fqdn is None:
-        # ipa-rewrite.conf doesn't exist, nothing to do
-        raise RuntimeError("ipa-rewrite.conf doesn't exists (is this server?)")
+    fqdn = api.env.host
 
     # Ok, we are an IPA server, do the additional tests
     ds_serverid = installutils.realm_to_serverid(api.env.realm)
@@ -1508,7 +1589,8 @@ def upgrade_configuration():
     if subject_base:
         sub_dict['SUBJECT_BASE'] = subject_base
 
-    ca = cainstance.CAInstance(api.env.realm, certs.NSS_DIR)
+    ca = cainstance.CAInstance(
+            api.env.realm, certs.NSS_DIR, host_name=api.env.host)
     ca_running = ca.is_running()
 
     with installutils.stopped_service('pki-tomcatd', 'pki-tomcat'):
@@ -1577,6 +1659,8 @@ def upgrade_configuration():
 
     ds.configure_dirsrv_ccache()
 
+    ntpinstance.ntp_ldap_enable(api.env.host, api.env.basedn, api.env.realm)
+
     # ldap2 connection is not valid after DS restart, close connection otherwise
     # it will cause network errors
     if api.Backend.ldap2.isconnected():
@@ -1614,6 +1698,7 @@ def upgrade_configuration():
         http.enable_kdcproxy()
 
     http.stop()
+    update_ipa_httpd_service_conf(http)
     update_mod_nss_protocol(http)
     update_mod_nss_cipher_suite(http)
     fix_trust_flags()
@@ -1653,6 +1738,16 @@ def upgrade_configuration():
     cleanup_kdc(fstore)
     cleanup_adtrust(fstore)
     setup_firefox_extension(fstore)
+    upgrade_adtrust_config()
+
+    bind = bindinstance.BindInstance(fstore)
+    if bind.is_configured() and not bind.is_running():
+        # some upgrade steps may require bind running
+        bind_started = True
+        bind.start()
+    else:
+        bind_started = False
+
     add_ca_dns_records()
 
     # Any of the following functions returns True iff the named.conf file
@@ -1671,6 +1766,7 @@ def upgrade_configuration():
                           named_update_global_forwarder_policy(),
                           mask_named_regular(),
                           fix_dyndb_ldap_workdir_permissions(),
+                          named_add_server_id(),
                          )
 
     if any(named_conf_changes):
@@ -1678,10 +1774,14 @@ def upgrade_configuration():
         root_logger.info('Changes to named.conf have been made, restart named')
         bind = bindinstance.BindInstance(fstore)
         try:
-            bind.restart()
+            if bind.is_running():
+                bind.restart()
         except ipautil.CalledProcessError as e:
             root_logger.error("Failed to restart %s: %s", bind.service_name, e)
 
+    if bind_started:
+        bind.stop()
+
     custodia = custodiainstance.CustodiaInstance(api.env.host, api.env.realm)
     custodia.upgrade_instance()
 
@@ -1692,6 +1792,9 @@ def upgrade_configuration():
         certificate_renewal_update(ca, ds, http),
         ca_enable_pkix(ca),
         ca_configure_profiles_acl(ca),
+        ca_configure_lightweight_ca_acls(ca),
+        ca_ensure_lightweight_cas_container(ca),
+        ca_add_default_ocsp_uri(ca),
     ])
 
     if ca_restart:
@@ -1713,6 +1816,8 @@ def upgrade_configuration():
 
     if ca.is_configured():
         cainstance.repair_profile_caIPAserviceCert()
+        ca.setup_lightweight_ca_key_retrieval()
+        cainstance.ensure_ipa_authority_entry()
 
     set_sssd_domain_option('ipa_server_mode', 'True')
 
@@ -1732,7 +1837,7 @@ def upgrade_check(options):
     try:
         installutils.check_server_configuration()
     except RuntimeError as e:
-        print(unicode(e))
+        root_logger.error(e)
         sys.exit(1)
 
     if not options.skip_version_check:
@@ -1758,6 +1863,9 @@ def upgrade():
     realm = api.env.realm
     schema_files = [os.path.join(ipautil.SHARE_DIR, f) for f
                     in dsinstance.ALL_SCHEMA_FILES]
+
+    schema_files.extend(dsinstance.get_all_external_schema_files(
+                        paths.EXTERNAL_SCHEMA_DIR))
     data_upgrade = IPAUpgrade(realm, schema_files=schema_files)
 
     try:
