@@ -18,15 +18,15 @@
 #
 
 import sys
-import os, socket
+import os
+import socket
 import tempfile
-import time
 import datetime
 import traceback
 
 from ipapython import sysrestore, ipautil, ipaldap
 from ipapython.dn import DN
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 from ipalib import api, errors, certstore
 from ipaplatform import services
 from ipaplatform.paths import paths
@@ -41,6 +41,7 @@ SERVICE_LIST = {
     'MEMCACHE': ('ipa_memcached', 39),
     'HTTP': ('httpd', 40),
     'KEYS': ('ipa-custodia', 41),
+    'NTP': ('ntpd', 45),
     'CA': ('pki-tomcatd', 50),
     'KRA': ('pki-tomcatd', 51),
     'ADTRUST': ('smb', 60),
@@ -251,6 +252,7 @@ class Service(object):
             # There is no service in the wrong location, nothing to do.
             # This can happen when installing a replica
             return None
+        entry.pop('krbpwdpolicyreference', None)  # don't copy virtual attr
         newdn = DN(('krbprincipalname', principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
         hostdn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
         self.admin_conn.delete_entry(entry)
@@ -346,8 +348,8 @@ class Service(object):
     def restart(self, instance_name="", capture_output=True, wait=True):
         self.service.restart(instance_name, capture_output=capture_output, wait=wait)
 
-    def is_running(self):
-        return self.service.is_running()
+    def is_running(self, instance_name="", wait=True):
+        return self.service.is_running(instance_name, wait)
 
     def install(self):
         self.service.install()
