@@ -47,6 +47,48 @@ from ipalib.constants import DOMAIN_LEVEL_0
 log = log_mgr.get_logger(__name__)
 
 
+def setup_server_logs_collecting(host):
+    """
+    This function setup logs to be collected on host. We should collect all
+    possible logs that may be helpful to debug IPA server
+    """
+    # dirsrv logs
+    inst = host.domain.realm.replace('.', '-')
+    host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+    host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
+
+    # IPA install logs
+    host.collect_log(paths.IPASERVER_INSTALL_LOG)
+    host.collect_log(paths.IPACLIENT_INSTALL_LOG)
+    host.collect_log(paths.IPAREPLICA_INSTALL_LOG)
+    host.collect_log(paths.IPAREPLICA_CONNCHECK_LOG)
+    host.collect_log(paths.IPAREPLICA_CA_INSTALL_LOG)
+    host.collect_log(paths.IPACLIENT_INSTALL_LOG)
+    host.collect_log(paths.IPASERVER_KRA_INSTALL_LOG)
+    host.collect_log(paths.IPA_CUSTODIA_AUDIT_LOG)
+
+    # IPA uninstall logs
+    host.collect_log(paths.IPASERVER_KRA_UNINSTALL_LOG)
+    host.collect_log(paths.IPACLIENT_UNINSTALL_LOG)
+    host.collect_log(paths.IPASERVER_KRA_UNINSTALL_LOG)
+
+    # IPA backup and restore logs
+    host.collect_log(paths.IPARESTORE_LOG)
+    host.collect_log(paths.IPABACKUP_LOG)
+
+    # kerberos related logs
+    host.collect_log(paths.KADMIND_LOG)
+
+    # httpd logs
+    host.collect_log(paths.VAR_LOG_HTTPD_ERROR)
+
+    # dogtag logs
+    host.collect_log(os.path.join(paths.VAR_LOG_PKI_DIR))
+
+    # SSSD debugging must be set after client is installed (function
+    # setup_sssd_debugging)
+
+
 def check_arguments_are(slice, instanceof):
     """
     :param: slice - tuple of integers denoting the beginning and the end
@@ -57,10 +99,10 @@ def check_arguments_are(slice, instanceof):
     and third arguments are integers
     """
     def wrapper(func):
-        def wrapped(*args):
+        def wrapped(*args, **kwargs):
             for i in args[slice[0]:slice[1]]:
                 assert isinstance(i, instanceof), "Wrong type: %s: %s" % (i, type(i))
-            return func(*args)
+            return func(*args, **kwargs)
         return wrapped
     return wrapper
 
@@ -96,9 +138,8 @@ def allow_sync_ptr(host):
 
 
 def apply_common_fixes(host):
-    fix_etc_hosts(host)
-    fix_hostname(host)
     prepare_host(host)
+    fix_hostname(host)
 
 
 def backup_file(host, filename):
@@ -118,24 +159,6 @@ def backup_file(host, filename):
         return False
 
 
-def fix_etc_hosts(host):
-    backup_file(host, paths.HOSTS)
-    contents = host.get_file_contents(paths.HOSTS)
-    # Remove existing mentions of the host's FQDN, short name, and IP
-    # Removing of IP must be done as first, otherwise hosts file may be
-    # corrupted
-    contents = re.sub('^%s.*' % re.escape(host.ip), '', contents,
-                      flags=re.MULTILINE)
-    contents = re.sub('\s%s(\s|$)' % re.escape(host.hostname), ' ', contents,
-                      flags=re.MULTILINE)
-    contents = re.sub('\s%s(\s|$)' % re.escape(host.shortname), ' ', contents,
-                      flags=re.MULTILINE)
-    # Add the host's info again
-    contents += '\n%s %s %s\n' % (host.ip, host.hostname, host.shortname)
-    log.debug('Writing the following to /etc/hosts:\n%s', contents)
-    host.put_file_contents(paths.HOSTS, contents)
-
-
 def fix_hostname(host):
     backup_file(host, paths.ETC_HOSTNAME)
     host.put_file_contents(paths.ETC_HOSTNAME, host.hostname + '\n')
@@ -234,12 +257,7 @@ def install_master(host, setup_dns=True, setup_kra=False, extra_args=(),
                    domain_level=None):
     if domain_level is None:
         domain_level = host.config.domain_level
-    host.collect_log(paths.IPASERVER_INSTALL_LOG)
-    host.collect_log(paths.IPACLIENT_INSTALL_LOG)
-    inst = host.domain.realm.replace('.', '-')
-    host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
-    host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
-
+    setup_server_logs_collecting(host)
     apply_common_fixes(host)
     fix_apache_semaphores(host)
 
@@ -281,8 +299,16 @@ def get_replica_filename(replica):
 
 
 def domainlevel(host):
-    # Dynamically determines the domainlevel on master. Needed for scenarios
-    # when domainlevel is changed during the test execution.
+    """
+    Dynamically determines the domainlevel on master. Needed for scenarios
+    when domainlevel is changed during the test execution.
+
+    Sometimes the master is even not installed. Please refer to ca-less
+    tests, where we call tasks.uninstall_master after every test while a lot
+    of them make sure that the server installation fails. Therefore we need
+    to not raise on failures here.
+    """
+    kinit_admin(host, raiseonerr=False)
     result = host.run_command(['ipa', 'domainlevel-get'], raiseonerr=False)
     level = 0
     domlevel_re = re.compile('.*(\d)')
@@ -321,8 +347,7 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
     if domain_level is None:
         domain_level = domainlevel(master)
     apply_common_fixes(replica)
-    replica.collect_log(paths.IPAREPLICA_INSTALL_LOG)
-    replica.collect_log(paths.IPAREPLICA_CONNCHECK_LOG)
+    setup_server_logs_collecting(replica)
     allow_sync_ptr(master)
     # Otherwise ipa-client-install would not create a PTR
     # and replica installation would fail
@@ -331,6 +356,9 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
             '-w', replica.config.admin_password]
     if setup_ca:
         args.append('--setup-ca')
+    if setup_kra:
+        assert setup_ca, "CA must be installed on replica with KRA"
+        args.append('--setup-kra')
     if setup_dns:
         args.extend([
             '--setup-dns',
@@ -355,17 +383,6 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
     enable_replication_debugging(replica)
     setup_sssd_debugging(replica)
 
-    if setup_kra:
-        assert setup_ca, "CA must be installed on replica with KRA"
-        args = [
-            "ipa-kra-install",
-            "-p", replica.config.dirman_password,
-            "-U",
-        ]
-        if domainlevel(master) == DOMAIN_LEVEL_0:
-            args.append(replica_filename)
-        replica.run_command(args)
-
     kinit_admin(replica)
 
 
@@ -400,12 +417,7 @@ def install_adtrust(host):
     Configures the compat tree for the legacy clients.
     """
 
-    # ipa-adtrust-install appends to ipaserver-install.log
-    host.collect_log(paths.IPASERVER_INSTALL_LOG)
-
-    inst = host.domain.realm.replace('.', '-')
-    host.collect_log(paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
-    host.collect_log(paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
+    setup_server_logs_collecting(host)
 
     kinit_admin(host)
     host.run_command(['ipa-adtrust-install', '-U',
@@ -436,50 +448,14 @@ def install_adtrust(host):
 
 def configure_dns_for_trust(master, ad):
     """
-    This configures DNS on IPA master according to the relationship of the
-    IPA's and AD's domains.
+    This method is intentionally left empty. Originally it served for DNS
+    configuration on IPA master according to the relationship of the IPA's
+    and AD's domains.
     """
-
-    def is_subdomain(subdomain, domain):
-        subdomain_unpacked = subdomain.split('.')
-        domain_unpacked = domain.split('.')
-
-        subdomain_unpacked.reverse()
-        domain_unpacked.reverse()
-
-        subdomain = False
-
-        if len(subdomain_unpacked) > len(domain_unpacked):
-            subdomain = True
-
-            for subdomain_segment, domain_segment in zip(subdomain_unpacked,
-                                                         domain_unpacked):
-                subdomain = subdomain and subdomain_segment == domain_segment
-
-        return subdomain
-
-    kinit_admin(master)
-
-    if is_subdomain(ad.domain.name, master.domain.name):
-        master.run_command(['ipa', 'dnsrecord-add', master.domain.name,
-                            '%s.%s' % (ad.shortname, ad.netbios),
-                            '--a-ip-address', ad.ip])
-
-        master.run_command(['ipa', 'dnsrecord-add', master.domain.name,
-                            ad.netbios,
-                            '--ns-hostname',
-                            '%s.%s' % (ad.shortname, ad.netbios)])
-
-        master.run_command(['ipa', 'dnszone-mod', master.domain.name,
-                            '--allow-transfer', ad.ip])
-    else:
-        master.run_command(['ipa', 'dnsforwardzone-add', ad.domain.name,
-                            '--forwarder', ad.ip,
-                            '--forward-policy', 'only',
-                            ])
+    pass
 
 
-def establish_trust_with_ad(master, ad, extra_args=()):
+def establish_trust_with_ad(master, ad_domain, extra_args=()):
     """
     Establishes trust with Active Directory. Trust type is detected depending
     on the presence of SfU (Services for Unix) support on the AD.
@@ -497,17 +473,20 @@ def establish_trust_with_ad(master, ad, extra_args=()):
     kinit_admin(master)
     master.run_command(['klist'])
     master.run_command(['smbcontrol', 'all', 'debug', '100'])
+
     util.run_repeatedly(master,
                         ['ipa', 'trust-add',
-                         '--type', 'ad', ad.domain.name,
+                         '--type', 'ad', ad_domain,
                          '--admin', 'Administrator',
                          '--password'] + list(extra_args),
                         stdin_text=master.config.ad_admin_password)
     master.run_command(['smbcontrol', 'all', 'debug', '1'])
     clear_sssd_cache(master)
+    master.run_command(['systemctl', 'restart', 'krb5kdc.service'])
+    time.sleep(60)
 
 
-def remove_trust_with_ad(master, ad):
+def remove_trust_with_ad(master, ad_domain):
     """
     Removes trust with Active Directory. Also removes the associated ID range.
     """
@@ -515,12 +494,23 @@ def remove_trust_with_ad(master, ad):
     kinit_admin(master)
 
     # Remove the trust
-    master.run_command(['ipa', 'trust-del', ad.domain.name])
+    master.run_command(['ipa', 'trust-del', ad_domain])
 
     # Remove the range
-    range_name = ad.domain.name.upper() + '_id_range'
+    range_name = ad_domain.upper() + '_id_range'
     master.run_command(['ipa', 'idrange-del', range_name])
 
+    remove_trust_info_from_ad(master, ad_domain)
+
+
+def remove_trust_info_from_ad(master, ad_domain):
+    # Remove record about trust from AD
+    master.run_command(['rpcclient', ad_domain,
+                        '-U\\Administrator%{}'.format(
+                            master.config.ad_admin_password),
+                        '-c', 'deletetrustdom {}'.format(master.domain.name)],
+                       raiseonerr=False)
+
 
 def configure_auth_to_local_rule(master, ad):
     """
@@ -563,7 +553,7 @@ def setup_sssd_debugging(host):
                       paths.SSSD_CONF],
                      raiseonerr=False)
 
-    host.collect_log('/var/log/sssd/*')
+    host.collect_log(os.path.join(paths.VAR_LOG_SSSD_DIR))
 
     # Clear the cache and restart SSSD
     clear_sssd_cache(host)
@@ -673,12 +663,13 @@ def disconnect_replica(master, replica, domain_level=None):
                             ])
 
 
-def kinit_admin(host):
-    host.run_command(['kinit', 'admin'],
+def kinit_admin(host, raiseonerr=True):
+    host.run_command(['kinit', 'admin'], raiseonerr=raiseonerr,
                      stdin_text=host.config.admin_password)
 
 
-def uninstall_master(host, ignore_topology_disconnect=True):
+def uninstall_master(host, ignore_topology_disconnect=True,
+                     ignore_last_of_role=True):
     host.collect_log(paths.IPASERVER_UNINSTALL_LOG)
     uninstall_cmd = ['ipa-server-install', '--uninstall', '-U']
 
@@ -687,6 +678,9 @@ def uninstall_master(host, ignore_topology_disconnect=True):
     if ignore_topology_disconnect and host_domain_level != DOMAIN_LEVEL_0:
         uninstall_cmd.append('--ignore-topology-disconnect')
 
+    if ignore_last_of_role and host_domain_level != DOMAIN_LEVEL_0:
+        uninstall_cmd.append('--ignore-last-of-role')
+
     host.run_command(uninstall_cmd, raiseonerr=False)
     host.run_command(['pkidestroy', '-s', 'CA', '-i', 'pki-tomcat'],
                      raiseonerr=False)
@@ -698,6 +692,10 @@ def uninstall_master(host, ignore_topology_disconnect=True):
                       paths.PKI_TOMCAT,
                       paths.REPLICA_INFO_GPG_TEMPLATE % host.hostname],
                      raiseonerr=False)
+    host.run_command("find /var/lib/sss/keytabs -name '*.keytab' | "
+                     "xargs rm -fv", raiseonerr=False)
+    host.run_command("find /run/ipa -name 'krb5*' | xargs rm -fv",
+                     raiseonerr=False)
     unapply_fixes(host)
 
 
@@ -721,7 +719,7 @@ def clean_replication_agreement(master, replica):
 
 
 @check_arguments_are((0, 3), Host)
-def create_segment(master, leftnode, rightnode):
+def create_segment(master, leftnode, rightnode, suffix=DOMAIN_SUFFIX_NAME):
     """
     creates a topology segment. The first argument is a node to run the command
     :returns: a hash object containing segment's name, leftnode, rightnode
@@ -731,7 +729,7 @@ def create_segment(master, leftnode, rightnode):
     lefthost = leftnode.hostname
     righthost = rightnode.hostname
     segment_name = "%s-to-%s" % (lefthost, righthost)
-    result = master.run_command(["ipa", "topologysegment-add", DOMAIN_SUFFIX_NAME,
+    result = master.run_command(["ipa", "topologysegment-add", suffix,
                                  segment_name,
                                  "--leftnode=%s" % lefthost,
                                  "--rightnode=%s" % righthost], raiseonerr=False)
@@ -743,7 +741,7 @@ def create_segment(master, leftnode, rightnode):
         return {}, result.stderr_text
 
 
-def destroy_segment(master, segment_name):
+def destroy_segment(master, segment_name, suffix=DOMAIN_SUFFIX_NAME):
     """
     Destroys topology segment.
     :param master: reference to master object of class Host
@@ -753,7 +751,7 @@ def destroy_segment(master, segment_name):
     kinit_admin(master)
     command = ["ipa",
                "topologysegment-del",
-               DOMAIN_SUFFIX_NAME,
+               suffix,
                segment_name]
     result = master.run_command(command, raiseonerr=False)
     return result.returncode, result.stderr_text
@@ -988,12 +986,20 @@ def double_circle_topo(master, replicas, site_size=6):
 
 
 def install_topo(topo, master, replicas, clients, domain_level=None,
-                 skip_master=False, setup_replica_cas=True):
+                 skip_master=False, setup_replica_cas=True,
+                 setup_replica_kras=False):
     """Install IPA servers and clients in the given topology"""
+    if setup_replica_kras and not setup_replica_cas:
+        raise ValueError("Option 'setup_replica_kras' requires "
+                         "'setup_replica_cas' set to True")
     replicas = list(replicas)
     installed = {master}
     if not skip_master:
-        install_master(master, domain_level=domain_level)
+        install_master(
+            master,
+            domain_level=domain_level,
+            setup_kra=setup_replica_kras
+        )
 
     add_a_records_for_hosts_in_master_domain(master)
 
@@ -1003,7 +1009,11 @@ def install_topo(topo, master, replicas, clients, domain_level=None,
             connect_replica(parent, child)
         else:
             log.info('Installing replica %s from %s' % (parent, child))
-            install_replica(parent, child, setup_ca=setup_replica_cas)
+            install_replica(
+                parent, child,
+                setup_ca=setup_replica_cas,
+                setup_kra=setup_replica_kras
+            )
         installed.add(child)
     install_clients([master] + replicas, clients)
 
@@ -1181,3 +1191,41 @@ def replicas_cleanup(func):
                                             "host-del",
                                             host.hostname], raiseonerr=False)
     return wrapped
+
+
+def run_server_del(host, server_to_delete, force=False,
+                   ignore_topology_disconnect=False,
+                   ignore_last_of_role=False):
+    kinit_admin(host)
+    args = ['ipa', 'server-del', server_to_delete]
+    if force:
+        args.append('--force')
+    if ignore_topology_disconnect:
+        args.append('--ignore-topology-disconnect')
+    if ignore_last_of_role:
+        args.append('--ignore-last-of-role')
+
+    return host.run_command(args, raiseonerr=False)
+
+
+def run_certutil(host, args, reqdir, stdin=None, raiseonerr=True):
+    new_args = [paths.CERTUTIL, "-d", reqdir]
+    new_args = " ".join(new_args + args)
+    return host.run_command(new_args, raiseonerr=raiseonerr,
+                            stdin_text=stdin)
+
+
+def assert_error(result, stderr_text, returncode=None):
+    "Assert that `result` command failed and its stderr contains `stderr_text`"
+    assert stderr_text in result.stderr_text, result.stderr_text
+    if returncode is not None:
+        assert result.returncode == returncode
+    else:
+        assert result.returncode > 0
+
+
+def restart_named(*args):
+    time.sleep(20)  # give a time to DNSSEC daemons to provide keys for named
+    for host in args:
+        host.run_command(["systemctl", "restart", "named-pkcs11.service"])
+    time.sleep(20)  # give a time to named to be ready (zone loading)
