@@ -35,6 +35,8 @@ from ipatests.test_integration import tasks
 
 _DEFAULT = object()
 
+assert_error = tasks.assert_error
+
 
 def get_install_stdin(cert_passwords=()):
     lines = [
@@ -56,15 +58,6 @@ def get_replica_prepare_stdin(cert_passwords=()):
     return '\n'.join(lines + [''])
 
 
-def assert_error(result, stderr_text, returncode=None):
-    "Assert that `result` command failed and its stderr contains `stderr_text`"
-    assert stderr_text in result.stderr_text, result.stderr_text
-    if returncode:
-        assert result.returncode == returncode
-    else:
-        assert result.returncode > 0
-
-
 class CALessBase(IntegrationTest):
     @classmethod
     def install(cls, mh):
@@ -286,10 +279,10 @@ class CALessBase(IntegrationTest):
 
     @classmethod
     def get_pem(cls, nickname):
-        pem_cert, _stderr, _returncode = ipautil.run(
+        result = ipautil.run(
             ['certutil', '-L', '-d', 'nssdb', '-n', nickname, '-a'],
-            cwd=cls.cert_dir)
-        return pem_cert
+            cwd=cls.cert_dir, capture_output=True)
+        return result.output
 
     def verify_installation(self):
         """Verify CA cert PEM file and LDAP entry created by install
