@@ -98,18 +98,18 @@ class test_DefaultFrom(ClassChecker):
             pass
 
         o = self.cls(stuff)
-        assert repr(o) == "DefaultFrom(stuff, 'one', 'two')"
+        assert repr(o) == "DefaultFrom('one', 'two')"
 
         o = self.cls(stuff, 'aye', 'bee', 'see')
-        assert repr(o) == "DefaultFrom(stuff, 'aye', 'bee', 'see')"
+        assert repr(o) == "DefaultFrom('aye', 'bee', 'see')"
 
         cb = lambda first, last: first[0] + last
 
         o = self.cls(cb)
-        assert repr(o) == "DefaultFrom(<lambda>, 'first', 'last')"
+        assert repr(o) == "DefaultFrom('first', 'last')"
 
         o = self.cls(cb, 'aye', 'bee', 'see')
-        assert repr(o) == "DefaultFrom(<lambda>, 'aye', 'bee', 'see')"
+        assert repr(o) == "DefaultFrom('aye', 'bee', 'see')"
 
     def test_call(self):
         """
@@ -222,7 +222,6 @@ class test_Param(ClassChecker):
         assert o.exclude is None
         assert o.flags == frozenset()
         assert o.sortorder == 2
-        assert o.csv is False
 
         # Test that doc defaults from label:
         o = self.cls('my_param', doc=_('Hello world'))
@@ -288,10 +287,6 @@ class test_Param(ClassChecker):
             'exclude', frozenset(['client', 'bar']),
         )
 
-        # Test that ValueError is raised if csv is set and multivalue is not set:
-        e = raises(ValueError, self.cls, 'my_param', csv=True)
-        assert str(e) == '%s: cannot have csv without multivalue' % "Param('my_param')"
-
         # Test that default_from gets set:
         call = lambda first, last: first[0] + last
         o = self.cls('my_param', default_from=call)
@@ -306,9 +301,9 @@ class test_Param(ClassChecker):
             o = self.cls(name)
             assert repr(o) == 'Param(%r)' % name
         o = self.cls('name', required=False)
-        assert repr(o) == "Param('name', required=False)"
+        assert repr(o) == "Param('name?')"
         o = self.cls('name', multivalue=True)
-        assert repr(o) == "Param('name', multivalue=True)"
+        assert repr(o) == "Param('name+')"
 
     def test_use_in_context(self):
         """
@@ -357,17 +352,17 @@ class test_Param(ClassChecker):
         assert type(clone) is self.cls
         assert clone.name is orig.name
         for (key, kind, default) in self.cls.kwargs:
-            assert getattr(clone, key) is getattr(orig, key)
+            assert getattr(clone, key) == getattr(orig, key)
 
         # Test with a param spec:
         orig = self.cls('my_param*')
         assert orig.param_spec == 'my_param*'
         clone = orig.clone()
-        assert clone.param_spec == 'my_param'
+        assert clone.param_spec == 'my_param*'
         assert clone is not orig
         assert type(clone) is self.cls
         for (key, kind, default) in self.cls.kwargs:
-            assert getattr(clone, key) is getattr(orig, key)
+            assert getattr(clone, key) == getattr(orig, key)
 
         # Test with overrides:
         orig = self.cls('my_param*')
@@ -378,7 +373,7 @@ class test_Param(ClassChecker):
         assert type(clone) is self.cls
         assert clone.required is True
         assert clone.multivalue is True
-        assert clone.param_spec == 'my_param'
+        assert clone.param_spec == 'my_param+'
         assert clone.name == 'my_param'
 
     def test_clone_rename(self):
@@ -394,6 +389,8 @@ class test_Param(ClassChecker):
         assert type(clone) is self.cls
         assert clone.name == new_name
         for (key, kind, default) in self.cls.kwargs:
+            if key in ('cli_name', 'label', 'doc', 'cli_metavar'):
+                continue
             assert getattr(clone, key) is getattr(orig, key)
 
         # Test with overrides:
@@ -405,7 +402,7 @@ class test_Param(ClassChecker):
         assert type(clone) is self.cls
         assert clone.required is True
         assert clone.multivalue is True
-        assert clone.param_spec == new_name
+        assert clone.param_spec == "{0}+".format(new_name)
         assert clone.name == new_name
 
 
@@ -450,7 +447,7 @@ class test_Param(ClassChecker):
         assert o._convert_scalar(None) is None
         assert dummy.called() is False
         # Test with incorrect type
-        e = raises(errors.ConversionError, o._convert_scalar, 'hello', index=17)
+        e = raises(errors.ConversionError, o._convert_scalar, 'hello')
 
     def test_validate(self):
         """
@@ -459,38 +456,32 @@ class test_Param(ClassChecker):
 
         # Test in default state (with no rules, no kwarg):
         o = self.cls('my_param')
-        e = raises(errors.RequirementError, o.validate, None, 'cli')
+        e = raises(errors.RequirementError, o.validate, None)
         assert e.name == 'my_param'
 
-        # Test in default state that cli_name gets returned in the exception
-        # when context == 'cli'
-        o = self.cls('my_param', cli_name='short')
-        e = raises(errors.RequirementError, o.validate, None, 'cli')
-        assert e.name == 'short'
-
         # Test with required=False
         o = self.cls('my_param', required=False)
         assert o.required is False
-        assert o.validate(None, 'cli') is None
+        assert o.validate(None) is None
 
         # Test with query=True:
         o = self.cls('my_param', query=True)
         assert o.query is True
-        e = raises(errors.RequirementError, o.validate, None, 'cli')
-        assert_equal(e.name, 'my_param')
+        e = raises(errors.RequirementError, o.validate, None)
+        assert_equal(e.name, u'my_param')
 
         # Test with multivalue=True:
         o = self.cls('my_param', multivalue=True)
-        e = raises(TypeError, o.validate, [], 'cli')
+        e = raises(TypeError, o.validate, [])
         assert str(e) == TYPE_ERROR % ('value', tuple, [], list)
-        e = raises(ValueError, o.validate, tuple(), 'cli')
+        e = raises(ValueError, o.validate, tuple())
         assert str(e) == 'value: empty tuple must be converted to None'
 
         # Test with wrong (scalar) type:
-        e = raises(TypeError, o.validate, (None, None, 42, None), 'cli')
+        e = raises(TypeError, o.validate, (None, None, 42, None))
         assert str(e) == TYPE_ERROR % ('my_param', type(None), 42, int)
         o = self.cls('my_param')
-        e = raises(TypeError, o.validate, 'Hello', 'cli')
+        e = raises(TypeError, o.validate, 'Hello')
         assert str(e) == TYPE_ERROR % ('my_param', type(None), 'Hello', str)
 
         class Example(self.cls):
@@ -502,16 +493,15 @@ class test_Param(ClassChecker):
         fail = DummyRule(u'no good')
         o = Example('example', pass1, pass2)
         assert o.multivalue is False
-        assert o.validate(11, 'cli') is None
+        assert o.validate(11) is None
         assert pass1.calls == [(text.ugettext, 11)]
         assert pass2.calls == [(text.ugettext, 11)]
         pass1.reset()
         pass2.reset()
         o = Example('example', pass1, pass2, fail)
-        e = raises(errors.ValidationError, o.validate, 42, 'cli')
+        e = raises(errors.ValidationError, o.validate, 42)
         assert e.name == 'example'
         assert e.error == u'no good'
-        assert e.index is None
         assert pass1.calls == [(text.ugettext, 42)]
         assert pass2.calls == [(text.ugettext, 42)]
         assert fail.calls == [(text.ugettext, 42)]
@@ -522,7 +512,7 @@ class test_Param(ClassChecker):
         fail = DummyRule(u'this one is not good')
         o = Example('example', pass1, pass2, multivalue=True)
         assert o.multivalue is True
-        assert o.validate((3, 9), 'cli') is None
+        assert o.validate((3, 9)) is None
         assert pass1.calls == [
             (text.ugettext, 3),
             (text.ugettext, 9),
@@ -535,10 +525,9 @@ class test_Param(ClassChecker):
         pass2.reset()
         o = Example('multi_example', pass1, pass2, fail, multivalue=True)
         assert o.multivalue is True
-        e = raises(errors.ValidationError, o.validate, (3, 9), 'cli')
+        e = raises(errors.ValidationError, o.validate, (3, 9))
         assert e.name == 'multi_example'
         assert e.error == u'this one is not good'
-        assert e.index == 0
         assert pass1.calls == [(text.ugettext, 3)]
         assert pass2.calls == [(text.ugettext, 3)]
         assert fail.calls == [(text.ugettext, 3)]
@@ -555,14 +544,10 @@ class test_Param(ClassChecker):
         # Test that TypeError is appropriately raised:
         e = raises(TypeError, o._validate_scalar, 0)
         assert str(e) == TYPE_ERROR % ('my_param', bool, 0, int)
-        e = raises(TypeError, o._validate_scalar, 'Hi', index=4)
-        assert str(e) == TYPE_ERROR % ('my_param', bool, 'Hi', str)
-        e = raises(TypeError, o._validate_scalar, True, index=3.0)
-        assert str(e) == TYPE_ERROR % ('index', int, 3.0, float)
 
         # Test with passing rule:
-        assert o._validate_scalar(True, index=None) is None
-        assert o._validate_scalar(False, index=None) is None
+        assert o._validate_scalar(True) is None
+        assert o._validate_scalar(False) is None
         assert okay.calls == [
             (text.ugettext, True),
             (text.ugettext, False),
@@ -575,11 +560,9 @@ class test_Param(ClassChecker):
         e = raises(errors.ValidationError, o._validate_scalar, True)
         assert e.name == 'my_param'
         assert e.error == u'this describes the error'
-        assert e.index is None
-        e = raises(errors.ValidationError, o._validate_scalar, False, index=2)
+        e = raises(errors.ValidationError, o._validate_scalar, False)
         assert e.name == 'my_param'
         assert e.error == u'this describes the error'
-        assert e.index == 2
         assert okay.calls == [
             (text.ugettext, True),
             (text.ugettext, False),
@@ -937,17 +920,11 @@ class test_Str(ClassChecker):
         for value in bad:
             e = raises(errors.ConversionError, mthd, value)
             assert e.name == 'my_str'
-            assert e.index is None
-            assert_equal(unicode(e.error), u'must be Unicode text')
-            e = raises(errors.ConversionError, mthd, value, index=18)
-            assert e.name == 'my_str'
-            assert e.index == 18
             assert_equal(unicode(e.error), u'must be Unicode text')
         bad = [(u'Hello',), [42.3]]
         for value in bad:
             e = raises(errors.ConversionError, mthd, value)
             assert e.name == 'my_str'
-            assert e.index is None
             assert_equal(unicode(e.error), u'Only one value is allowed')
         assert o.convert(None) is None
 
@@ -1091,7 +1068,6 @@ class test_Password(ClassChecker):
         o = self.cls('my_password')
         e = raises(errors.PasswordMismatch, o._convert_scalar, [u'one', u'two'])
         assert e.name == 'my_password'
-        assert e.index is None
         assert o._convert_scalar([u'one', u'one']) == u'one'
         assert o._convert_scalar(u'one') == u'one'
 
@@ -1194,7 +1170,6 @@ def check_int_scalar_conversions(o):
     for bad in ['hello', u'hello', True, None, u'', u'.', 8j, ()]:
         e = raises(errors.ConversionError, o._convert_scalar, bad)
         assert e.name == 'my_number'
-        assert e.index is None
     # Assure large magnitude values are handled correctly
     assert type(o._convert_scalar(sys.maxsize * 2)) == long
     assert o._convert_scalar(sys.maxsize * 2) == sys.maxsize * 2
@@ -1556,7 +1531,7 @@ def test_create_param():
     for spec in ('one?', 'two+', 'three*', 'four'):
         (name, kw) = parameters.parse_param_spec(spec)
         p = f(spec)
-        assert p.param_spec is spec
+        assert p.param_spec == spec
         assert p.name == name
         assert p.required is kw['required']
         assert p.multivalue is kw['multivalue']
@@ -1604,11 +1579,10 @@ class test_IA5Str(ClassChecker):
         for value in bad:
             e = raises(errors.ConversionError, mthd, value)
             assert e.name == 'my_str'
-            assert e.index is None
             if six.PY2:
-                assert_equal(e.error, "The character '\\xc3' is not allowed.")
+                assert_equal(e.error, u"The character '\\xc3' is not allowed.")
             else:
-                assert_equal(e.error, "The character 'á' is not allowed.")
+                assert_equal(e.error, u"The character 'á' is not allowed.")
 
 
 class test_DateTime(ClassChecker):
