@@ -24,12 +24,9 @@ Test the `ipalib.plugable` module.
 # FIXME: Pylint errors
 # pylint: disable=no-member
 
-import inspect
-from ipatests.util import raises, no_set, no_del, read_only
-from ipatests.util import getitem, setitem, delitem
+from ipatests.util import raises, read_only
 from ipatests.util import ClassChecker, create_test_api
-from ipalib import plugable, errors, text
-from ipaplatform.paths import paths
+from ipalib import plugable, errors
 
 import pytest
 
@@ -55,9 +52,7 @@ class test_Plugin(ClassChecker):
         api = 'the api instance'
         o = self.cls(api)
         assert o.name == 'Plugin'
-        assert o.module == 'ipalib.plugable'
-        assert o.fullname == 'ipalib.plugable.Plugin'
-        assert isinstance(o.doc, text.Gettext)
+        assert isinstance(o.doc, str)
         class some_subclass(self.cls):
             """
             Do sub-classy things.
@@ -70,14 +65,13 @@ class test_Plugin(ClassChecker):
             """
         o = some_subclass(api)
         assert o.name == 'some_subclass'
-        assert o.module == __name__
-        assert o.fullname == '%s.some_subclass' % __name__
         assert o.summary == 'Do sub-classy things.'
-        assert isinstance(o.doc, text.Gettext)
+        assert isinstance(o.doc, str)
         class another_subclass(self.cls):
             pass
         o = another_subclass(api)
-        assert o.summary == '<%s>' % o.fullname
+        assert o.summary == u'<%s.%s>' % (another_subclass.__module__,
+                                          another_subclass.__name__)
 
         # Test that Plugin makes sure the subclass hasn't defined attributes
         # whose names conflict with the logger methods set in Plugin.__init__():
@@ -101,9 +95,9 @@ class test_Plugin(ClassChecker):
         assert o.__islocked__()
 
 
-def test_Registrar():
+def test_Registry():
     """
-    Test the `ipalib.plugable.Registrar` class
+    Test the `ipalib.plugable.Registry` class
     """
     class Base1(object):
         pass
@@ -118,59 +112,49 @@ def test_Registrar():
     class plugin3(Base3):
         pass
 
-    # Test creation of Registrar:
-    r = plugable.Registrar()
+    # Test creation of Registry:
+    r = plugable.Registry()
 
     # Check that TypeError is raised trying to register something that isn't
     # a class:
     p = plugin1()
-    e = raises(TypeError, r, p)
-    assert str(e) == 'plugin must be a class; got %r' % p
+    e = raises(TypeError, r(), p)
+    assert str(e) == 'plugin must be callable; got %r' % p
 
     # Check that registration works
-    r(plugin1)
-    assert len(r) == 1
-    assert plugin1 in r
-    assert r[plugin1] == dict(override=False)
+    r()(plugin1)
 
     # Check that DuplicateError is raised trying to register exact class
     # again:
-    e = raises(errors.PluginDuplicateError, r, plugin1)
+    e = raises(errors.PluginDuplicateError, r(), plugin1)
     assert e.plugin is plugin1
 
     # Check that overriding works
-    orig1 = plugin1
     class base1_extended(Base1):
         pass
     class plugin1(base1_extended):  # pylint: disable=function-redefined
         pass
-    r(plugin1, override=True)
-    assert len(r) == 2
-    assert plugin1 in r
-    assert r[plugin1] == dict(override=True)
+    r(override=True)(plugin1)
 
     # Test that another plugin can be registered:
-    r(plugin2)
-    assert len(r) == 3
-    assert plugin2 in r
-    assert r[plugin2] == dict(override=False)
+    r()(plugin2)
 
     # Setup to test more registration:
     class plugin1a(Base1):
         pass
-    r(plugin1a)
+    r()(plugin1a)
 
     class plugin1b(Base1):
         pass
-    r(plugin1b)
+    r()(plugin1b)
 
     class plugin2a(Base2):
         pass
-    r(plugin2a)
+    r()(plugin2a)
 
     class plugin2b(Base2):
         pass
-    r(plugin2b)
+    r()(plugin2b)
 
 
 class test_API(ClassChecker):
@@ -246,7 +230,7 @@ class test_API(ClassChecker):
             base_name = get_base_name(b)
             base = locals()[base_name]
             ns = getattr(api, base_name)
-            assert isinstance(ns, plugable.NameSpace)
+            assert isinstance(ns, plugable.APINameSpace)
             assert read_only(api, base_name) is ns
             assert len(ns) == 3
             for p in range(3):
@@ -256,7 +240,6 @@ class test_API(ClassChecker):
                 assert isinstance(inst, base)
                 assert isinstance(inst, plugin)
                 assert inst.name == plugin_name
-                assert read_only(ns, plugin_name) is inst
                 assert inst.method(7) == 7 + b
 
         # Test that calling finilize again raises AssertionError:
