@@ -23,6 +23,7 @@ DNS tests
 
 from ipatests.test_webui.ui_driver import UI_driver
 from ipatests.test_webui.ui_driver import screenshot
+import pytest
 
 ZONE_ENTITY = 'dnszone'
 FORWARD_ZONE_ENTITY = 'dnsforwardzone'
@@ -37,9 +38,6 @@ ZONE_DATA = {
     'pkey': ZONE_PKEY,
     'add': [
         ('textbox', 'idnsname', ZONE_PKEY),
-        ('textbox', 'idnssoamname', 'ns'),
-        ('textbox', 'ip_address', '192.168.1.1'),
-        ('checkbox', 'force', 'checked'),
     ],
     'mod': [
         ('checkbox', 'idnsallowsyncptr', 'checked'),
@@ -88,10 +86,11 @@ CONFIG_MOD_DATA = {
 }
 
 
+@pytest.mark.tier1
 class test_dns(UI_driver):
 
-    def __init__(self, *args, **kwargs):
-        super(test_dns, self).__init__(args, kwargs)
+    def setup(self, *args, **kwargs):
+        super(test_dns, self).setup(*args, **kwargs)
 
         if not self.has_dns():
             self.skip('DNS not configured')
