@@ -615,12 +615,17 @@ class UI_driver(object):
         s = "a[name='%s'].action-button" % name
         self._button_click(s, parent, name)
 
-    def button_click(self, name, parent=None):
+    def button_click(self, name, parent=None,
+                     parents_css_sel=None):
         """
         Click on .ui-button
         """
         if not parent:
-            parent = self.get_form()
+            if parents_css_sel:
+                parent = self.find(parents_css_sel, By.CSS_SELECTOR,
+                                   strict=True)
+            else:
+                parent = self.get_form()
 
         s = "[name='%s'].btn" % name
         self._button_click(s, parent, name)
@@ -938,12 +943,8 @@ class UI_driver(object):
         s = self.get_table_selector(table_name)
         input_s = s + " tbody td input[value='%s']" % pkey
         checkbox = self.find(input_s, By.CSS_SELECTOR, parent, strict=True)
-        checkbox_id = checkbox.get_attribute('id')
-        label_s = s + " tbody td label[for='%s']" % checkbox_id
-        print(label_s)
-        label = self.find(label_s, By.CSS_SELECTOR, parent, strict=True)
         try:
-            ActionChains(self.driver).move_to_element(label).click().perform()
+            ActionChains(self.driver).move_to_element(checkbox).click().perform()
         except WebDriverException as e:
             assert False, 'Can\'t click on checkbox label: %s \n%s' % (s, e)
         self.wait()
@@ -992,7 +993,9 @@ class UI_driver(object):
         link.click()
         self.wait_for_request()
 
-    def delete_record(self, pkeys, fields=None, parent=None, table_name=None):
+    def delete_record(
+            self, pkeys, fields=None, parent=None, table_name=None,
+            facet_btn='remove'):
         """
         Delete records with given pkeys in currently opened search table.
         """
@@ -1012,9 +1015,9 @@ class UI_driver(object):
             if table_name and parent:
                 s = self.get_table_selector(table_name)
                 table = self.find(s, By.CSS_SELECTOR, parent, strict=True)
-                self.button_click('remove', table)
+                self.button_click(facet_btn, table)
             else:
-                self.facet_button_click('remove')
+                self.facet_button_click(facet_btn)
             if fields:
                 self.fill_fields(fields)
             self.dialog_button_click('ok')
@@ -1291,7 +1294,6 @@ class UI_driver(object):
                         )
 
         # Find
-
         self.find_record(parent_entity, data, search_facet)
 
         # 3. Navigate to details facet
@@ -1334,14 +1336,15 @@ class UI_driver(object):
         self.dialog_button_click('add')
         self.wait_for_request()
 
-    def add_associations(self, pkeys, facet=None, delete=False):
+    def prepare_associations(
+            self, pkeys, facet=None, facet_btn='add', member_pkeys=None):
         """
-        Add associations
+        Helper function for add_associations and delete_associations
         """
         if facet:
             self.switch_to_facet(facet)
 
-        self.facet_button_click('add')
+        self.facet_button_click(facet_btn)
         self.wait()
         self.wait_for_request()
 
@@ -1352,12 +1355,39 @@ class UI_driver(object):
         self.dialog_button_click('add')
         self.wait_for_request()
 
-        for key in pkeys:
+        if member_pkeys:
+            check_pkeys = member_pkeys
+        else:
+            check_pkeys = pkeys
+
+        return check_pkeys
+
+    def add_associations(
+            self, pkeys, facet=None, delete=False, facet_btn='add',
+            member_pkeys=None):
+        """
+        Add associations
+        """
+        check_pkeys = self.prepare_associations(
+            pkeys, facet, facet_btn, member_pkeys)
+
+        for key in check_pkeys:
             self.assert_record(key)
             if delete:
                 self.delete_record(key)
                 self.assert_record(key, negative=True)
 
+    def delete_associations(
+            self, pkeys, facet=None, facet_btn='remove', member_pkeys=None):
+        """
+        Remove associations
+        """
+        check_pkeys = self.prepare_associations(
+            pkeys, facet, facet_btn, member_pkeys)
+
+        for key in check_pkeys:
+            self.assert_record(key, negative=True)
+
     def add_table_associations(self, table_name, pkeys, parent=False, delete=False):
         """
         Add value to table (association|rule|...)
@@ -1388,14 +1418,25 @@ class UI_driver(object):
             for key in pkeys:
                 self.assert_record(key, parent, table_name, negative=True)
 
-    def action_list_action(self, name, confirm=True, confirm_btn="ok"):
+    def action_list_action(self, name, confirm=True, confirm_btn="ok",
+                           parents_css_sel=None):
         """
         Execute action list action
         """
-        cont = self.find(".active-facet .facet-actions", By.CSS_SELECTOR, strict=True)
-        expand = self.find(".dropdown-toggle", By.CSS_SELECTOR, cont, strict=True)
+        context = None
+
+        if not parents_css_sel:
+            context = self.find(".active-facet .facet-actions",
+                                By.CSS_SELECTOR, strict=True)
+        else:
+            context = self.find(parents_css_sel, By.CSS_SELECTOR,
+                                strict=True)
+
+        expand = self.find(".dropdown-toggle", By.CSS_SELECTOR, context,
+                           strict=True)
         expand.click()
-        action_link = self.find("li[data-name=%s] a" % name, By.CSS_SELECTOR, cont, strict=True)
+        action_link = self.find("li[data-name=%s] a" % name, By.CSS_SELECTOR,
+                                context, strict=True)
         action_link.click()
         if confirm:
             self.wait(0.5)  # wait for dialog
@@ -1432,11 +1473,11 @@ class UI_driver(object):
         self.assert_no_error_dialog()
         self.assert_class(title, 'disabled')
 
-    def delete_action(self, entity, pkey, facet='search'):
+    def delete_action(self, entity, pkey, action='delete', facet='search'):
         """
         Execute and test 'delete' action panel action.
         """
-        self.action_list_action('delete')
+        self.action_list_action(action)
         self.wait_for_request(n=4)
         self.assert_no_error_dialog()
         self.assert_facet(entity, facet)
@@ -1525,6 +1566,17 @@ class UI_driver(object):
         s = "div[name='%s'] %s[name='%s']" % (name, element, name)
         self.assert_text(s, value, parent)
 
+    def assert_empty_value(self, selector, parent=None, negative=False):
+        """
+        Assert empty value of some field in details page or in a form
+        """
+        value = self.get_value(selector, parent)
+
+        if negative:
+            assert not value == ''
+        else:
+            assert value == ''
+
     def assert_no_dialog(self):
         """
         Assert that no dialog is opened
@@ -1714,17 +1766,26 @@ class UI_driver(object):
             assert is_enabled == enabled, ('Invalid enabled state of action button %s. '
                                            'Expected: %s') % (action, str(visible))
 
-    def assert_action_list_action(self, action, visible=True, enabled=True, parent=None):
+    def assert_action_list_action(self, action, visible=True, enabled=True,
+                                  parent=None, parents_css_sel=None,
+                                  facet_actions=True):
         """
         Assert that action dropdown action is visible/hidden, and enabled/disabled
 
         Enabled is checked only if action is visible.
         """
+
+        li_s = " li[data-name='%s']" % action
+
         if not parent:
             parent = self.get_form()
 
-        s = ".facet-actions li[data-name='%s']" % action
-        li = self.find(s, By.CSS_SELECTOR, parent)
+        if facet_actions:
+            li_s = ".facet-actions" + li_s
+        else:
+            li_s = parents_css_sel + li_s
+
+        li = self.find(li_s, By.CSS_SELECTOR, parent)
         link = self.find("a", By.CSS_SELECTOR, li)
 
         is_visible = li is not None and link is not None
