@@ -17,26 +17,26 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-Test the `ipalib/plugins/cert.py` module against a RA.
+Test the `ipaserver/plugins/cert.py` module against a RA.
 """
+from __future__ import print_function
 
-import sys
+import base64
+import nose
 import os
+import pytest
 import shutil
-from nose.tools import raises, assert_raises  # pylint: disable=E0611
-
-from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test, assert_attr_equal
+import six
+import tempfile
 from ipalib import api
 from ipalib import errors
 from ipalib import x509
-import tempfile
-from ipapython import ipautil
-import six
-import nose
-import base64
 from ipaplatform.paths import paths
+from ipapython import ipautil
 from ipapython.dn import DN
-import pytest
+from ipapython.ipautil import run
+from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test
+from nose.tools import raises, assert_raises
 
 if six.PY3:
     unicode = str
@@ -44,6 +44,11 @@ if six.PY3:
 # So we can save the cert from issuance and compare it later
 cert = None
 newcert = None
+sn = None
+
+_DOMAIN = api.env.domain
+_EXP_CRL_URI = ''.join(['http://ipa-ca.', _DOMAIN, '/ipa/crl/MasterCRL.bin'])
+_EXP_OCSP_URI = ''.join(['http://ipa-ca.', _DOMAIN, '/ca/ocsp'])
 
 def is_db_configured():
     """
@@ -73,15 +78,16 @@ def is_db_configured():
 # running as the lite-server.
 
 
-@pytest.mark.tier1
-class test_cert(XMLRPC_test):
+class BaseCert(XMLRPC_test):
 
     @classmethod
     def setup_class(cls):
-        super(test_cert, cls).setup_class()
+        super(BaseCert, cls).setup_class()
 
         if 'cert_request' not in api.Command:
             raise nose.SkipTest('cert_request not registered')
+        if 'cert_show' not in api.Command:
+            raise nose.SkipTest('cert_show not registered')
 
         is_db_configured()
 
@@ -94,6 +100,7 @@ class test_cert(XMLRPC_test):
         self.reqdir = tempfile.mkdtemp(prefix = "tmp-")
         self.reqfile = self.reqdir + "/test.csr"
         self.pwname = self.reqdir + "/pwd"
+        self.certfile = self.reqdir + "/cert.crt"
 
         # Create an empty password file
         fp = open(self.pwname, "w")
@@ -120,11 +127,20 @@ class test_cert(XMLRPC_test):
         fp.close()
         return data
 
+    host_fqdn = u'ipatestcert.%s' % api.env.domain
+    service_princ = u'test/%s@%s' % (host_fqdn, api.env.realm)
+
+
+@pytest.mark.tier1
+class test_cert(BaseCert):
+
+    @classmethod
+    def setup_class(cls):
+        super(test_cert, cls).setup_class()
+
     """
     Test the `cert` plugin.
     """
-    host_fqdn = u'ipatestcert.%s' % api.env.domain
-    service_princ = u'test/%s@%s' % (host_fqdn, api.env.realm)
 
     def test_0001_cert_add(self):
         """
@@ -144,13 +160,16 @@ class test_cert(XMLRPC_test):
         Test the `xmlrpc.cert_request` method with --add.
         """
         # Our host should exist from previous test
-        global cert
+        global cert, sn
 
         csr = unicode(self.generateCSR(str(self.subject)))
         res = api.Command['cert_request'](csr, principal=self.service_princ, add=True)['result']
         assert DN(res['subject']) == self.subject
+        assert 'cacn' in res
         # save the cert for the service_show/find tests
         cert = res['certificate'].encode('ascii')
+        # save cert's SN for URI test
+        sn = res['serial_number']
 
     def test_0003_service_show(self):
         """
@@ -171,7 +190,20 @@ class test_cert(XMLRPC_test):
         res = api.Command['service_find'](self.service_princ)['result']
         assert base64.b64encode(res[0]['usercertificate'][0]) == cert
 
-    def test_0005_cert_renew(self):
+    def test_0005_cert_uris(self):
+        """Test URI details and OCSP-URI in certificate.
+
+        See https://fedorahosted.org/freeipa/ticket/5881
+        """
+        result = api.Command.cert_show(sn, out=unicode(self.certfile))
+        with open(self.certfile, "r") as f:
+            pem_cert = unicode(f.read())
+        result = run(['openssl', 'x509', '-text'],
+                     stdin=pem_cert, capture_output=True)
+        assert _EXP_CRL_URI in result.output
+        assert _EXP_OCSP_URI in result.output
+
+    def test_0006_cert_renew(self):
         """
         Issue a new certificate for a service
         """
@@ -183,7 +215,7 @@ class test_cert(XMLRPC_test):
         # save the cert for the service_show/find tests
         newcert = res['certificate'].encode('ascii')
 
-    def test_0006_service_show(self):
+    def test_0007_service_show(self):
         """
         Verify the new certificate with service-show.
         """
@@ -192,10 +224,31 @@ class test_cert(XMLRPC_test):
         res = api.Command['service_show'](self.service_princ)['result']
 
         # Both the old and the new certs should be listed as certificates now
-        certs_encoded = (base64.b64encode(cert) for cert in res['usercertificate'])
+        certs_encoded = (
+            base64.b64encode(usercert) for usercert in res['usercertificate']
+        )
         assert set(certs_encoded) == set([cert, newcert])
 
-    def test_0007_cleanup(self):
+    def test_0008_cert_show(self):
+        """
+        Verify that cert-show shows CA of the certificate without --all
+        """
+        res = api.Command['cert_show'](sn)['result']
+        assert 'cacn' in res
+        assert 'valid_not_before' in res
+        assert 'valid_not_after' in res
+
+    def test_0009_cert_find(self):
+        """
+        Verify that cert-find shows CA of the certificate without --all
+        """
+        res = api.Command['cert_find'](min_serial_number=sn,
+                                       max_serial_number=sn)['result'][0]
+        assert 'cacn' in res
+        assert 'valid_not_before' in res
+        assert 'valid_not_after' in res
+
+    def test_00010_cleanup(self):
         """
         Clean up cert test data
         """
@@ -271,80 +324,7 @@ class test_cert_find(XMLRPC_test):
         res = api.Command['cert_find'](subject=self.short, exactly=True)
         assert 'count' in res and res['count'] == 0
 
-    def test_0007_find_revocation_reason_0(self):
-        """
-        Find all certificates with revocation reason 0
-        """
-        res = api.Command['cert_find'](revocation_reason=0)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0008_find_revocation_reason_1(self):
-        """
-        Find all certificates with revocation reason 1
-        """
-        res = api.Command['cert_find'](revocation_reason=1)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0009_find_revocation_reason_2(self):
-        """
-        Find all certificates with revocation reason 2
-        """
-        res = api.Command['cert_find'](revocation_reason=2)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0010_find_revocation_reason_3(self):
-        """
-        Find all certificates with revocation reason 3
-        """
-        res = api.Command['cert_find'](revocation_reason=3)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0011_find_revocation_reason_4(self):
-        """
-        Find all certificates with revocation reason 4
-
-        There is no way to know in advance how many revoked certificates
-        we'll have but in the context of make-test we'll have at least one.
-        """
-        res = api.Command['cert_find'](revocation_reason=4)
-        assert 'count' in res and res['count'] >= 1
-
-    def test_0012_find_revocation_reason_5(self):
-        """
-        Find all certificates with revocation reason 5
-        """
-        res = api.Command['cert_find'](revocation_reason=5)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0013_find_revocation_reason_6(self):
-        """
-        Find all certificates with revocation reason 6
-        """
-        res = api.Command['cert_find'](revocation_reason=6)
-        assert 'count' in res and res['count'] == 0
-
-    # There is no revocation reason #7
-
-    def test_0014_find_revocation_reason_8(self):
-        """
-        Find all certificates with revocation reason 8
-        """
-        res = api.Command['cert_find'](revocation_reason=8)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0015_find_revocation_reason_9(self):
-        """
-        Find all certificates with revocation reason 9
-        """
-        res = api.Command['cert_find'](revocation_reason=9)
-        assert 'count' in res and res['count'] == 0
-
-    def test_0016_find_revocation_reason_10(self):
-        """
-        Find all certificates with revocation reason 10
-        """
-        res = api.Command['cert_find'](revocation_reason=10)
-        assert 'count' in res and res['count'] == 0
+    # tests 0007 to 0016 removed
 
     def test_0017_find_by_issuedon(self):
         """
@@ -430,8 +410,9 @@ class test_cert_find(XMLRPC_test):
         """
         Search with a sizelimit of 0
         """
+        count_all = api.Command['cert_find']()['count']
         res = api.Command['cert_find'](sizelimit=0)
-        assert 'count' in res and res['count'] == 0
+        assert 'count' in res and res['count'] == count_all
 
     @raises(errors.ValidationError)
     def test_0028_find_negative_size(self):
@@ -454,9 +435,71 @@ class test_cert_find(XMLRPC_test):
         res = api.Command['cert_find'](subject=u'ipatestcert.%s' % api.env.domain)
         assert 'count' in res and res['count'] >= 1
 
-    @raises(errors.ValidationError)
+    @raises(errors.ConversionError)
     def test_0031_search_on_invalid_date(self):
         """
         Search using invalid date format
         """
         res = api.Command['cert_find'](issuedon_from=u'xyz')
+
+
+@pytest.mark.tier1
+class test_cert_revocation(BaseCert):
+
+    @classmethod
+    def setup_class(cls):
+        super(test_cert_revocation, cls).setup_class()
+
+    # create CSR, request cert, revoke cert, check cert attributes
+    def revoke_cert(self, reason):
+        # add host
+        assert 'result' in api.Command['host_add'](self.host_fqdn, force=True)
+
+        # generate CSR, request certificate, obtain serial number
+        self.csr = unicode(self.generateCSR(str(self.subject)))
+        res = api.Command['cert_request'](self.csr,
+                                          principal=self.service_princ,
+                                          add=True, all=True)['result']
+        serial_number = res['serial_number']
+
+        # revoke created certificate
+        assert 'result' in api.Command['cert_revoke'](
+            serial_number, revocation_reason=reason)
+
+        # verify that certificate is revoked with correct reason
+        res2 = api.Command['cert_show'](serial_number, all=True)['result']
+        assert res2['revoked']
+        assert res2['revocation_reason'] == reason
+
+        # remove host
+        assert 'result' in api.Command['host_del'](self.host_fqdn)
+
+    def test_revoke_with_reason_0(self):
+        self.revoke_cert(0)
+
+    def test_revoke_with_reason_1(self):
+        self.revoke_cert(1)
+
+    def test_revoke_with_reason_2(self):
+        self.revoke_cert(2)
+
+    def test_revoke_with_reason_3(self):
+        self.revoke_cert(3)
+
+    def test_revoke_with_reason_4(self):
+        self.revoke_cert(4)
+
+    def test_revoke_with_reason_5(self):
+        self.revoke_cert(5)
+
+    def test_revoke_with_reason_6(self):
+        self.revoke_cert(6)
+
+    def test_revoke_with_reason_8(self):
+        self.revoke_cert(8)
+
+    def test_revoke_with_reason_9(self):
+        self.revoke_cert(9)
+
+    def test_revoke_with_reason_10(self):
+        self.revoke_cert(10)
