@@ -18,10 +18,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-Test the `ipalib/plugins/hbactest.py` module.
+Test the `ipaserver/plugins/hbactest.py` module.
 """
 
-from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test, assert_attr_equal
+from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test
 from ipalib import api
 from ipalib import errors
 from nose.tools import raises
