@@ -42,6 +42,7 @@ from ipatests.test_xmlrpc import objectclasses
 from ipatests.test_xmlrpc.tracker.host_plugin import HostTracker
 from ipatests.test_xmlrpc.testcert import get_testcert
 from ipatests.util import assert_deepequal
+from ipaplatform.paths import paths
 
 # Constants DNS integration tests
 # TODO: Use tracker fixtures for zones/records/users/groups
@@ -357,6 +358,7 @@ class TestHostWithService(XMLRPC_test):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     objectclass=objectclasses.service,
                     managedby_host=[host.fqdn],
                     ipauniqueid=[fuzzy_uuid],
@@ -417,7 +419,7 @@ class TestManagedHosts(XMLRPC_test):
             count=1,
             truncated=False,
             summary=u'1 host matched',
-            result=[host.filter_attrs(host.retrieve_keys)],
+            result=[host.filter_attrs(host.find_keys)],
         ), result)
 
     def search_man_hosts(self, host1, host2):
@@ -519,15 +521,22 @@ class TestHostFalsePwdChange(XMLRPC_test):
     def test_join_host(self, host, keytabname):
         """
         Create a test host and join it into IPA.
+
+        This test must not run remotely.
         """
 
-        join_command = 'ipa-client/ipa-join'
-        if not os.path.isfile(join_command):
-            pytest.skip("Command '%s' not found" % join_command)
+        if not os.path.isfile(paths.SBIN_IPA_JOIN):
+            pytest.skip("Command '%s' not found. "
+                        "The test must not run remotely."
+                        % paths.SBIN_IPA_JOIN)
 
         # create a test host with bulk enrollment password
         host.track_create()
+
+        # manipulate host.attrs to correspond with real attributes of host
+        # after creating it with random password
         del host.attrs['krbprincipalname']
+        del host.attrs['krbcanonicalname']
         host.attrs['has_password'] = True
         objclass = list(set(
             host.attrs['objectclass']) - {u'krbprincipal', u'krbprincipalaux'})
@@ -544,7 +553,7 @@ class TestHostFalsePwdChange(XMLRPC_test):
 
         # joint the host with the bulk password
         new_args = [
-            join_command,
+            paths.SBIN_IPA_JOIN,
             "-s", host.api.env.host,
             "-h", host.fqdn,
             "-k", keytabname,
@@ -559,10 +568,13 @@ class TestHostFalsePwdChange(XMLRPC_test):
             # the keytab is not necessary for further tests
             print(e)
 
+        # fix host.attrs again to correspond with current state
         host.attrs['has_keytab'] = True
         host.attrs['has_password'] = False
         host.attrs['krbprincipalname'] = [u'host/%s@%s' % (host.fqdn,
                                                            host.api.env.realm)]
+        host.attrs['krbcanonicalname'] = [u'host/%s@%s' % (host.fqdn,
+                                                           host.api.env.realm)]
         host.retrieve()
 
         # Try to change the password of enrolled host with specified password
