@@ -42,11 +42,6 @@ if six.PY3:
 idview1 = u'idview1'
 idview2 = u'idview2'
 
-host1 = u'host1.test'
-host2 = u'host2.test'
-host3 = u'host3.test'
-host4 = u'host4.test'
-
 hostgroup1 = u'hostgroup1'
 hostgroup2 = u'hostgroup2'
 
@@ -804,6 +799,8 @@ class test_idviews(Declarative):
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[
                         u'host/%s@%s' % (get_fqdn(host1), api.env.realm)],
+                    krbcanonicalname=[
+                        u'host/%s@%s' % (get_fqdn(host1), api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[get_fqdn(host1)],
@@ -832,6 +829,8 @@ class test_idviews(Declarative):
                     l=[u'Undisclosed location 2'],
                     krbprincipalname=[
                         u'host/%s@%s' % (get_fqdn(host2), api.env.realm)],
+                    krbcanonicalname=[
+                        u'host/%s@%s' % (get_fqdn(host2), api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[get_fqdn(host2)],
@@ -860,6 +859,8 @@ class test_idviews(Declarative):
                     l=[u'Undisclosed location 3'],
                     krbprincipalname=[
                         u'host/%s@%s' % (get_fqdn(host3), api.env.realm)],
+                    krbcanonicalname=[
+                        u'host/%s@%s' % (get_fqdn(host3), api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[get_fqdn(host3)],
@@ -1016,6 +1017,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 3'],
                     l=[u'Undisclosed location 3'],
                     krbprincipalname=[get_host_principal(host3)],
+                    krbcanonicalname=[get_host_principal(host3)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host3)],
@@ -1026,6 +1028,12 @@ class test_idviews(Declarative):
                     objectclass=objectclasses.host,
                     serverhostname=[host3],
                     ipaassignedidview=[idview1],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
@@ -1043,6 +1051,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
                     krbprincipalname=[get_host_principal(host2)],
+                    krbcanonicalname=[get_host_principal(host2)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host2)],
@@ -1054,6 +1063,12 @@ class test_idviews(Declarative):
                     serverhostname=[host2],
                     memberof_hostgroup=[hostgroup2],
                     memberofindirect_hostgroup=[hostgroup1],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
@@ -1094,6 +1109,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
                     krbprincipalname=[get_host_principal(host2)],
+                    krbcanonicalname=[get_host_principal(host2)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host2)],
@@ -1106,6 +1122,12 @@ class test_idviews(Declarative):
                     memberof_hostgroup=[hostgroup2],
                     memberofindirect_hostgroup=[hostgroup1],
                     ipaassignedidview=[idview1],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
@@ -1123,6 +1145,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[get_host_principal(host1)],
+                    krbcanonicalname=[get_host_principal(host1)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host1)],
@@ -1134,6 +1157,12 @@ class test_idviews(Declarative):
                     serverhostname=[host1],
                     memberof_hostgroup=[hostgroup1],
                     ipaassignedidview=[idview1],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
@@ -1195,6 +1224,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[get_host_principal(host1)],
+                    krbcanonicalname=[get_host_principal(host1)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host1)],
@@ -1205,6 +1235,12 @@ class test_idviews(Declarative):
                     objectclass=objectclasses.host,
                     serverhostname=[host1],
                     memberof_hostgroup=[hostgroup1],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
@@ -1222,6 +1258,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 3'],
                     l=[u'Undisclosed location 3'],
                     krbprincipalname=[get_host_principal(host3)],
+                    krbcanonicalname=[get_host_principal(host3)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host3)],
@@ -1231,6 +1268,12 @@ class test_idviews(Declarative):
                     managing_host=[get_fqdn(host3)],
                     objectclass=objectclasses.host,
                     serverhostname=[host3],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
@@ -1441,6 +1484,8 @@ class test_idviews(Declarative):
                     l=[u'Undisclosed location 4'],
                     krbprincipalname=[
                         u'host/%s@%s' % (get_fqdn(host4), api.env.realm)],
+                    krbcanonicalname=[
+                        u'host/%s@%s' % (get_fqdn(host4), api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[get_fqdn(host4)],
@@ -1473,6 +1518,7 @@ class test_idviews(Declarative):
                     description=[u'Test host 4'],
                     l=[u'Undisclosed location 4'],
                     krbprincipalname=[get_host_principal(host4)],
+                    krbcanonicalname=[get_host_principal(host4)],
                     has_keytab=False,
                     has_password=False,
                     managedby_host=[get_fqdn(host4)],
@@ -1482,6 +1528,12 @@ class test_idviews(Declarative):
                     managing_host=[get_fqdn(host4)],
                     objectclass=objectclasses.host,
                     serverhostname=[host4],
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Host Password Policy',
+                        api.env.container_host,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
