@@ -18,17 +18,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-Test the `ipalib/plugins/service.py` module.
+Test the `ipaserver/plugins/service.py` module.
 """
 
 from ipalib import api, errors, x509
 from ipatests.test_xmlrpc.xmlrpc_test import Declarative, fuzzy_uuid, fuzzy_hash
 from ipatests.test_xmlrpc.xmlrpc_test import fuzzy_digits, fuzzy_date, fuzzy_issuer
-from ipatests.test_xmlrpc.xmlrpc_test import fuzzy_hex
+from ipatests.test_xmlrpc.xmlrpc_test import fuzzy_hex, XMLRPC_test
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.test_xmlrpc.testcert import get_testcert
-from ipatests.test_xmlrpc.test_user_plugin import (
-    get_user_result, get_user_dn, get_group_dn)
+from ipatests.test_xmlrpc.test_user_plugin import get_user_result, get_group_dn
+
+from ipatests.test_xmlrpc.tracker.service_plugin import ServiceTracker
+from ipatests.test_xmlrpc.tracker.host_plugin import HostTracker
+
 import base64
 from ipapython.dn import DN
 import pytest
@@ -49,7 +52,20 @@ role1_dn = DN(('cn', role1), api.env.container_rolegroup, api.env.basedn)
 
 servercert= get_testcert(DN(('CN', api.env.host), x509.subject_base()),
                          'unittest/%s@%s' % (api.env.host, api.env.realm))
-badservercert = 'MIICbzCCAdigAwIBAgICA/4wDQYJKoZIhvcNAQEFBQAwKTEnMCUGA1UEAxMeSVBBIFRlc3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5MB4XDTEwMDgwOTE1MDIyN1oXDTIwMDgwOTE1MDIyN1owKTEMMAoGA1UEChMDSVBBMRkwFwYDVQQDExBwdW1hLmdyZXlvYWsuY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwYbfEOQPgGenPn9vt1JFKvWm/Je3y2tawGWA3LXDuqfFJyYtZ8ib3TcBUOnLk9WK5g2qCwHaNlei7bj8ggIfr5hegAVe10cun+wYErjnYo7hsHYd+57VZezeipWrXu+7NoNd4+c4A5lk4A/xJay9j3bYx2oOM8BEox4xWYoWge1ljPrc5JK46f0X7AGW4F2VhnKPnf8rwSuzI1U8VGjutyM9TWNy3m9KMWeScjyG/ggIpOjUDMV7HkJL0Di61lznR9jXubpiEC7gWGbTp84eGl/Nn9bgK1AwHfJ2lHwfoY4uiL7ge1gyP6EvuUlHoBzdb7pekiX28iePjW3iEG9IawIDAQABoyIwIDARBglghkgBhvhCAQEEBAMCBkAwCwYDVR0PBAQDAgUgMA0GCSqGSIb3DQEBBQUAA4GBACRESLemRV9BPxfEgbALuxH5oE8jQm8WZ3pm2pALbpDlAd9wQc3yVf6RtkfVthyDnM18bg7IhxKpd77/p3H8eCnS8w5MLVRda6ktUC6tGhFTS4QKAf0WyDGTcIgkXbeDw0OPAoNHivoXbIXIIRxlw/XgaSaMzJQDBG8iROsN4kCv'
+randomissuercert = (
+    "MIICbzCCAdigAwIBAgICA/4wDQYJKoZIhvcNAQEFBQAwKTEnMCUGA1UEAxMeSVBBIFRlc3Q"
+    "gQ2VydGlmaWNhdGUgQXV0aG9yaXR5MB4XDTEwMDgwOTE1MDIyN1oXDTIwMDgwOTE1MDIyN1"
+    "owKTEMMAoGA1UEChMDSVBBMRkwFwYDVQQDExBwdW1hLmdyZXlvYWsuY29tMIIBIjANBgkqh"
+    "kiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwYbfEOQPgGenPn9vt1JFKvWm/Je3y2tawGWA3LXD"
+    "uqfFJyYtZ8ib3TcBUOnLk9WK5g2qCwHaNlei7bj8ggIfr5hegAVe10cun+wYErjnYo7hsHY"
+    "d+57VZezeipWrXu+7NoNd4+c4A5lk4A/xJay9j3bYx2oOM8BEox4xWYoWge1ljPrc5JK46f"
+    "0X7AGW4F2VhnKPnf8rwSuzI1U8VGjutyM9TWNy3m9KMWeScjyG/ggIpOjUDMV7HkJL0Di61"
+    "lznR9jXubpiEC7gWGbTp84eGl/Nn9bgK1AwHfJ2lHwfoY4uiL7ge1gyP6EvuUlHoBzdb7pe"
+    "kiX28iePjW3iEG9IawIDAQABoyIwIDARBglghkgBhvhCAQEEBAMCBkAwCwYDVR0PBAQDAgU"
+    "gMA0GCSqGSIb3DQEBBQUAA4GBACRESLemRV9BPxfEgbALuxH5oE8jQm8WZ3pm2pALbpDlAd"
+    "9wQc3yVf6RtkfVthyDnM18bg7IhxKpd77/p3H8eCnS8w5MLVRda6ktUC6tGhFTS4QKAf0Wy"
+    "DGTcIgkXbeDw0OPAoNHivoXbIXIIRxlw/XgaSaMzJQDBG8iROsN4kCv")
+randomissuer = DN(('CN', 'puma.greyoak.com'), 'O=IPA')
 
 user1 = u'tuser1'
 user2 = u'tuser2'
@@ -115,6 +131,7 @@ class test_service(Declarative):
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
+                    krbcanonicalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[u'%s' % fqdn1],
@@ -143,6 +160,7 @@ class test_service(Declarative):
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
                     krbprincipalname=[u'host/%s@%s' % (fqdn2, api.env.realm)],
+                    krbcanonicalname=[u'host/%s@%s' % (fqdn2, api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[u'%s' % fqdn2],
@@ -171,6 +189,8 @@ class test_service(Declarative):
                     description=[u'Test host 3'],
                     l=[u'Undisclosed location 3'],
                     krbprincipalname=[u'host/%s@%s' % (fqdn3.lower(), api.env.realm)],
+                    krbcanonicalname=[u'host/%s@%s' % (
+                        fqdn3.lower(), api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[u'%s' % fqdn3.lower()],
@@ -194,6 +214,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     objectclass=objectclasses.service,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[fqdn1],
@@ -223,6 +244,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     has_keytab=False,
                     managedby_host=[fqdn1],
                 ),
@@ -240,17 +262,44 @@ class test_service(Declarative):
                     dn=service1dn,
                     krbprincipalname=[service1],
                     ipakrbprincipalalias=[service1],
+                    krbcanonicalname=[service1],
                     objectclass=objectclasses.service,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[fqdn1],
                     has_keytab=False,
                     ipakrbrequirespreauth=True,
                     ipakrbokasdelegate=False,
+                    ipakrboktoauthasdelegate=False,
+                    krbpwdpolicyreference=[DN(
+                        u'cn=Default Service Password Policy',
+                        api.env.container_service,
+                        api.env.basedn,
+                    )],
                 ),
             ),
         ),
 
 
+        dict(
+            desc='Search for %r with members' % service1,
+            command=('service_find', [service1], {'no_members': False}),
+            expected=dict(
+                count=1,
+                truncated=False,
+                summary=u'1 service matched',
+                result=[
+                    dict(
+                        dn=service1dn,
+                        krbprincipalname=[service1],
+                        krbcanonicalname=[service1],
+                        managedby_host=[fqdn1],
+                        has_keytab=False,
+                    ),
+                ],
+            ),
+        ),
+
+
         dict(
             desc='Search for %r' % service1,
             command=('service_find', [service1], {}),
@@ -262,7 +311,7 @@ class test_service(Declarative):
                     dict(
                         dn=service1dn,
                         krbprincipalname=[service1],
-                        managedby_host=[fqdn1],
+                        krbcanonicalname=[service1],
                         has_keytab=False,
                     ),
                 ],
@@ -282,12 +331,19 @@ class test_service(Declarative):
                         dn=service1dn,
                         krbprincipalname=[service1],
                         ipakrbprincipalalias=[service1],
+                        krbcanonicalname=[service1],
                         objectclass=objectclasses.service,
                         ipauniqueid=[fuzzy_uuid],
                         has_keytab=False,
                         managedby_host=[fqdn1],
                         ipakrbrequirespreauth=True,
                         ipakrbokasdelegate=False,
+                        ipakrboktoauthasdelegate=False,
+                        krbpwdpolicyreference=[DN(
+                            u'cn=Default Service Password Policy',
+                            api.env.container_service,
+                            api.env.basedn,
+                        )],
                     ),
                 ],
             ),
@@ -303,6 +359,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -318,6 +375,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -333,6 +391,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1, fqdn2],
                 ),
             ),
@@ -348,6 +407,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -363,6 +423,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1, fqdn3.lower()],
                 ),
             ),
@@ -378,6 +439,7 @@ class test_service(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -385,18 +447,31 @@ class test_service(Declarative):
 
 
         dict(
-            desc='Update %r with a bad certificate' % service1,
+            desc='Update %r with a random issuer certificate' % service1,
             command=(
                 'service_mod',
                 [service1],
-                dict(usercertificate=base64.b64decode(badservercert))
+                dict(usercertificate=base64.b64decode(randomissuercert))),
+            expected=dict(
+                value=service1,
+                summary=u'Modified service "%s"' % service1,
+                result=dict(
+                    usercertificate=[base64.b64decode(randomissuercert)],
+                    krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
+                    managedby_host=[fqdn1],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=randomissuer,
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    issuer=fuzzy_issuer,
                 ),
-            expected=errors.CertificateOperationError(
-                error=u'Issuer "CN=IPA Test Certificate Authority" does not ' +
-                    u'match the expected issuer'),
+            ),
         ),
 
-
         dict(
             desc='Update %r' % service1,
             command=('service_mod', [service1], dict(usercertificate=servercert)),
@@ -406,6 +481,7 @@ class test_service(Declarative):
                 result=dict(
                     usercertificate=[base64.b64decode(servercert)],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                     valid_not_before=fuzzy_date,
                     valid_not_after=fuzzy_date,
@@ -441,6 +517,7 @@ class test_service(Declarative):
                 result=dict(
                     usercertificate=[base64.b64decode(servercert)],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                     ipakrbauthzdata=[u'MS-PAC'],
                     valid_not_before=fuzzy_date,
@@ -466,6 +543,7 @@ class test_service(Declarative):
                     dn=service1dn,
                     usercertificate=[base64.b64decode(servercert)],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     has_keytab=False,
                     managedby_host=[fqdn1],
                     ipakrbauthzdata=[u'MS-PAC'],
@@ -493,6 +571,7 @@ class test_service(Declarative):
                 result=dict(
                     usercertificate=[base64.b64decode(servercert)],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                     ipakrbauthzdata=[u'MS-PAC'],
                     valid_not_before=fuzzy_date,
@@ -520,6 +599,7 @@ class test_service(Declarative):
                 result=dict(
                     usercertificate=[base64.b64decode(servercert)],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                     ipakrbauthzdata=[u'MS-PAC'],
                     valid_not_before=fuzzy_date,
@@ -545,6 +625,7 @@ class test_service(Declarative):
                 result=dict(
                     usercertificate=[base64.b64decode(servercert)],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                     ipakrbauthzdata=[u'MS-PAC'],
                     valid_not_before=fuzzy_date,
@@ -600,7 +681,9 @@ class test_service(Declarative):
         dict(
             desc='Create service with malformed principal "foo"',
             command=('service_add', [u'foo'], {}),
-            expected=errors.MalformedServicePrincipal(reason='missing service')
+            expected=errors.ValidationError(
+                name='canonical_principal',
+                error='Service principal is required')
         ),
 
 
@@ -678,6 +761,7 @@ class test_service_in_role(Declarative):
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
+                    krbcanonicalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[u'%s' % fqdn1],
@@ -696,6 +780,7 @@ class test_service_in_role(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     objectclass=objectclasses.service,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[fqdn1],
@@ -751,6 +836,7 @@ class test_service_in_role(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                     memberof_role=[role1],
                     has_keytab=False,
@@ -864,6 +950,7 @@ class test_service_allowed_to(Declarative):
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
+                    krbcanonicalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[u'%s' % fqdn1],
@@ -900,6 +987,7 @@ class test_service_allowed_to(Declarative):
                 result=dict(
                     dn=service1dn,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     objectclass=objectclasses.service,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[fqdn1],
@@ -926,6 +1014,7 @@ class test_service_allowed_to(Declarative):
                     dn=service1dn,
                     ipaallowedtoperform_read_keys_user=[user1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -949,6 +1038,7 @@ class test_service_allowed_to(Declarative):
                     dn=service1dn,
                     ipaallowedtoperform_read_keys_user=[user1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -977,6 +1067,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_read_keys_host=[fqdn1],
                     ipaallowedtoperform_read_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1003,6 +1094,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_read_keys_host=[fqdn1],
                     ipaallowedtoperform_read_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1029,6 +1121,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_read_keys_host=[fqdn1],
                     ipaallowedtoperform_read_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1061,6 +1154,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_write_keys_host=[fqdn1],
                     ipaallowedtoperform_write_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1092,6 +1186,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_write_keys_host=[fqdn1],
                     ipaallowedtoperform_write_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1122,6 +1217,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_write_keys_host=[fqdn1],
                     ipaallowedtoperform_write_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1152,6 +1248,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_write_keys_host=[fqdn1],
                     ipaallowedtoperform_write_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1175,6 +1272,7 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_write_keys_host=[fqdn1],
                     ipaallowedtoperform_write_keys_hostgroup=[hostgroup1],
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     managedby_host=[fqdn1],
                 ),
             ),
@@ -1199,9 +1297,49 @@ class test_service_allowed_to(Declarative):
                     ipaallowedtoperform_write_keys_hostgroup=[hostgroup1],
                     ipakrbokasdelegate=True,
                     krbprincipalname=[service1],
+                    krbcanonicalname=[service1],
                     krbticketflags=[u'1048704'],
                     managedby_host=[fqdn1],
                 ),
             ),
         ),
     ]
+
+
+@pytest.fixture(scope='function')
+def indicators_host(request):
+    tracker = HostTracker(name=u'testhost1', fqdn=fqdn1)
+    return tracker.make_fixture(request)
+
+
+@pytest.fixture(scope='function')
+def indicators_service(request):
+    tracker = ServiceTracker(
+        name=u'SRV1', host_fqdn=fqdn1, options={
+            u'krbprincipalauthind': u'otp'})
+    return tracker.make_fixture(request)
+
+
+@pytest.mark.tier1
+class TestAuthenticationIndicators(XMLRPC_test):
+    def test_create_service_with_otp_indicator(
+            self, indicators_host, indicators_service):
+        """ Since range of authentication indicator values is not limited,
+        only 'otp' option is tested """
+        indicators_host.create()
+        indicators_service.create()
+
+    def test_adding_second_indicator(
+            self, indicators_host, indicators_service):
+        indicators_host.create()
+        indicators_service.create()
+        indicators_service.update(
+            updates={u'krbprincipalauthind': [u'otp', u'radius']})
+
+    def test_update_indicator(self, indicators_host, indicators_service):
+        indicators_host.create()
+        indicators_service.create()
+        indicators_service.update(
+            updates={u'krbprincipalauthind': u'radius'},
+            expected_updates={u'krbprincipalauthind': [u'radius']}
+        )
