@@ -21,7 +21,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-Test the `ipalib/plugins/user.py` module.
+Test the `ipaserver/plugins/user.py` module.
 """
 
 import pytest
@@ -316,24 +316,10 @@ class TestUpdate(XMLRPC_test):
         renameduser.ensure_missing()
         olduid = user.uid
 
-        # using user.update(dict(uid=value)) results in
-        # OverlapError: overlapping arguments and options: ['uid']
-        user.attrs.update(uid=[renameduser.uid])
-        command = user.make_update_command(
-            updates=dict(setattr=(u'uid=%s' % renameduser.uid))
-        )
-        result = command()
-        user.check_update(result)
-        user.uid = renameduser.uid
+        user.update(updates=dict(rename=renameduser.uid))
 
         # rename the test user back so it gets properly deleted
-        user.attrs.update(uid=[olduid])
-        command = user.make_update_command(
-            updates=dict(setattr=(u'uid=%s' % olduid))
-        )
-        result = command()
-        user.check_update(result)
-        user.uid = olduid
+        user.update(updates=dict(rename=olduid))
 
     def test_rename_to_the_same_value(self, user):
         """ Try to rename user to the same value """
@@ -640,18 +626,13 @@ class TestUserWithGroup(XMLRPC_test):
             if its manager is also renamed """
         renamed_name = u'renamed_npg2'
         old_name = user_npg2.uid
-        command = user_npg2.make_update_command(dict(rename=renamed_name))
-        result = command()
-        user_npg2.attrs.update(uid=[renamed_name])
-        user_npg2.check_update(result)
+
+        user_npg2.update(updates=dict(rename=renamed_name))
+
         user_npg.attrs.update(manager=[renamed_name])
         user_npg.retrieve(all=True)
 
-        command = user_npg2.make_command(
-            'user_mod', renamed_name, **dict(rename=old_name)
-        )
-        # we rename the user back otherwise the tracker is too confused
-        result = command()
+        user_npg2.update(updates=dict(rename=old_name))
 
     def test_check_if_manager_gets_removed(self, user_npg, user_npg2):
         """ Delete manager and check if it's gone from user's attributes """
@@ -691,7 +672,7 @@ class TestManagers(XMLRPC_test):
         """ Find user by his manager's UID """
         command = user.make_find_command(manager=user2.uid)
         result = command()
-        user.check_find(result, expected_override=dict(manager=[user2.uid]))
+        user.check_find(result)
 
     def test_delete_both_user_and_manager(self, user, user2):
         """ Delete both user and its manager at once """
@@ -806,8 +787,9 @@ class TestPrincipals(XMLRPC_test):
         )
 
         command = testuser.make_create_command()
-        with raises_exact(errors.MalformedUserPrincipal(
-                principal=u'tuser1@BAD@NOTFOUND.ORG')):
+        with raises_exact(errors.ConversionError(
+                name='principal', error="Malformed principal: '{}'".format(
+                    testuser.kwargs['krbprincipalname']))):
             command()
 
     def test_set_principal_expiration(self, user):
@@ -954,6 +936,8 @@ def get_user_result(uid, givenname, sn, operation='show', omit=[],
         uid=[uid],
         uidnumber=[fuzzy_digits],
         gidnumber=[fuzzy_digits],
+        krbcanonicalname=[u'%s@%s' % (uid, api.env.realm)],
+        krbprincipalname=[u'%s@%s' % (uid, api.env.realm)],
         mail=[u'%s@%s' % (uid, api.env.domain)],
         has_keytab=False,
         has_password=False,
@@ -976,6 +960,7 @@ def get_user_result(uid, givenname, sn, operation='show', omit=[],
             mepmanagedentry=[get_group_dn(uid)],
             objectclass=add_oc(objectclasses.user, u'ipantuserattrs'),
             krbprincipalname=[u'%s@%s' % (uid, api.env.realm)],
+            krbcanonicalname=[u'%s@%s' % (uid, api.env.realm)]
         )
     if operation in ('show', 'show-all', 'find', 'mod'):
         result.update(
