@@ -7,13 +7,14 @@ from __future__ import print_function
 
 from ipapython.dn import DN
 from ipatests.test_xmlrpc.tracker.base import Tracker
+from ipatests.test_xmlrpc.tracker.kerberos_aliases import KerberosAliasMixin
 from ipatests.test_xmlrpc.xmlrpc_test import fuzzy_uuid
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.util import assert_deepequal
 from ipalib import errors
 
 
-class HostTracker(Tracker):
+class HostTracker(KerberosAliasMixin, Tracker):
     """Wraps and tracks modifications to a Host object
 
     Implements the helper functions for host plugin.
@@ -22,11 +23,13 @@ class HostTracker(Tracker):
     ``fqdn`` and ``dn``.
     """
     retrieve_keys = {
-        'dn', 'fqdn', 'description', 'l', 'krbprincipalname', 'managedby_host',
+        'dn', 'fqdn', 'description', 'l', 'krbcanonicalname',
+        'krbprincipalname', 'managedby_host',
         'has_keytab', 'has_password', 'issuer', 'md5_fingerprint',
         'serial_number', 'serial_number_hex', 'sha1_fingerprint',
         'subject', 'usercertificate', 'valid_not_after', 'valid_not_before',
         'macaddress', 'sshpubkeyfp', 'ipaallowedtoperform_read_keys_user',
+        'memberof_hostgroup', 'memberofindirect_hostgroup',
         'ipaallowedtoperform_read_keys_group',
         'ipaallowedtoperform_read_keys_host',
         'ipaallowedtoperform_read_keys_hostgroup',
@@ -36,12 +39,19 @@ class HostTracker(Tracker):
         'ipaallowedtoperform_write_keys_hostgroup'}
     retrieve_all_keys = retrieve_keys | {
         u'cn', u'ipakrbokasdelegate', u'ipakrbrequirespreauth', u'ipauniqueid',
-        u'managing_host', u'objectclass', u'serverhostname'}
+        u'krbcanonicalname', u'managing_host', u'objectclass',
+        u'serverhostname', u'ipakrboktoauthasdelegate',
+        u'krbpwdpolicyreference'}
     create_keys = retrieve_keys | {'objectclass', 'ipauniqueid',
                                    'randompassword'}
     update_keys = retrieve_keys - {'dn'}
     managedby_keys = retrieve_keys - {'has_keytab', 'has_password'}
     allowedto_keys = retrieve_keys - {'has_keytab', 'has_password'}
+    find_keys = retrieve_keys - {
+        'has_keytab', 'has_password', 'memberof_hostgroup',
+        'memberofindirect_hostgroup', 'managedby_host',
+    }
+    find_all_keys = retrieve_all_keys - {'has_keytab', 'has_password'}
 
     def __init__(self, name, fqdn=None, default_version=None):
         super(HostTracker, self).__init__(default_version=default_version)
@@ -92,6 +102,7 @@ class HostTracker(Tracker):
             description=[self.description],
             l=[self.location],
             krbprincipalname=[u'host/%s@%s' % (self.fqdn, self.api.env.realm)],
+            krbcanonicalname=[u'host/%s@%s' % (self.fqdn, self.api.env.realm)],
             objectclass=objectclasses.host,
             ipauniqueid=[fuzzy_uuid],
             managedby_host=[self.fqdn],
@@ -102,6 +113,12 @@ class HostTracker(Tracker):
             ipakrbrequirespreauth=True,
             managing_host=[self.fqdn],
             serverhostname=[self.shortname],
+            ipakrboktoauthasdelegate=False,
+            krbpwdpolicyreference=[DN(
+                u'cn=Default Host Password Policy',
+                self.api.env.container_host,
+                self.api.env.basedn,
+            )],
         )
         self.exists = True
 
@@ -136,9 +153,9 @@ class HostTracker(Tracker):
     def check_find(self, result, all=False, raw=False):
         """Check `host_find` command result"""
         if all:
-            expected = self.filter_attrs(self.retrieve_all_keys)
+            expected = self.filter_attrs(self.find_all_keys)
         else:
-            expected = self.filter_attrs(self.retrieve_keys)
+            expected = self.filter_attrs(self.find_keys)
         assert_deepequal(dict(
             count=1,
             truncated=False,
@@ -166,3 +183,10 @@ class HostTracker(Tracker):
                 pass
 
         request.addfinalizer(cleanup)
+
+    #  Kerberos aliases methods
+    def _make_add_alias_cmd(self):
+        return self.make_command('host_add_principal', self.name)
+
+    def _make_remove_alias_cmd(self):
+        return self.make_command('host_remove_principal', self.name)
