@@ -69,7 +69,7 @@ fake_api_env = {'env': [
 fake_api = {'api': [fake_api_env] + NAMESPACE_ATTRS}
 
 _LOGGING_ATTRS = ['debug', 'info', 'warning', 'error', 'exception',
-                  'critical', 'warn']
+                  'critical']
 LOGGING_ATTRS = [
     {'log': _LOGGING_ATTRS},
 ] + _LOGGING_ATTRS
@@ -114,6 +114,7 @@ ipa_class_members = {
     'ipalib.errors.PublicError': [
         'msg',
         'strerror',
+        'kw',
     ],
     'ipalib.errors.SingleMatchExpected': [
         'found',
@@ -124,10 +125,15 @@ ipa_class_members = {
     'ipalib.errors.ValidationError': [
         'error',
     ],
+    'ipalib.errors.SchemaUpToDate': [
+        'fingerprint',
+        'ttl',
+    ],
     'ipalib.messages.PublicMessage': [
         'msg',
         'strerror',
         'type',
+        'kw',
     ],
     'ipalib.parameters.Param': [
         'cli_name',
@@ -149,8 +155,9 @@ ipa_class_members = {
         'hint',
         'alwaysask',
         'sortorder',
-        'csv',
         'option_group',
+        'no_convert',
+        'deprecated',
      ],
     'ipalib.parameters.Bool': [
         'truths',
@@ -165,10 +172,6 @@ ipa_class_members = {
     'ipalib.parameters.Str': ['noextrawhitespace'],
     'ipalib.parameters.Password': ['confirm'],
     'ipalib.parameters.File': ['stdin_if_missing'],
-    'ipalib.plugins.dns.DNSRecord': [
-        'validatedns',
-        'normalizedns',
-    ],
     'ipalib.parameters.Enum': ['values'],
     'ipalib.parameters.Number': [
         'minvalue',
@@ -183,6 +186,9 @@ ipa_class_members = {
         'only_absolute',
         'only_relative',
     ],
+    'ipalib.parameters.Principal': [
+        'require_service',
+    ],
     'ipalib.plugable.API': [
         fake_api_env,
     ] + NAMESPACE_ATTRS + LOGGING_ATTRS,
@@ -192,16 +198,20 @@ ipa_class_members = {
         'Updater',
         'Advice',
     ] + LOGGING_ATTRS,
-    'ipalib.session.AuthManager': LOGGING_ATTRS,
-    'ipalib.session.SessionAuthManager': LOGGING_ATTRS,
-    'ipalib.session.SessionManager': LOGGING_ATTRS,
     'ipalib.util.ForwarderValidationError': [
         'msg',
     ],
     'ipaserver.install.ldapupdate.LDAPUpdate': LOGGING_ATTRS,
+    'ipaserver.plugins.dns.DNSRecord': [
+        'validatedns',
+        'normalizedns',
+    ],
     'ipaserver.rpcserver.KerberosSession': [
         fake_api,
     ] + LOGGING_ATTRS,
+    'ipaserver.session.AuthManager': LOGGING_ATTRS,
+    'ipaserver.session.SessionAuthManager': LOGGING_ATTRS,
+    'ipaserver.session.SessionManager': LOGGING_ATTRS,
     'ipatests.test_integration.base.IntegrationTest': [
         'domain',
         {'master': [
@@ -227,7 +237,7 @@ ipa_class_members = {
                 'stderr_text',
                 'returncode',
             ]},
-            {'transport': ['put_file']},
+            {'transport': ['put_file', 'file_exists']},
             'put_file_contents',
             'get_file_contents',
             'ldap_connect',
